@@ -32,6 +32,7 @@
 #include "protocol-common.h"
 #include "glusterfs3.h"
 #include "portmap-xdr.h"
+#include "xdr-common.h"
 #include "xdr-generic.h"
 
 #include "syncop.h"
@@ -39,6 +40,7 @@
 
 #include "glfs-internal.h"
 #include "glfs-mem-types.h"
+#include "gfapi-messages.h"
 
 int glfs_volfile_fetch (struct glfs *fs);
 int32_t glfs_get_volume_info_rpc (call_frame_t *frame, xlator_t *this,
@@ -55,13 +57,16 @@ glfs_process_volfp (struct glfs *fs, FILE *fp)
 	ctx = fs->ctx;
 	graph = glusterfs_graph_construct (fp);
 	if (!graph) {
-		gf_log ("glfs", GF_LOG_ERROR, "failed to construct the graph");
+		gf_msg ("glfs", GF_LOG_ERROR, errno,
+                        API_MSG_GRAPH_CONSTRUCT_FAILED,
+                        "failed to construct the graph");
 		goto out;
 	}
 
 	for (trav = graph->first; trav; trav = trav->next) {
 		if (strcmp (trav->type, "mount/fuse") == 0) {
-			gf_log ("glfs", GF_LOG_ERROR,
+			gf_msg ("glfs", GF_LOG_ERROR, EINVAL,
+                                API_MSG_FUSE_XLATOR_ERROR,
 				"fuse xlator cannot be specified "
 				"in volume file");
 			goto out;
@@ -182,7 +187,8 @@ mgmt_submit_request (void *req, call_frame_t *frame,
 		/* Create the xdr payload */
 		ret = xdr_serialize_generic (iov, req, xdrproc);
 		if (ret == -1) {
-			gf_log (THIS->name, GF_LOG_WARNING,
+			gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                                API_MSG_XDR_PAYLOAD_FAILED,
 				"failed to create XDR payload");
 			goto out;
 		}
@@ -226,7 +232,8 @@ mgmt_get_volinfo_cbk (struct rpc_req *req, struct iovec *iov,
         args = frame->local;
 
         if (!ctx) {
-                gf_log (frame->this->name, GF_LOG_ERROR, "NULL context");
+                gf_msg (frame->this->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_INVALID_ENTRY, "NULL context");
                 errno = EINVAL;
                 ret = -1;
                 goto out;
@@ -235,7 +242,8 @@ mgmt_get_volinfo_cbk (struct rpc_req *req, struct iovec *iov,
         fs = ((xlator_t *)ctx->master)->private;
 
         if (-1 == req->rpc_status) {
-                gf_log (frame->this->name, GF_LOG_ERROR,
+                gf_msg (frame->this->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_INVALID_ENTRY,
                         "GET_VOLUME_INFO RPC call is not successfull");
                 errno = EINVAL;
                 ret = -1;
@@ -245,13 +253,14 @@ mgmt_get_volinfo_cbk (struct rpc_req *req, struct iovec *iov,
         ret = xdr_to_generic (*iov, &rsp, (xdrproc_t)xdr_gf_get_volume_info_rsp);
 
         if (ret < 0) {
-                gf_log (frame->this->name, GF_LOG_ERROR,
+                gf_msg (frame->this->name, GF_LOG_ERROR, 0,
+                        API_MSG_XDR_RESPONSE_DECODE_FAILED,
                         "Failed to decode xdr response for GET_VOLUME_INFO");
                 goto out;
         }
 
-        gf_log (frame->this->name, GF_LOG_DEBUG,
-                "Received resp to GET_VOLUME_INFO RPC: %d", rsp.op_ret);
+        gf_msg_debug (frame->this->name, 0, "Received resp to GET_VOLUME_INFO "
+                      "RPC: %d", rsp.op_ret);
 
         if (rsp.op_ret == -1) {
                 errno = rsp.op_errno;
@@ -260,8 +269,9 @@ mgmt_get_volinfo_cbk (struct rpc_req *req, struct iovec *iov,
         }
 
         if (!rsp.dict.dict_len) {
-                gf_log (frame->this->name, GF_LOG_ERROR,
-                        "Response received for GET_VOLUME_INFO RPC call is not valid");
+                gf_msg (frame->this->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_INVALID_ENTRY, "Response received for "
+                        "GET_VOLUME_INFO RPC call is not valid");
                 ret = -1;
                 errno = EINVAL;
                 goto out;
@@ -294,17 +304,17 @@ mgmt_get_volinfo_cbk (struct rpc_req *req, struct iovec *iov,
         ret = 0;
 out:
         if (volume_id_str) {
-                gf_log (frame->this->name, GF_LOG_DEBUG,
-                        "Volume Id: %s", volume_id_str);
+                gf_msg_debug (frame->this->name, 0,
+                              "Volume Id: %s", volume_id_str);
                 pthread_mutex_lock (&fs->mutex);
-                uuid_parse (volume_id_str, fs->vol_uuid);
+                gf_uuid_parse (volume_id_str, fs->vol_uuid);
                 pthread_mutex_unlock (&fs->mutex);
         }
 
         if (ret) {
-               gf_log (frame->this->name, GF_LOG_ERROR,
-                       "In GET_VOLUME_INFO cbk, received error: %s",
-                       strerror(errno));
+                gf_msg (frame->this->name, GF_LOG_ERROR, errno,
+                        API_MSG_GET_VOLINFO_CBK_FAILED, "In GET_VOLUME_INFO "
+                        "cbk, received error: %s", strerror(errno));
         }
 
         if (dict)
@@ -316,7 +326,7 @@ out:
         if (rsp.op_errstr && *rsp.op_errstr)
                 free (rsp.op_errstr);
 
-        gf_log (frame->this->name, GF_LOG_DEBUG, "Returning: %d", ret);
+        gf_msg_debug (frame->this->name, 0, "Returning: %d", ret);
 
         __wake (args);
 
@@ -324,15 +334,18 @@ out:
 }
 
 int
-glfs_get_volumeid (struct glfs *fs, char *volid, size_t size)
+pub_glfs_get_volumeid (struct glfs *fs, char *volid, size_t size)
 {
         /* TODO: Define a global macro to store UUID size */
         size_t uuid_size = 16;
 
+        DECLARE_OLD_THIS;
+        __GLFS_ENTRY_VALIDATE_FS (fs, invalid_fs);
+
         pthread_mutex_lock (&fs->mutex);
         {
                 /* check if the volume uuid is initialized */
-                if (!uuid_is_null (fs->vol_uuid)) {
+                if (!gf_uuid_is_null (fs->vol_uuid)) {
                         pthread_mutex_unlock (&fs->mutex);
                         goto done;
                 }
@@ -342,30 +355,42 @@ glfs_get_volumeid (struct glfs *fs, char *volid, size_t size)
         /* Need to fetch volume_uuid */
         glfs_get_volume_info (fs);
 
-        if (uuid_is_null (fs->vol_uuid)) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Unable to fetch volume UUID");
-                return -1;
+        if (gf_uuid_is_null (fs->vol_uuid)) {
+                gf_msg (THIS->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_FETCH_VOLUUID_FAILED, "Unable to fetch "
+                        "volume UUID");
+                goto out;
         }
 
 done:
         if (!volid || !size) {
-                gf_log (THIS->name, GF_LOG_DEBUG, "volumeid/size is null");
+                gf_msg_debug (THIS->name, 0, "volumeid/size is null");
+                __GLFS_EXIT_FS;
                 return uuid_size;
         }
 
         if (size < uuid_size) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Insufficient size passed");
+                gf_msg (THIS->name, GF_LOG_ERROR, ERANGE, API_MSG_INSUFF_SIZE,
+                        "Insufficient size passed");
                 errno = ERANGE;
-                return -1;
+                goto out;
         }
 
         memcpy (volid, fs->vol_uuid, uuid_size);
 
-        gf_log (THIS->name, GF_LOG_INFO, "volume uuid: %s", volid);
+        __GLFS_EXIT_FS;
 
         return uuid_size;
+
+out:
+        __GLFS_EXIT_FS;
+
+invalid_fs:
+        return -1;
 }
 
+GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_get_volumeid, 3.5.0);
+
 int
 glfs_get_volume_info (struct glfs *fs)
 {
@@ -376,6 +401,14 @@ glfs_get_volume_info (struct glfs *fs)
 
         ctx = fs->ctx;
         frame = create_frame (THIS, ctx->pool);
+        if (!frame) {
+                gf_msg ("glfs", GF_LOG_ERROR, ENOMEM,
+                        API_MSG_FRAME_CREAT_FAILED,
+                        "failed to create the frame");
+                ret = -1;
+                goto out;
+        }
+
         frame->local = &args;
 
         __yawn ((&args));
@@ -426,7 +459,8 @@ glfs_get_volume_info_rpc (call_frame_t *frame, xlator_t *this,
         flags = (int32_t)GF_GET_VOLUME_UUID; //ctx->flags;
         ret = dict_set_int32 (dict, "flags", flags);
         if (ret) {
-                gf_log (frame->this->name, GF_LOG_ERROR, "failed to set flags");
+                gf_msg (frame->this->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_DICT_SET_FAILED, "failed to set flags");
                 goto out;
         }
 
@@ -453,6 +487,8 @@ glusterfs_oldvolfile_update (struct glfs *fs, char *volfile, ssize_t size)
 {
 	int ret = -1;
 
+        pthread_mutex_lock (&fs->mutex);
+
 	fs->oldvollen = size;
 	if (!fs->oldvolfile) {
 		fs->oldvolfile = GF_CALLOC (1, size+1, glfs_mt_volfile_t);
@@ -468,13 +504,15 @@ glusterfs_oldvolfile_update (struct glfs *fs, char *volfile, ssize_t size)
 		ret = 0;
 	}
 
+        pthread_mutex_unlock (&fs->mutex);
+
 	return ret;
 }
 
 
 int
-mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
-		  void *myframe)
+glfs_mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
+                       void *myframe)
 {
 	gf_getspec_rsp		 rsp   = {0,};
 	call_frame_t		*frame = NULL;
@@ -482,11 +520,20 @@ mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
 	int			 ret   = 0;
 	ssize_t			 size = 0;
 	FILE			*tmpfp = NULL;
-	int                      need_retry = 0;
+	int			 need_retry = 0;
 	struct glfs		*fs = NULL;
 
 	frame = myframe;
 	ctx = frame->this->ctx;
+
+	if (!ctx) {
+		gf_msg (frame->this->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_INVALID_ENTRY, "NULL context");
+		errno = EINVAL;
+		ret = -1;
+		goto out;
+	}
+
 	fs = ((xlator_t *)ctx->master)->private;
 
 	if (-1 == req->rpc_status) {
@@ -497,16 +544,18 @@ mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
 
 	ret = xdr_to_generic (*iov, &rsp, (xdrproc_t)xdr_gf_getspec_rsp);
 	if (ret < 0) {
-		gf_log (frame->this->name, GF_LOG_ERROR, "XDR decoding error");
+		gf_msg (frame->this->name, GF_LOG_ERROR, 0,
+                        API_MSG_XDR_DECODE_FAILED, "XDR decoding error");
 		ret   = -1;
 		goto out;
 	}
 
 	if (-1 == rsp.op_ret) {
-		gf_log (frame->this->name, GF_LOG_ERROR,
+		gf_msg (frame->this->name, GF_LOG_ERROR, rsp.op_errno,
+                        API_MSG_GET_VOLFILE_FAILED,
 			"failed to get the 'volume file' from server");
 		ret = -1;
-                errno = rsp.op_errno;
+		errno = rsp.op_errno;
 		goto out;
 	}
 
@@ -515,7 +564,8 @@ mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
 
 	if ((size == fs->oldvollen) &&
 	    (memcmp (fs->oldvolfile, rsp.spec, size) == 0)) {
-		gf_log (frame->this->name, GF_LOG_INFO,
+		gf_msg (frame->this->name, GF_LOG_INFO, 0,
+                        API_MSG_VOLFILE_INFO,
 			"No change in volfile, continuing");
 		goto out;
 	}
@@ -542,17 +592,16 @@ mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
 	*/
 
 	ret = glusterfs_volfile_reconfigure (fs->oldvollen, tmpfp, fs->ctx,
-                                             fs->oldvolfile);
+					     fs->oldvolfile);
 	if (ret == 0) {
-		gf_log ("glusterfsd-mgmt", GF_LOG_DEBUG,
-			"No need to re-load volfile, reconfigure done");
+		gf_msg_debug ("glusterfsd-mgmt", 0, "No need to re-load "
+                              "volfile, reconfigure done");
 		ret = glusterfs_oldvolfile_update (fs, rsp.spec, size);
 		goto out;
 	}
 
 	if (ret < 0) {
-		gf_log ("glusterfsd-mgmt", GF_LOG_DEBUG,
-			"Reconfigure failed !!");
+		gf_msg_debug ("glusterfsd-mgmt", 0, "Reconfigure failed !!");
 		goto out;
 	}
 
@@ -569,26 +618,28 @@ out:
 	if (rsp.spec)
 		free (rsp.spec);
 
-        // Stop if server is running at an unsupported op-version
-        if (ENOTSUP == ret) {
-                gf_log ("mgmt", GF_LOG_ERROR, "Server is operating at an "
-                        "op-version which is not supported");
-                errno = ENOTSUP;
-                glfs_init_done (fs, -1);
-        }
+	// Stop if server is running at an unsupported op-version
+	if (ENOTSUP == ret) {
+		gf_msg ("mgmt", GF_LOG_ERROR, ENOTSUP, API_MSG_WRONG_OPVERSION,
+                        "Server is operating at an op-version which is not "
+                        "supported");
+		errno = ENOTSUP;
+		glfs_init_done (fs, -1);
+	}
 
 	if (ret && ctx && !ctx->active) {
 		/* Do it only for the first time */
 		/* Failed to get the volume file, something wrong,
 		   restart the process */
-		gf_log ("glfs-mgmt", GF_LOG_ERROR,
+		gf_msg ("glfs-mgmt", GF_LOG_ERROR, EINVAL,
+                        API_MSG_INVALID_ENTRY,
 			"failed to fetch volume file (key:%s)",
 			ctx->cmd_args.volfile_id);
 		if (!need_retry) {
-                        if (!errno)
-                                errno = EINVAL;
+			if (!errno)
+				errno = EINVAL;
 			glfs_init_done (fs, -1);
-                }
+		}
 	}
 
 	if (tmpfp)
@@ -626,28 +677,31 @@ glfs_volfile_fetch (struct glfs *fs)
         // decision
         ret = dict_set_int32 (dict, "min-op-version", GD_OP_VERSION_MIN);
         if (ret) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Failed to set min-op-version"
-                        " in request dict");
+                gf_msg (THIS->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_DICT_SET_FAILED,
+                        "Failed to set min-op-version in request dict");
                 goto out;
         }
 
         ret = dict_set_int32 (dict, "max-op-version", GD_OP_VERSION_MAX);
         if (ret) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Failed to set max-op-version"
-                        " in request dict");
+                gf_msg (THIS->name, GF_LOG_ERROR, EINVAL,
+                        API_MSG_DICT_SET_FAILED,
+                        "Failed to set max-op-version in request dict");
                 goto out;
         }
 
         ret = dict_allocate_and_serialize (dict, &req.xdata.xdata_val,
                                            &req.xdata.xdata_len);
         if (ret < 0) {
-                gf_log (THIS->name, GF_LOG_ERROR,
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                        API_MSG_DICT_SERIALIZE_FAILED,
                         "Failed to serialize dictionary");
                 goto out;
         }
 
 	ret = mgmt_submit_request (&req, frame, ctx, &clnt_handshake_prog,
-				   GF_HNDSK_GETSPEC, mgmt_getspec_cbk,
+				   GF_HNDSK_GETSPEC, glfs_mgmt_getspec_cbk,
 				   (xdrproc_t)xdr_gf_getspec_req);
 out:
         return ret;
@@ -660,29 +714,34 @@ mgmt_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
 {
 	xlator_t	*this = NULL;
 	glusterfs_ctx_t *ctx = NULL;
-	struct glfs	 *fs = NULL;
-	int		 ret = 0;
         server_cmdline_t *server = NULL;
         rpc_transport_t  *rpc_trans = NULL;
+	struct glfs	 *fs = NULL;
+	int		 ret = 0;
 
-        this = mydata;
+	this = mydata;
         rpc_trans = rpc->conn.trans;
 
-        ctx = this->ctx;
-        fs = ((xlator_t *)ctx->master)->private;
+	ctx = this->ctx;
+	if (!ctx)
+		goto out;
 
-        switch (event) {
-        case RPC_CLNT_DISCONNECT:
-                if (!ctx->active) {
-                        gf_log ("glfs-mgmt", GF_LOG_ERROR,
+	fs = ((xlator_t *)ctx->master)->private;
+
+	switch (event) {
+	case RPC_CLNT_DISCONNECT:
+		if (!ctx->active) {
+                        gf_msg ("glfs-mgmt", GF_LOG_ERROR, errno,
+                                API_MSG_REMOTE_HOST_CONN_FAILED,
                                 "failed to connect with remote-host: %s (%s)",
                                 ctx->cmd_args.volfile_server,
                                 strerror (errno));
                         server = ctx->cmd_args.curr_server;
                         if (server->list.next == &ctx->cmd_args.volfile_servers) {
                                 errno = ENOTCONN;
-                                gf_log("glfs-mgmt", GF_LOG_INFO,
-                                       "Exhausted all volfile servers");
+                                gf_msg ("glfs-mgmt", GF_LOG_INFO, ENOTCONN,
+                                        API_MSG_VOLFILE_SERVER_EXHAUST,
+                                        "Exhausted all volfile servers");
                                 glfs_init_done (fs, -1);
                                 break;
                         }
@@ -692,11 +751,13 @@ mgmt_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                         ctx->cmd_args.volfile_server_port = server->port;
                         ctx->cmd_args.volfile_server = server->volfile_server;
                         ctx->cmd_args.volfile_server_transport = server->transport;
+
                         ret = dict_set_int32 (rpc_trans->options,
                                               "remote-port",
                                               server->port);
                         if (ret != 0) {
-                                gf_log ("glfs-mgmt", GF_LOG_ERROR,
+                                gf_msg ("glfs-mgmt", GF_LOG_ERROR, ENOTCONN,
+                                        API_MSG_DICT_SET_FAILED,
                                         "failed to set remote-port: %d",
                                         server->port);
                                 errno = ENOTCONN;
@@ -708,7 +769,8 @@ mgmt_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                                             "remote-host",
                                             server->volfile_server);
                         if (ret != 0) {
-                                gf_log ("glfs-mgmt", GF_LOG_ERROR,
+                                gf_msg ("glfs-mgmt", GF_LOG_ERROR, ENOTCONN,
+                                        API_MSG_DICT_SET_FAILED,
                                         "failed to set remote-host: %s",
                                         server->volfile_server);
                                 errno = ENOTCONN;
@@ -720,14 +782,16 @@ mgmt_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                                             "transport-type",
                                             server->transport);
                         if (ret != 0) {
-                                gf_log ("glfs-mgmt", GF_LOG_ERROR,
+                                gf_msg ("glfs-mgmt", GF_LOG_ERROR, ENOTCONN,
+                                        API_MSG_DICT_SET_FAILED,
                                         "failed to set transport-type: %s",
                                         server->transport);
                                 errno = ENOTCONN;
                                 glfs_init_done (fs, -1);
                                 break;
                         }
-                        gf_log ("glfs-mgmt", GF_LOG_INFO,
+                        gf_msg ("glfs-mgmt", GF_LOG_INFO, 0,
+                                API_MSG_VOLFILE_CONNECTING,
                                 "connecting to next volfile server %s"
                                 " at port %d with transport: %s",
                                 server->volfile_server, server->port,
@@ -738,21 +802,22 @@ mgmt_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
 		rpc_clnt_set_connected (&((struct rpc_clnt*)ctx->mgmt)->conn);
 
 		ret = glfs_volfile_fetch (fs);
-		if (ret && ctx && (ctx->active == NULL)) {
+		if (ret && (ctx->active == NULL)) {
 			/* Do it only for the first time */
 			/* Exit the process.. there are some wrong options */
-			gf_log ("glfs-mgmt", GF_LOG_ERROR,
+			gf_msg ("glfs-mgmt", GF_LOG_ERROR, EINVAL,
+                                API_MSG_INVALID_ENTRY,
 				"failed to fetch volume file (key:%s)",
-				ctx->cmd_args.volfile_id);
+                                ctx->cmd_args.volfile_id);
                         errno = EINVAL;
-			glfs_init_done (fs, -1);
-		}
+                        glfs_init_done (fs, -1);
+                }
 
                 break;
 	default:
 		break;
 	}
-
+out:
 	return 0;
 }
 
@@ -803,24 +868,27 @@ glfs_mgmt_init (struct glfs *fs)
 	if (ret)
 		goto out;
 
-	rpc = rpc_clnt_new (options, THIS->ctx, THIS->name, 8);
+	rpc = rpc_clnt_new (options, ctx, THIS->name, 8);
 	if (!rpc) {
 		ret = -1;
-		gf_log (THIS->name, GF_LOG_WARNING,
+		gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                        API_MSG_CREATE_RPC_CLIENT_FAILED,
 			"failed to create rpc clnt");
 		goto out;
 	}
 
 	ret = rpc_clnt_register_notify (rpc, mgmt_rpc_notify, THIS);
 	if (ret) {
-		gf_log (THIS->name, GF_LOG_WARNING,
+		gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                        API_MSG_REG_NOTIFY_FUNC_FAILED,
 			"failed to register notify function");
 		goto out;
 	}
 
 	ret = rpcclnt_cbk_program_register (rpc, &mgmt_cbk_prog, THIS);
 	if (ret) {
-		gf_log (THIS->name, GF_LOG_WARNING,
+		gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                        API_MSG_REG_CBK_FUNC_FAILED,
 			"failed to register callback function");
 		goto out;
 	}
