@@ -97,8 +97,12 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         cmd_args_t    *cmd_args = NULL;
         struct rlimit  lim = {0, };
         call_pool_t   *pool = NULL;
+        int            ret         = -1;
 
-        xlator_mem_acct_init (THIS, cli_mt_end);
+        ret = xlator_mem_acct_init (THIS, cli_mt_end);
+        if (ret != 0) {
+                return ret;
+        }
 
         ctx->process_uuid = generate_glusterfs_ctx_id ();
         if (!ctx->process_uuid)
@@ -110,7 +114,8 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         if (!ctx->iobuf_pool)
                 return -1;
 
-        ctx->event_pool = event_pool_new (DEFAULT_EVENT_POOL_SIZE);
+        ctx->event_pool = event_pool_new (DEFAULT_EVENT_POOL_SIZE,
+                                          STARTING_EVENT_THREADS);
         if (!ctx->event_pool)
                 return -1;
 
@@ -146,6 +151,10 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         if (!ctx->dict_data_pool)
                 return -1;
 
+        ctx->logbuf_pool = mem_pool_new (log_buf_t, 256);
+        if (!ctx->logbuf_pool)
+                return -1;
+
         INIT_LIST_HEAD (&pool->all_frames);
         LOCK_INIT (&pool->lock);
         ctx->pool = pool;
@@ -179,7 +188,7 @@ logging_init (glusterfs_ctx_t *ctx, struct cli_state *state)
 
         /* CLI should not have something to DEBUG after the release,
            hence defaulting to INFO loglevel */
-        gf_log_set_loglevel ((state->log_level == -1) ? GF_LOG_INFO :
+        gf_log_set_loglevel ((state->log_level == GF_LOG_NONE) ? GF_LOG_INFO :
                              state->log_level);
 
         return 0;
@@ -297,7 +306,8 @@ cli_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
 int
 cli_opt_parse (char *opt, struct cli_state *state)
 {
-        char *oarg;
+        char            *oarg           = NULL;
+        gf_boolean_t    secure_mgmt_tmp = 0;
 
         if (strcmp (opt, "") == 0)
                 return 1;
@@ -326,14 +336,21 @@ cli_opt_parse (char *opt, struct cli_state *state)
                 return 0;
         }
 
+        if (strcmp (opt, "wignore") == 0) {
+                state->mode |= GLUSTER_MODE_WIGNORE;
+                return 0;
+        }
+
         oarg = strtail (opt, "mode=");
         if (oarg) {
                 if (strcmp (oarg, "script") == 0) {
                         state->mode |= GLUSTER_MODE_SCRIPT;
                         return 0;
                 }
+
                 if (strcmp (oarg, "interactive") == 0)
                         return 0;
+
                 return -1;
         }
 
@@ -363,23 +380,48 @@ cli_opt_parse (char *opt, struct cli_state *state)
                 return 0;
         }
 
+        oarg = strtail (opt, "secure-mgmt=");
+        if (oarg) {
+                if (gf_string2boolean(oarg,&secure_mgmt_tmp) == 0) {
+                        if (secure_mgmt_tmp) {
+                                /* See declaration for why this is an int. */
+                                state->ctx->secure_mgmt = 1;
+                        }
+                }
+                else {
+                        cli_err ("invalide secure-mgmt value (ignored)");
+                }
+                return 0;
+        }
+
         return -1;
 }
 
 int
 parse_cmdline (int argc, char *argv[], struct cli_state *state)
 {
-        int         ret = 0;
-        int         i = 0;
-        int         j = 0;
-        char        *opt = NULL;
+        int                 ret            = 0;
+        int                 i              = 0;
+        int                 j              = 0;
+        char               *opt            = NULL;
+        gf_boolean_t       geo_rep_config  = _gf_false;
 
         state->argc=argc-1;
         state->argv=&argv[1];
 
+        /* Do this first so that an option can override. */
+        if (access(SECURE_ACCESS_FILE,F_OK) == 0) {
+                state->ctx->secure_mgmt = 1;
+        }
+
+        if (state->argc > GEO_REP_CMD_CONFIG_INDEX &&
+            strtail (state->argv[GEO_REP_CMD_INDEX], "geo") &&
+            strtail (state->argv[GEO_REP_CMD_CONFIG_INDEX], "co"))
+                geo_rep_config = _gf_true;
+
         for (i = 0; i < state->argc; i++) {
                 opt = strtail (state->argv[i], "--");
-                if (opt) {
+                if (opt && !geo_rep_config) {
                         ret = cli_opt_parse (opt, state);
                         if (ret == -1) {
                                 cli_out ("unrecognized option --%s", opt);
@@ -424,7 +466,7 @@ cli_state_init (struct cli_state *state)
         int                   ret = 0;
 
 
-        state->log_level = -1;
+        state->log_level = GF_LOG_NONE;
 
         tree = &state->tree;
         tree->state = state;
@@ -516,7 +558,7 @@ cli_quotad_clnt_rpc_init (void)
                 goto out;
 
         ret = dict_set_str (rpc_opts, "transport.socket.connect-path",
-                                            "/tmp/quotad.socket");
+                                            "/var/run/gluster/quotad.socket");
         if (ret)
                 goto out;
 
@@ -526,7 +568,10 @@ cli_quotad_clnt_rpc_init (void)
 
         global_quotad_rpc = rpc;
 out:
-        dict_unref (rpc_opts);
+        if (ret) {
+                if (rpc_opts)
+                        dict_destroy(rpc_opts);
+        }
         return rpc;
 }
 
@@ -539,16 +584,15 @@ cli_rpc_init (struct cli_state *state)
         int                     port = CLI_GLUSTERD_PORT;
         xlator_t                *this = NULL;
 
-
         this = THIS;
         cli_rpc_prog = &cli_prog;
         options = dict_new ();
         if (!options)
                 goto out;
 
-        /* Connect using to glusterd using the specified method, giving
-         * preference to unix socket connection. If nothing is specified connect
-         * to the default glusterd socket
+        /* Connect to glusterd using the specified method, giving preference
+         * to a unix socket connection.  If nothing is specified, connect to
+         * the default glusterd socket.
          */
         if (state->glusterd_sock) {
                 gf_log ("cli", GF_LOG_INFO, "Connecting to glusterd using "
@@ -558,7 +602,8 @@ cli_rpc_init (struct cli_state *state)
                                                         0);
                 if (ret)
                         goto out;
-        } else if (state->remote_host) {
+        }
+        else if (state->remote_host) {
                 gf_log ("cli", GF_LOG_INFO, "Connecting to remote glusterd at "
                         "%s", state->remote_host);
                 ret = dict_set_str (options, "remote-host", state->remote_host);
@@ -576,7 +621,8 @@ cli_rpc_init (struct cli_state *state)
                                     "inet");
                 if (ret)
                         goto out;
-        } else {
+        }
+        else {
                 gf_log ("cli", GF_LOG_DEBUG, "Connecting to glusterd using "
                         "default socket");
                 ret = rpc_transport_unix_options_build
@@ -670,6 +716,9 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
 
+        gf_log ("cli", GF_LOG_INFO, "Started running %s with version %s",
+                argv[0], PACKAGE_VERSION);
+
         global_rpc = cli_rpc_init (&state);
         if (!global_rpc)
                 goto out;
@@ -708,3 +757,35 @@ cli_print_line (int len)
 
         printf ("\n");
 }
+
+void
+print_quota_list_header (int type)
+{
+        if (type == GF_QUOTA_OPTION_TYPE_LIST) {
+                cli_out ("                  Path                   Hard-limit "
+                         "Soft-limit   Used  Available  Soft-limit exceeded?"
+                         " Hard-limit exceeded?");
+                cli_out ("-----------------------------------------------------"
+                         "-----------------------------------------------------"
+                         "-----------------");
+        } else {
+                cli_out ("                  Path                   Hard-limit  "
+                         "Soft-limit     Files       Dirs     Available  "
+                         "Soft-limit exceeded? Hard-limit exceeded?");
+                cli_out ("-----------------------------------------------------"
+                         "-----------------------------------------------------"
+                         "-----------------------------------");
+        }
+}
+
+void
+print_quota_list_empty (char *path, int type)
+{
+        if (type == GF_QUOTA_OPTION_TYPE_LIST)
+                cli_out ("%-40s %7s %9s %10s %7s %15s %20s", path,
+                         "N/A", "N/A", "N/A", "N/A", "N/A", "N/A");
+        else
+                cli_out ("%-40s %9s %9s %12s %10s %10s %15s %20s", path,
+                         "N/A", "N/A", "N/A", "N/A", "N/A", "N/A", "N/A");
+}
+
