@@ -19,6 +19,7 @@
 #include "glusterfs.h"
 #include "protocol-common.h"
 #include "logging.h"
+#include "quota-common-utils.h"
 
 #include "cli1-xdr.h"
 
@@ -33,10 +34,14 @@
 #define CLI_DEFAULT_CMD_TIMEOUT              120
 #define CLI_TEN_MINUTES_TIMEOUT              600 //Longer timeout for volume top
 #define DEFAULT_CLI_LOG_FILE_DIRECTORY     DATADIR "/log/glusterfs"
-#define CLI_VOL_STATUS_BRICK_LEN              55
+#define CLI_VOL_STATUS_BRICK_LEN              43
 #define CLI_TAB_LENGTH                         8
 #define CLI_BRICK_STATUS_LINE_LEN             78
 
+/* Geo-rep command positional arguments' index  */
+#define GEO_REP_CMD_INDEX                      1
+#define GEO_REP_CMD_CONFIG_INDEX               4
+
 enum argp_option_keys {
 	ARGP_DEBUG_KEY = 133,
 	ARGP_PORT_KEY = 'p',
@@ -45,6 +50,7 @@ enum argp_option_keys {
 #define GLUSTER_MODE_SCRIPT    (1 << 0)
 #define GLUSTER_MODE_ERR_FATAL (1 << 1)
 #define GLUSTER_MODE_XML       (1 << 2)
+#define GLUSTER_MODE_WIGNORE   (1 << 3)
 
 
 #define GLUSTERD_GET_QUOTA_AUX_MOUNT_PATH(abspath, volname, path)      \
@@ -149,6 +155,7 @@ struct cli_local {
 
 struct cli_volume_status {
         int            port;
+        int            rdma_port;
         int            online;
         uint64_t       block_size;
         uint64_t       total_inodes;
@@ -165,6 +172,11 @@ struct cli_volume_status {
 #endif
 };
 
+struct snap_config_opt_vals_ {
+        char           *op_name;
+        char           *question;
+};
+
 typedef struct cli_volume_status cli_volume_status_t;
 
 typedef struct cli_local cli_local_t;
@@ -175,6 +187,8 @@ extern struct cli_state *global_state; /* use only in readline callback */
 
 typedef const char *(*cli_selector_t) (void *wcon);
 
+char *get_struct_variable (int mem_num, gf_gsync_status_t *sts_val);
+
 void *cli_getunamb (const char *tok, void **choices, cli_selector_t sel);
 
 int cli_cmd_register (struct cli_cmd_tree *tree, struct cli_cmd *cmd);
@@ -215,8 +229,8 @@ cli_submit_request (struct rpc_clnt *rpc, void *req, call_frame_t *frame,
                     xlator_t *this, fop_cbk_fn_t cbkfn, xdrproc_t xdrproc);
 
 int32_t
-cli_cmd_volume_create_parse (const char **words, int wordcount,
-                             dict_t **options);
+cli_cmd_volume_create_parse (struct cli_state *state, const char **words,
+                             int wordcount, dict_t **options);
 
 int32_t
 cli_cmd_volume_reset_parse (const char **words, int wordcount, dict_t **opt);
@@ -228,12 +242,25 @@ int32_t
 cli_cmd_quota_parse (const char **words, int wordcount, dict_t **opt);
 
 int32_t
-cli_cmd_volume_set_parse (const char **words, int wordcount,
-                          dict_t **options, char **op_errstr);
+cli_cmd_inode_quota_parse (const char **words, int wordcount, dict_t **opt);
+
+int32_t
+cli_cmd_bitrot_parse (const char **words, int wordcount, dict_t **opt);
+
+int32_t
+cli_cmd_volume_set_parse (struct cli_state *state, const char **words,
+                          int wordcount, dict_t **options, char **op_errstr);
+int32_t
+cli_cmd_ganesha_parse (struct cli_state *state, const char **words,
+                       int wordcount, dict_t **options, char **op_errstr);
 
 int32_t
 cli_cmd_volume_add_brick_parse (const char **words, int wordcount,
-                                dict_t **options);
+                                dict_t **options, int *type);
+
+int32_t
+cli_cmd_volume_detach_tier_parse (const char **words, int wordcount,
+                                  dict_t **options);
 
 int32_t
 cli_cmd_volume_remove_brick_parse (const char **words, int wordcount,
@@ -348,9 +375,24 @@ int
 cli_xml_output_vol_info (cli_local_t *local, dict_t *dict);
 
 int
-cli_xml_output_vol_quota_limit_list (char *volname, char *limit_list,
-                                      int op_ret, int op_errno,
-                                      char *op_errstr);
+cli_xml_output_vol_quota_limit_list_begin (cli_local_t *local, int op_ret,
+                                           int op_errno, char *op_errstr);
+int
+cli_xml_output_vol_quota_limit_list_end (cli_local_t *local);
+
+int
+cli_quota_list_xml_error (cli_local_t *local, char *path,
+                          char *errstr);
+
+int
+cli_quota_xml_output (cli_local_t *local, char *path, char *hl_str,
+                      char *sl_final, void *used, void *avail,
+                      char *sl, char *hl);
+
+int
+cli_quota_object_xml_output (cli_local_t *local, char *path, char *sl_str,
+                             quota_limits_t *limits, quota_meta_t *used_space,
+                             int64_t avail, char *sl, char *hl);
 
 int
 cli_xml_output_peer_status (dict_t *dict, int op_ret, int op_errno,
@@ -361,12 +403,14 @@ cli_xml_output_vol_rebalance (gf_cli_defrag_type op, dict_t *dict, int op_ret,
                               int op_errno, char *op_errstr);
 
 int
-cli_xml_output_vol_remove_brick (gf_boolean_t status_op, dict_t *dict,
-                                 int op_ret, int op_errno, char *op_errstr);
+cli_xml_output_vol_remove_brick_detach_tier (gf_boolean_t status_op,
+                                             dict_t *dict, int op_ret,
+                                             int op_errno, char *op_errstr,
+                                             const char *op);
 
 int
-cli_xml_output_vol_replace_brick (gf1_cli_replace_op op, dict_t *dict,
-                                  int op_ret, int op_errno, char *op_errstr);
+cli_xml_output_vol_replace_brick (char *op, dict_t *dict, int op_ret,
+                                  int op_errno, char *op_errstr);
 
 int
 cli_xml_output_vol_create (dict_t *dict, int op_ret, int op_errno,
@@ -382,6 +426,35 @@ cli_xml_output_vol_gsync (dict_t *dict, int op_ret, int op_errno,
 int
 cli_xml_output_vol_status_tasks_detail (cli_local_t *local, dict_t *dict);
 
+int
+cli_xml_output_snap_status_begin (cli_local_t *local, int op_ret, int op_errno,
+                                  char *op_errstr);
+int
+cli_xml_output_snap_status_end (cli_local_t *local);
+int
+cli_xml_output_snapshot (int cmd_type, dict_t *dict, int op_ret,
+                         int op_errno, char *op_errstr);
+int
+cli_xml_snapshot_status_single_snap (cli_local_t *local, dict_t *dict,
+                                     char *key);
 char *
 is_server_debug_xlator (void *myframe);
+
+int32_t
+cli_cmd_snapshot_parse (const char **words, int wordcount, dict_t **options,
+                        struct cli_state *state);
+
+int
+cli_xml_output_vol_getopts (dict_t *dict, int op_ret, int op_errno,
+                             char *op_errstr);
+
+void
+print_quota_list_header (int type);
+
+void
+print_quota_list_empty (char *path, int type);
+
+int
+gf_gsync_status_t_comparator (const void *p, const void *q);
+
 #endif /* __CLI_H__ */
