@@ -13,7 +13,7 @@
 #include "inode.h"
 #include "dict.h"
 #include "statedump.h"
-
+#include "libglusterfs-messages.h"
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
@@ -35,7 +35,8 @@ gf_fd_chain_fd_entries (fdentry_t *entries, uint32_t startidx,
         uint32_t        i = 0;
 
         if (!entries) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!entries");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!entries");
                 return -1;
         }
 
@@ -45,7 +46,7 @@ gf_fd_chain_fd_entries (fdentry_t *entries, uint32_t startidx,
         for (i = startidx; i < (endcount - 1); i++)
                 entries[i].next_free = i + 1;
 
-        /* i has already been incremented upto the last entry. */
+        /* i has already been incremented up to the last entry. */
         entries[i].next_free = GF_FDTABLE_END;
 
         return 0;
@@ -59,8 +60,9 @@ gf_fd_fdtable_expand (fdtable_t *fdtable, uint32_t nr)
         uint32_t     oldmax_fds = -1;
         int          ret = -1;
 
-        if (fdtable == NULL || nr < 0) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+        if (fdtable == NULL || nr > UINT32_MAX) {
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 ret = EINVAL;
                 goto out;
         }
@@ -127,7 +129,8 @@ __gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
         fdentry_t       *fdentries = NULL;
 
         if (count == NULL) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!count");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!count");
                 goto out;
         }
 
@@ -166,7 +169,8 @@ __gf_fd_fdtable_copy_all_fds (fdtable_t *fdtable, uint32_t *count)
         int        i         = 0;
 
         if (count == NULL) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!count");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!count");
                 goto out;
         }
 
@@ -218,7 +222,8 @@ gf_fd_fdtable_destroy (fdtable_t *fdtable)
         INIT_LIST_HEAD (&list);
 
         if (!fdtable) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!fdtable");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!fdtable");
                 return;
         }
 
@@ -253,7 +258,8 @@ gf_fd_unused_get (fdtable_t *fdtable, fd_t *fdptr)
         int             alloc_attempts = 0;
 
         if (fdtable == NULL || fdptr == NULL) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return EINVAL;
         }
 
@@ -271,7 +277,8 @@ gf_fd_unused_get (fdtable_t *fdtable, fd_t *fdptr)
                          * seriously wrong with our data structures.
                          */
                         if (alloc_attempts >= 2) {
-                                gf_log ("fd", GF_LOG_ERROR,
+                                gf_msg ("fd", GF_LOG_ERROR, 0,
+                                        LG_MSG_EXPAND_FD_TABLE_FAILED,
                                         "multiple attempts to expand fd table"
                                         " have failed.");
                                 goto out;
@@ -279,7 +286,8 @@ gf_fd_unused_get (fdtable_t *fdtable, fd_t *fdptr)
                         error = gf_fd_fdtable_expand (fdtable,
                                                       fdtable->max_fds + 1);
                         if (error) {
-                                gf_log ("fd", GF_LOG_ERROR,
+                                gf_msg ("fd", GF_LOG_ERROR, 0,
+                                        LG_MSG_EXPAND_FD_TABLE_FAILED,
                                         "Cannot expand fdtable: %s",
                                         strerror (error));
                                 goto out;
@@ -312,12 +320,14 @@ gf_fd_put (fdtable_t *fdtable, int32_t fd)
                 return;
 
         if (fdtable == NULL || fd < 0) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return;
         }
 
         if (!(fd < fdtable->max_fds)) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return;
         }
 
@@ -354,7 +364,8 @@ gf_fdptr_put (fdtable_t *fdtable, fd_t *fd)
         int32_t    i     = 0;
 
         if ((fdtable == NULL) || (fd == NULL)) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return;
         }
 
@@ -368,8 +379,10 @@ gf_fdptr_put (fdtable_t *fdtable, fd_t *fd)
                 }
 
                 if (fde == NULL) {
-                        gf_log_callingfn ("fd", GF_LOG_WARNING,
-                                "fd (%p) is not present in fdtable", fd);
+                        gf_msg_callingfn ("fd", GF_LOG_WARNING, 0,
+                                          LG_MSG_FD_NOT_FOUND_IN_FDTABLE,
+                                          "fd (%p) is not present in fdtable",
+                                          fd);
                         goto unlock_out;
                 }
 
@@ -401,13 +414,15 @@ gf_fd_fdptr_get (fdtable_t *fdtable, int64_t fd)
         fd_t *fdptr = NULL;
 
         if (fdtable == NULL || fd < 0) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 errno = EINVAL;
                 return NULL;
         }
 
         if (!(fd < fdtable->max_fds)) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 errno = EINVAL;
                 return NULL;
         }
@@ -440,7 +455,8 @@ fd_ref (fd_t *fd)
         fd_t *refed_fd = NULL;
 
         if (!fd) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "null fd");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "null fd");
                 return NULL;
         }
 
@@ -475,12 +491,15 @@ fd_destroy (fd_t *fd)
         xlator_t    *old_THIS = NULL;
 
         if (fd == NULL){
-                gf_log_callingfn ("xlator", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("xlator", GF_LOG_ERROR,  EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 goto out;
         }
 
         if (fd->inode == NULL){
-                gf_log_callingfn ("xlator", GF_LOG_ERROR, "fd->inode is NULL");
+                gf_msg_callingfn ("xlator", GF_LOG_ERROR, 0,
+                                  LG_MSG_FD_INODE_NULL,
+                                  "fd->inode is NULL");
                 goto out;
         }
         if (!fd->_ctx)
@@ -533,7 +552,8 @@ fd_unref (fd_t *fd)
         int32_t refcount = 0;
 
         if (!fd) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "fd is NULL");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "fd is NULL");
                 return;
         }
 
@@ -567,7 +587,8 @@ fd_t *
 fd_bind (fd_t *fd)
 {
         if (!fd || !fd->inode) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "!fd || !fd->inode");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!fd || !fd->inode");
                 return NULL;
         }
 
@@ -587,7 +608,8 @@ __fd_create (inode_t *inode, uint64_t pid)
         fd_t *fd = NULL;
 
         if (inode == NULL) {
-                gf_log_callingfn ("fd", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("fd", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
 
@@ -687,7 +709,8 @@ fd_lookup (inode_t *inode, pid_t pid)
         fd_t *fd = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!inode");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!inode");
                 return NULL;
         }
 
@@ -706,7 +729,8 @@ fd_lookup_uint64 (inode_t *inode, uint64_t pid)
         fd_t *fd = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!inode");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!inode");
                 return NULL;
         }
 
@@ -786,7 +810,8 @@ fd_lookup_anonymous (inode_t *inode)
         fd_t *fd = NULL;
 
         if (!inode) {
-                gf_log_callingfn ("fd", GF_LOG_WARNING, "!inode");
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "!inode");
                 return NULL;
         }
 
@@ -855,10 +880,6 @@ __fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
                                   (sizeof (struct _fd_ctx)
                                    * new_xl_count));
                 if (tmp == NULL) {
-                        gf_log_callingfn (THIS->name, GF_LOG_WARNING,
-                                          "realloc of fd->_ctx for fd "
-                                          "(ptr: %p) failed, cannot set the key"
-                                          , fd);
                         ret = -1;
                         goto out;
                 }
@@ -890,7 +911,8 @@ fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
         int ret = 0;
 
 	if (!fd || !xlator) {
-                gf_log_callingfn ("", GF_LOG_WARNING, "%p %p", fd, xlator);
+                gf_msg_callingfn ("fd", GF_LOG_WARNING, EINVAL,
+                                  LG_MSG_INVALID_ARG, "%p %p", fd, xlator);
 		return -1;
         }
 
