@@ -33,8 +33,8 @@ typedef struct _dentry dentry_t;
 #include "list.h"
 #include "xlator.h"
 #include "iatt.h"
-#include "uuid.h"
-
+#include "compat-uuid.h"
+#include "fd.h"
 
 struct _inode_table {
         pthread_mutex_t    lock;
@@ -109,6 +109,12 @@ struct _inode {
 inode_table_t *
 inode_table_new (size_t lru_limit, xlator_t *xl);
 
+void
+inode_table_destroy_all (glusterfs_ctx_t *ctx);
+
+void
+inode_table_destroy (inode_table_t *inode_table);
+
 inode_t *
 inode_new (inode_table_t *table);
 
@@ -134,6 +140,9 @@ inode_lookup (inode_t *inode);
 int
 inode_forget (inode_t *inode, uint64_t nlookup);
 
+int
+inode_ref_reduce_by_n (inode_t *inode, uint64_t nref);
+
 int
 inode_invalidate(inode_t *inode);
 
@@ -251,4 +260,22 @@ inode_ctx_put(inode_t *inode, xlator_t *this, uint64_t v)
 gf_boolean_t
 __is_root_gfid (uuid_t gfid);
 
+void
+__inode_table_set_lru_limit (inode_table_t *table, uint32_t lru_limit);
+
+void
+inode_table_set_lru_limit (inode_table_t *table, uint32_t lru_limit);
+
+void
+inode_ctx_merge (fd_t *fd, inode_t *inode, inode_t *linked_inode);
+
+int
+inode_is_linked (inode_t *inode);
+
+void
+inode_set_need_lookup (inode_t *inode, xlator_t *this);
+
+gf_boolean_t
+inode_needs_lookup (inode_t *inode, xlator_t *this);
+
 #endif /* _INODE_H */
