@@ -17,6 +17,7 @@
 #define RPC_CLNT_DEFAULT_REQUEST_COUNT 512
 
 #include "rpc-clnt.h"
+#include "rpc-clnt-ping.h"
 #include "byte-order.h"
 #include "xdr-rpcclnt.h"
 #include "rpc-transport.h"
@@ -109,34 +110,11 @@ out:
 }
 
 
-void
-saved_frames_delete (struct saved_frame *saved_frame,
-                     rpc_clnt_connection_t *conn)
-{
-        GF_VALIDATE_OR_GOTO ("rpc-clnt", saved_frame, out);
-        GF_VALIDATE_OR_GOTO ("rpc-clnt", conn, out);
-
-        pthread_mutex_lock (&conn->lock);
-        {
-                list_del_init (&saved_frame->list);
-                conn->saved_frames->count--;
-        }
-        pthread_mutex_unlock (&conn->lock);
-
-        if (saved_frame->rpcreq != NULL) {
-                rpc_clnt_reply_deinit (saved_frame->rpcreq,
-                                       conn->rpc_clnt->reqpool);
-        }
-
-        mem_put (saved_frame);
-out:
-        return;
-}
-
 
 static void
 call_bail (void *data)
 {
+        rpc_transport_t       *trans = NULL;
         struct rpc_clnt       *clnt = NULL;
         rpc_clnt_connection_t *conn = NULL;
         struct timeval         current;
@@ -147,12 +125,27 @@ call_bail (void *data)
         char                   frame_sent[256] = {0,};
         struct timespec        timeout = {0,};
         struct iovec           iov = {0,};
+        char                   peerid[UNIX_PATH_MAX] = {0};
 
         GF_VALIDATE_OR_GOTO ("client", data, out);
 
         clnt = data;
 
         conn = &clnt->conn;
+        pthread_mutex_lock (&conn->lock);
+        {
+            trans = conn->trans;
+            if (trans) {
+                    strncpy (peerid, conn->trans->peerinfo.identifier,
+                             sizeof (peerid)-1);
+
+            }
+        }
+        pthread_mutex_unlock (&conn->lock);
+        /*rpc_clnt_connection_cleanup will be unwinding all saved frames,
+         * bailed or otherwise*/
+        if (!trans)
+                goto out;
 
         gettimeofday (&current, NULL);
         INIT_LIST_HEAD (&list);
@@ -172,9 +165,9 @@ call_bail (void *data)
                                                            (void *) clnt);
 
                         if (conn->timer == NULL) {
-                                gf_log (conn->trans->name, GF_LOG_WARNING,
+                                gf_log (conn->name, GF_LOG_WARNING,
                                         "Cannot create bailout timer for %s",
-                                        conn->trans->peerinfo.identifier);
+                                        peerid);
                         }
                 }
 
@@ -197,7 +190,7 @@ call_bail (void *data)
                           256 - strlen (frame_sent),
                           ".%"GF_PRI_SUSECONDS, trav->saved_at.tv_usec);
 
-		gf_log (conn->trans->name, GF_LOG_ERROR,
+		gf_log (conn->name, GF_LOG_ERROR,
 			"bailing out frame type(%s) op(%s(%d)) xid = 0x%x "
                         "sent = %s. timeout = %d for %s",
 			trav->rpcreq->prog->progname,
@@ -205,7 +198,7 @@ call_bail (void *data)
                         trav->rpcreq->prog->procnames[trav->rpcreq->procnum] :
                         "--",
                         trav->rpcreq->procnum, trav->rpcreq->xid, frame_sent,
-                        conn->frame_timeout, conn->trans->peerinfo.identifier);
+                        conn->frame_timeout, peerid);
 
                 clnt = rpc_clnt_ref (clnt);
                 trav->rpcreq->rpc_status = -1;
@@ -357,7 +350,7 @@ saved_frames_unwind (struct saved_frames *saved_frames)
                 if (!trav->rpcreq || !trav->rpcreq->prog)
                         continue;
 
-                gf_log_callingfn (trav->rpcreq->conn->trans->name,
+                gf_log_callingfn (trav->rpcreq->conn->name,
                                   GF_LOG_ERROR,
                                   "forced unwinding frame type(%s) op(%s(%d)) "
                                   "called at %s (xid=0x%x)",
@@ -394,7 +387,7 @@ saved_frames_destroy (struct saved_frames *frames)
 
 
 void
-rpc_clnt_reconnect (void *trans_ptr)
+rpc_clnt_reconnect (void *conn_ptr)
 {
         rpc_transport_t         *trans = NULL;
         rpc_clnt_connection_t   *conn  = NULL;
@@ -402,15 +395,16 @@ rpc_clnt_reconnect (void *trans_ptr)
         int32_t                  ret   = 0;
         struct rpc_clnt         *clnt  = NULL;
 
-        trans = trans_ptr;
-        if (!trans || !trans->mydata)
-                return;
-
-        conn  = trans->mydata;
+        conn  = conn_ptr;
         clnt = conn->rpc_clnt;
 
         pthread_mutex_lock (&conn->lock);
         {
+                trans = conn->trans;
+                if (!trans) {
+                        pthread_mutex_unlock (&conn->lock);
+                        return;
+                }
                 if (conn->reconnect)
                         gf_timer_call_cancel (clnt->ctx,
                                               conn->reconnect);
@@ -420,25 +414,21 @@ rpc_clnt_reconnect (void *trans_ptr)
                         ts.tv_sec = 3;
                         ts.tv_nsec = 0;
 
-                        gf_log (trans->name, GF_LOG_TRACE,
+                        gf_log (conn->name, GF_LOG_TRACE,
                                 "attempting reconnect");
                         ret = rpc_transport_connect (trans,
                                                      conn->config.remote_port);
                         conn->reconnect =
                                 gf_timer_call_after (clnt->ctx, ts,
                                                      rpc_clnt_reconnect,
-                                                     trans);
+                                                     conn);
                 } else {
-                        gf_log (trans->name, GF_LOG_TRACE,
+                        gf_log (conn->name, GF_LOG_TRACE,
                                 "breaking reconnect chain");
                 }
         }
         pthread_mutex_unlock (&conn->lock);
 
-        if ((ret == -1) && (errno != EINPROGRESS) && (clnt->notifyfn)) {
-                clnt->notifyfn (clnt, clnt->mydata, RPC_CLNT_DISCONNECT, NULL);
-        }
-
         return;
 }
 
@@ -457,7 +447,7 @@ rpc_clnt_fill_request_info (struct rpc_clnt *clnt, rpc_request_info_t *info)
         pthread_mutex_unlock (&clnt->conn.lock);
 
         if (ret == -1) {
-                gf_log (clnt->conn.trans->name, GF_LOG_CRITICAL,
+                gf_log (clnt->conn.name, GF_LOG_CRITICAL,
                         "cannot lookup the saved "
                         "frame corresponding to xid (%d)", info->xid);
                 goto out;
@@ -517,11 +507,9 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
 
         clnt = conn->rpc_clnt;
 
-        gf_log (conn->trans->name, GF_LOG_TRACE,
-                "cleaning up state in transport object %p", conn->trans);
-
         pthread_mutex_lock (&conn->lock);
         {
+
                 saved_frames = conn->saved_frames;
                 conn->saved_frames = saved_frames_new ();
 
@@ -537,7 +525,11 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
                         gf_timer_call_cancel (clnt->ctx, conn->ping_timer);
                         conn->ping_timer = NULL;
                         conn->ping_started = 0;
+                        rpc_clnt_unref (clnt);
                 }
+                /*reset rpc msgs stats*/
+                conn->pingcnt = 0;
+                conn->msgcnt = 0;
         }
         pthread_mutex_unlock (&conn->lock);
 
@@ -651,7 +643,7 @@ rpc_clnt_reply_init (rpc_clnt_connection_t *conn, rpc_transport_pollin_t *msg,
         ret = xdr_to_rpc_reply (msgbuf, msglen, &rpcmsg, &progmsg,
                                 req->verf.authdata);
         if (ret != 0) {
-                gf_log (conn->trans->name, GF_LOG_WARNING,
+                gf_log (conn->name, GF_LOG_WARNING,
                         "RPC reply decoding failed");
                 goto out;
         }
@@ -662,13 +654,13 @@ rpc_clnt_reply_init (rpc_clnt_connection_t *conn, rpc_transport_pollin_t *msg,
                 goto out;
         }
 
-        gf_log (conn->trans->name, GF_LOG_TRACE,
+        gf_log (conn->name, GF_LOG_TRACE,
                 "received rpc message (RPC XID: 0x%x"
                 " Program: %s, ProgVers: %d, Proc: %d) from rpc-transport (%s)",
                 saved_frame->rpcreq->xid,
                 saved_frame->rpcreq->prog->progname,
                 saved_frame->rpcreq->prog->progver,
-                saved_frame->rpcreq->procnum, conn->trans->name);
+                saved_frame->rpcreq->procnum, conn->name);
 
 out:
         if (ret != 0) {
@@ -696,18 +688,19 @@ rpc_clnt_handle_cbk (struct rpc_clnt *clnt, rpc_transport_pollin_t *msg)
         clnt = rpc_clnt_ref (clnt);
         ret = xdr_to_rpc_call (msgbuf, msglen, &rpcmsg, &progmsg, NULL,NULL);
         if (ret == -1) {
-                gf_log (clnt->conn.trans->name, GF_LOG_WARNING,
+                gf_log (clnt->conn.name, GF_LOG_WARNING,
                         "RPC call decoding failed");
                 goto out;
         }
 
-        gf_log (clnt->conn.trans->name, GF_LOG_TRACE,
-                "received rpc message (XID: 0x%lx, "
-                "Ver: %ld, Program: %ld, ProgVers: %ld, Proc: %ld) "
+        gf_log (clnt->conn.name, GF_LOG_TRACE,
+		"receivd rpc message (XID: 0x%" GF_PRI_RPC_XID ", "
+		"Ver: %" GF_PRI_RPC_VERSION ", Program: %" GF_PRI_RPC_PROG_ID ", "
+		"ProgVers: %" GF_PRI_RPC_PROG_VERS ", Proc: %" GF_PRI_RPC_PROC ") "
                 "from rpc-transport (%s)", rpc_call_xid (&rpcmsg),
                 rpc_call_rpcvers (&rpcmsg), rpc_call_program (&rpcmsg),
                 rpc_call_progver (&rpcmsg), rpc_call_progproc (&rpcmsg),
-                clnt->conn.trans->name);
+                clnt->conn.name);
 
         procnum = rpc_call_progproc (&rpcmsg);
 
@@ -731,7 +724,7 @@ rpc_clnt_handle_cbk (struct rpc_clnt *clnt, rpc_transport_pollin_t *msg)
         }
 
 out:
-        clnt = rpc_clnt_unref (clnt);
+        rpc_clnt_unref (clnt);
         return ret;
 }
 
@@ -750,7 +743,7 @@ rpc_clnt_handle_reply (struct rpc_clnt *clnt, rpc_transport_pollin_t *pollin)
         xid = ntoh32 (*((uint32_t *)pollin->vector[0].iov_base));
         saved_frame = lookup_frame (conn, xid);
         if (saved_frame == NULL) {
-                gf_log (conn->trans->name, GF_LOG_ERROR,
+                gf_log (conn->name, GF_LOG_ERROR,
                         "cannot lookup the saved frame for reply with xid (%u)",
                         xid);
                 goto out;
@@ -758,7 +751,7 @@ rpc_clnt_handle_reply (struct rpc_clnt *clnt, rpc_transport_pollin_t *pollin)
 
         req = saved_frame->rpcreq;
         if (req == NULL) {
-                gf_log (conn->trans->name, GF_LOG_ERROR,
+                gf_log (conn->name, GF_LOG_ERROR,
                         "no request with frame for xid (%u)", xid);
                 goto out;
         }
@@ -766,7 +759,7 @@ rpc_clnt_handle_reply (struct rpc_clnt *clnt, rpc_transport_pollin_t *pollin)
         ret = rpc_clnt_reply_init (conn, pollin, req, saved_frame);
         if (ret != 0) {
                 req->rpc_status = -1;
-                gf_log (conn->trans->name, GF_LOG_WARNING,
+                gf_log (conn->name, GF_LOG_WARNING,
                         "initialising rpc reply failed");
         }
 
@@ -781,7 +774,7 @@ out:
                 mem_put (saved_frame);
         }
 
-        clnt = rpc_clnt_unref (clnt);
+        rpc_clnt_unref (clnt);
         return ret;
 }
 
@@ -859,7 +852,7 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                                 conn->reconnect =
                                         gf_timer_call_after (clnt->ctx, ts,
                                                              rpc_clnt_reconnect,
-                                                             conn->trans);
+                                                             conn);
                         }
                 }
                 pthread_mutex_unlock (&conn->lock);
@@ -961,10 +954,17 @@ rpc_clnt_connection_init (struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
 {
         int                    ret  = -1;
         rpc_clnt_connection_t *conn = NULL;
+        rpc_transport_t       *trans = NULL;
 
         conn = &clnt->conn;
         pthread_mutex_init (&clnt->conn.lock, NULL);
 
+        conn->name = gf_strdup (name);
+        if (!conn->name) {
+                ret = -1;
+                goto out;
+        }
+
         ret = dict_get_int32 (options, "frame-timeout",
                               &conn->frame_timeout);
         if (ret >= 0) {
@@ -975,25 +975,41 @@ rpc_clnt_connection_init (struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
                         "defaulting frame-timeout to 30mins");
                 conn->frame_timeout = 1800;
         }
+        conn->rpc_clnt = clnt;
 
-        conn->trans = rpc_transport_load (ctx, options, name);
-        if (!conn->trans) {
+        ret = dict_get_int32 (options, "ping-timeout",
+                              &conn->ping_timeout);
+        if (ret >= 0) {
+                gf_log (name, GF_LOG_DEBUG,
+                        "setting ping-timeout to %d", conn->ping_timeout);
+        } else {
+                /*TODO: Once the epoll thread model is fixed,
+                  change the default ping-timeout to 30sec */
+                gf_log (name, GF_LOG_DEBUG,
+                        "disable ping-timeout");
+                conn->ping_timeout = 0;
+        }
+
+        trans = rpc_transport_load (ctx, options, name);
+        if (!trans) {
                 gf_log (name, GF_LOG_WARNING, "loading of new rpc-transport"
                         " failed");
                 ret = -1;
                 goto out;
         }
+        rpc_transport_ref (trans);
 
-        rpc_transport_ref (conn->trans);
-
-        conn->rpc_clnt = clnt;
+        pthread_mutex_lock (&conn->lock);
+        {
+                conn->trans = trans;
+                trans = NULL;
+        }
+        pthread_mutex_unlock (&conn->lock);
 
         ret = rpc_transport_register_notify (conn->trans, rpc_clnt_notify,
                                              conn);
         if (ret == -1) {
                 gf_log (name, GF_LOG_WARNING, "registering notify failed");
-                rpc_clnt_connection_cleanup (conn);
-                conn = NULL;
                 goto out;
         }
 
@@ -1001,13 +1017,26 @@ rpc_clnt_connection_init (struct rpc_clnt *clnt, glusterfs_ctx_t *ctx,
         if (!conn->saved_frames) {
                 gf_log (name, GF_LOG_WARNING, "creation of saved_frames "
                         "failed");
-                rpc_clnt_connection_cleanup (conn);
+                ret = -1;
                 goto out;
         }
 
         ret = 0;
 
 out:
+        if (ret) {
+                pthread_mutex_lock (&conn->lock);
+                {
+                        trans = conn->trans;
+                        conn->trans = NULL;
+                }
+                pthread_mutex_unlock (&conn->lock);
+                if (trans)
+                        rpc_transport_unref (trans);
+                //conn cleanup needs to be done since we might have failed to
+                // register notification.
+                rpc_clnt_connection_cleanup (conn);
+        }
         return ret;
 }
 
@@ -1079,7 +1108,12 @@ rpc_clnt_start (struct rpc_clnt *rpc)
 
         conn = &rpc->conn;
 
-        rpc_clnt_reconnect (conn->trans);
+        pthread_mutex_lock (&conn->lock);
+        {
+                rpc->disabled = 0;
+        }
+        pthread_mutex_unlock (&conn->lock);
+        rpc_clnt_reconnect (conn);
 
         return 0;
 }
@@ -1103,7 +1137,7 @@ xdr_serialize_glusterfs_auth (char *dest, struct auth_glusterfs_parms_v2 *au)
 {
         ssize_t ret = -1;
         XDR     xdr;
-        uint64_t ngroups = 0;
+        u_long  ngroups = 0;
         int     max_groups = 0;
 
         if ((!dest) || (!au))
@@ -1254,7 +1288,7 @@ rpc_clnt_record_build_record (struct rpc_clnt *clnt, int prognum, int progver,
                                              xid, au, &request, auth_data);
 
         if (ret == -1) {
-                gf_log (clnt->conn.trans->name, GF_LOG_WARNING,
+                gf_log (clnt->conn.name, GF_LOG_WARNING,
                         "cannot build a rpc-request xid (%"PRIu64")", xid);
                 goto out;
         }
@@ -1277,7 +1311,7 @@ rpc_clnt_record_build_record (struct rpc_clnt *clnt, int prognum, int progver,
                                                   hdrsize);
 
         if (!recordhdr.iov_base) {
-                gf_log (clnt->conn.trans->name, GF_LOG_ERROR,
+                gf_log (clnt->conn.name, GF_LOG_ERROR,
                         "Failed to build record header");
                 iobuf_unref (request_iob);
                 request_iob = NULL;
@@ -1333,8 +1367,7 @@ rpc_clnt_record (struct rpc_clnt *clnt, call_frame_t *call_frame,
          * We can truncate the groups, but should not touch the lk_owner. */
         max_groups = GF_AUTH_GLUSTERFS_MAX_GROUPS (au.lk_owner.lk_owner_len);
         if (au.groups.groups_len > max_groups) {
-                GF_LOG_OCCASIONALLY (gf_auth_max_groups_log,
-                                     clnt->conn.trans->name,
+                GF_LOG_OCCASIONALLY (gf_auth_max_groups_log, clnt->conn.name,
                                      GF_LOG_WARNING, "truncating grouplist "
                                      "from %d to %d", au.groups.groups_len,
                                      max_groups);
@@ -1344,14 +1377,14 @@ rpc_clnt_record (struct rpc_clnt *clnt, call_frame_t *call_frame,
 
         max_lkowner_len = GF_AUTH_GLUSTERFS_MAX_LKOWNER (au.groups.groups_len);
         if (au.lk_owner.lk_owner_len > max_lkowner_len) {
-                gf_log (clnt->conn.trans->name, GF_LOG_ERROR, "lkowner field "
-                        "is too big (%d), it does not fit in the rpc-header",
+                gf_log (clnt->conn.name, GF_LOG_ERROR, "lkowner field is too "
+                        "big (%d), it does not fit in the rpc-header",
                         au.lk_owner.lk_owner_len);
                 errno = E2BIG;
                 goto out;
         }
 
-        gf_log (clnt->conn.trans->name, GF_LOG_TRACE, "Auth Info: pid: %u, uid: %d"
+        gf_log (clnt->conn.name, GF_LOG_TRACE, "Auth Info: pid: %u, uid: %d"
                 ", gid: %d, owner: %s", au.pid, au.uid, au.gid,
                 lkowner_utoa (&call_frame->root->lk_owner));
 
@@ -1361,7 +1394,7 @@ rpc_clnt_record (struct rpc_clnt *clnt, call_frame_t *call_frame,
                                                     callid, &au,
                                                     rpchdr);
         if (!request_iob) {
-                gf_log (clnt->conn.trans->name, GF_LOG_WARNING,
+                gf_log (clnt->conn.name, GF_LOG_WARNING,
                         "cannot build rpc-record");
                 goto out;
         }
@@ -1397,7 +1430,7 @@ rpcclnt_cbk_program_register (struct rpc_clnt *clnt,
         pthread_mutex_unlock (&clnt->lock);
 
         if (already_registered) {
-                gf_log_callingfn (clnt->conn.trans->name, GF_LOG_DEBUG,
+                gf_log_callingfn (clnt->conn.name, GF_LOG_DEBUG,
                                   "already registered");
                 ret = 0;
                 goto out;
@@ -1421,17 +1454,18 @@ rpcclnt_cbk_program_register (struct rpc_clnt *clnt,
         pthread_mutex_unlock (&clnt->lock);
 
         ret = 0;
-        gf_log (clnt->conn.trans->name, GF_LOG_DEBUG,
+        gf_log (clnt->conn.name, GF_LOG_DEBUG,
                 "New program registered: %s, Num: %d, Ver: %d",
                 program->progname, program->prognum,
                 program->progver);
 
 out:
-        if (ret == -1) {
-                gf_log (clnt->conn.trans->name, GF_LOG_ERROR,
-                        "Program registration failed:"
-                        " %s, Num: %d, Ver: %d", program->progname,
-                        program->prognum, program->progver);
+        if (ret == -1 && clnt) {
+                        gf_log (clnt->conn.name, GF_LOG_ERROR,
+                                        "Program registration failed:"
+                                        " %s, Num: %d, Ver: %d",
+                                        program->progname,
+                                        program->prognum, program->progver);
         }
 
         return ret;
@@ -1463,10 +1497,6 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
 
         conn = &rpc->conn;
 
-        if (conn->trans == NULL) {
-                goto out;
-        }
-
         rpcreq = mem_get (rpc->reqpool);
         if (rpcreq == NULL) {
                 goto out;
@@ -1502,7 +1532,7 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                                        procnum, proglen,
                                        &rpchdr, callid);
         if (!request_iob) {
-                gf_log (conn->trans->name, GF_LOG_WARNING,
+                gf_log (conn->name, GF_LOG_WARNING,
                         "cannot build rpc-record");
                 goto out;
         }
@@ -1531,26 +1561,26 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                                                      conn->config.remote_port);
                 }
 
-                ret = rpc_transport_submit_request (rpc->conn.trans,
-                                                    &req);
+                ret = rpc_transport_submit_request (conn->trans, &req);
                 if (ret == -1) {
-                        gf_log (conn->trans->name, GF_LOG_WARNING,
+                        gf_log (conn->name, GF_LOG_WARNING,
                                 "failed to submit rpc-request "
                                 "(XID: 0x%x Program: %s, ProgVers: %d, "
                                 "Proc: %d) to rpc-transport (%s)", rpcreq->xid,
                                 rpcreq->prog->progname, rpcreq->prog->progver,
-                                rpcreq->procnum, rpc->conn.trans->name);
+                                rpcreq->procnum, conn->name);
                 }
 
                 if ((ret >= 0) && frame) {
                         /* Save the frame in queue */
                         __save_frame (rpc, frame, rpcreq);
+                        conn->msgcnt++;
 
                         gf_log ("rpc-clnt", GF_LOG_TRACE, "submitted request "
                                 "(XID: 0x%x Program: %s, ProgVers: %d, "
                                 "Proc: %d) to rpc-transport (%s)", rpcreq->xid,
                                 rpcreq->prog->progname, rpcreq->prog->progver,
-                                rpcreq->procnum, rpc->conn.trans->name);
+                                rpcreq->procnum, conn->name);
                 }
         }
         pthread_mutex_unlock (&conn->lock);
@@ -1559,6 +1589,7 @@ rpc_clnt_submit (struct rpc_clnt *rpc, rpc_clnt_prog_t *prog,
                 goto out;
         }
 
+        rpc_clnt_check_and_start_ping (rpc);
         ret = 0;
 
 out:
@@ -1598,11 +1629,22 @@ rpc_clnt_ref (struct rpc_clnt *rpc)
 static void
 rpc_clnt_trigger_destroy (struct rpc_clnt *rpc)
 {
+        rpc_clnt_connection_t  *conn  = NULL;
+        rpc_transport_t        *trans = NULL;
+
         if (!rpc)
                 return;
 
-        rpc_clnt_disable (rpc);
-        rpc_transport_unref (rpc->conn.trans);
+        /* reading conn->trans outside conn->lock is OK, since this is the last
+         * ref*/
+        conn = &rpc->conn;
+        trans = conn->trans;
+        rpc_clnt_disconnect (rpc);
+
+        /* This is to account for rpc_clnt_disable that might have been called
+         * before rpc_clnt_unref */
+        if (trans)
+                rpc_transport_unref (trans);
 }
 
 static void
@@ -1671,6 +1713,7 @@ void
 rpc_clnt_disable (struct rpc_clnt *rpc)
 {
         rpc_clnt_connection_t *conn = NULL;
+        rpc_transport_t       *trans = NULL;
 
         if (!rpc) {
                 goto out;
@@ -1697,12 +1740,59 @@ rpc_clnt_disable (struct rpc_clnt *rpc)
                         gf_timer_call_cancel (rpc->ctx, conn->ping_timer);
                         conn->ping_timer = NULL;
                         conn->ping_started = 0;
+                        rpc_clnt_unref (rpc);
                 }
+                trans = conn->trans;
+                conn->trans = NULL;
 
         }
         pthread_mutex_unlock (&conn->lock);
 
-        rpc_transport_disconnect (rpc->conn.trans);
+        if (trans) {
+                rpc_transport_disconnect (trans);
+        }
+
+out:
+        return;
+}
+
+void
+rpc_clnt_disconnect (struct rpc_clnt *rpc)
+{
+        rpc_clnt_connection_t *conn  = NULL;
+        rpc_transport_t       *trans = NULL;
+
+        if (!rpc)
+                goto out;
+
+        conn = &rpc->conn;
+
+        pthread_mutex_lock (&conn->lock);
+        {
+                rpc->disabled = 1;
+                if (conn->timer) {
+                        gf_timer_call_cancel (rpc->ctx, conn->timer);
+                        conn->timer = NULL;
+                }
+
+                if (conn->reconnect) {
+                        gf_timer_call_cancel (rpc->ctx, conn->reconnect);
+                        conn->reconnect = NULL;
+                }
+                conn->connected = 0;
+
+                if (conn->ping_timer) {
+                        gf_timer_call_cancel (rpc->ctx, conn->ping_timer);
+                        conn->ping_timer = NULL;
+                        conn->ping_started = 0;
+                }
+                trans = conn->trans;
+        }
+        pthread_mutex_unlock (&conn->lock);
+
+        if (trans) {
+                rpc_transport_disconnect (trans);
+        }
 
 out:
         return;
@@ -1714,7 +1804,7 @@ rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
 {
         if (config->rpc_timeout) {
                 if (config->rpc_timeout != rpc->conn.config.rpc_timeout)
-                        gf_log (rpc->conn.trans->name, GF_LOG_INFO,
+                        gf_log (rpc->conn.name, GF_LOG_INFO,
                                 "changing timeout to %d (from %d)",
                                 config->rpc_timeout,
                                 rpc->conn.config.rpc_timeout);
@@ -1723,7 +1813,7 @@ rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
 
         if (config->remote_port) {
                 if (config->remote_port != rpc->conn.config.remote_port)
-                        gf_log (rpc->conn.trans->name, GF_LOG_INFO,
+                        gf_log (rpc->conn.name, GF_LOG_INFO,
                                 "changing port to %d (from %d)",
                                 config->remote_port,
                                 rpc->conn.config.remote_port);
@@ -1735,13 +1825,13 @@ rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
                 if (rpc->conn.config.remote_host) {
                         if (strcmp (rpc->conn.config.remote_host,
                                     config->remote_host))
-                                gf_log (rpc->conn.trans->name, GF_LOG_INFO,
+                                gf_log (rpc->conn.name, GF_LOG_INFO,
                                         "changing hostname to %s (from %s)",
                                         config->remote_host,
                                         rpc->conn.config.remote_host);
-                        FREE (rpc->conn.config.remote_host);
+                        GF_FREE (rpc->conn.config.remote_host);
                 } else {
-                        gf_log (rpc->conn.trans->name, GF_LOG_INFO,
+                        gf_log (rpc->conn.name, GF_LOG_INFO,
                                 "setting hostname to %s",
                                 config->remote_host);
                 }
