@@ -107,7 +107,7 @@ out:
 
 rpcsvc_vector_sizer
 rpcsvc_get_program_vector_sizer (rpcsvc_t *svc, uint32_t prognum,
-                                 uint32_t progver, uint32_t procnum)
+                                 uint32_t progver, int procnum)
 {
         rpcsvc_program_t        *program = NULL;
         char                    found    = 0;
@@ -130,7 +130,7 @@ rpcsvc_get_program_vector_sizer (rpcsvc_t *svc, uint32_t prognum,
 
         if (found) {
                 /* Make sure the requested procnum is supported by RPC prog */
-                if (procnum >= program->numactors) {
+                if ((procnum < 0) || (procnum >= program->numactors)) {
                         gf_log (GF_RPCSVC, GF_LOG_ERROR,
                                 "RPC procedure %d not available for Program %s",
                                 procnum, program->progname);
@@ -179,13 +179,25 @@ rpcsvc_can_outstanding_req_be_ignored (rpcsvc_request_t *req)
 int
 rpcsvc_request_outstanding (rpcsvc_request_t *req, int delta)
 {
-        int ret = 0;
-        int old_count = 0;
-        int new_count = 0;
-        int limit = 0;
+        int             ret = -1;
+        int             old_count = 0;
+        int             new_count = 0;
+        int             limit = 0;
+        gf_boolean_t    throttle = _gf_false;
 
-        if (rpcsvc_can_outstanding_req_be_ignored (req))
-                return 0;
+        if (!req)
+                goto out;
+
+        throttle = rpcsvc_get_throttle (req->svc);
+        if (!throttle) {
+                ret = 0;
+                goto out;
+        }
+
+        if (rpcsvc_can_outstanding_req_be_ignored (req)) {
+                ret = 0;
+                goto out;
+        }
 
         pthread_mutex_lock (&req->trans->lock);
         {
@@ -206,6 +218,7 @@ rpcsvc_request_outstanding (rpcsvc_request_t *req, int delta)
 unlock:
         pthread_mutex_unlock (&req->trans->lock);
 
+out:
         return ret;
 }
 
@@ -222,11 +235,13 @@ rpcsvc_program_actor (rpcsvc_request_t *req)
         rpcsvc_actor_t          *actor   = NULL;
         rpcsvc_t                *svc     = NULL;
         char                    found    = 0;
+        char                    *peername = NULL;
 
         if (!req)
                 goto err;
 
         svc = req->svc;
+        peername = req->trans->peerinfo.identifier;
         pthread_mutex_lock (&svc->rpclock);
         {
                 list_for_each_entry (program, &svc->programs, program) {
@@ -250,30 +265,34 @@ rpcsvc_program_actor (rpcsvc_request_t *req)
                          */
                         gf_log (GF_RPCSVC, (req->prognum == ACL_PROGRAM) ?
                                 GF_LOG_DEBUG : GF_LOG_WARNING,
-                                "RPC program not available (req %u %u)",
-                                req->prognum, req->progver);
+                                "RPC program not available (req %u %u) for %s",
+                                req->prognum, req->progver,
+                                peername);
                         err = PROG_UNAVAIL;
                         goto err;
                 }
 
                 gf_log (GF_RPCSVC, GF_LOG_WARNING,
-                        "RPC program version not available (req %u %u)",
-                        req->prognum, req->progver);
+                        "RPC program version not available (req %u %u) for %s",
+                        req->prognum, req->progver,
+                        peername);
                 goto err;
         }
         req->prog = program;
         if (!program->actors) {
                 gf_log (GF_RPCSVC, GF_LOG_WARNING,
-                        "RPC Actor not found for program %s %d",
-                        program->progname, program->prognum);
+                        "RPC Actor not found for program %s %d for %s",
+                        program->progname, program->prognum,
+                        peername);
                 err = SYSTEM_ERR;
                 goto err;
         }
 
         if ((req->procnum < 0) || (req->procnum >= program->numactors)) {
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "RPC Program procedure not"
-                        " available for procedure %d in %s", req->procnum,
-                        program->progname);
+                        " available for procedure %d in %s for  %s",
+                        req->procnum, program->progname,
+                        peername);
                 err = PROC_UNAVAIL;
                 goto err;
         }
@@ -281,8 +300,9 @@ rpcsvc_program_actor (rpcsvc_request_t *req)
         actor = &program->actors[req->procnum];
         if (!actor->actor) {
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "RPC Program procedure not"
-                        " available for procedure %d in %s", req->procnum,
-                        program->progname);
+                        " available for procedure %d in %s for %s",
+                        req->procnum, program->progname,
+                        peername);
                 err = PROC_UNAVAIL;
                 actor = NULL;
                 goto err;
@@ -291,8 +311,9 @@ rpcsvc_program_actor (rpcsvc_request_t *req)
         req->synctask = program->synctask;
 
         err = SUCCESS;
-        gf_log (GF_RPCSVC, GF_LOG_TRACE, "Actor found: %s - %s",
-                program->progname, actor->procname);
+        gf_log (GF_RPCSVC, GF_LOG_TRACE, "Actor found: %s - %s for %s",
+                program->progname, actor->procname,
+                peername);
 err:
         if (req)
                 req->rpc_err = err;
@@ -401,9 +422,9 @@ rpcsvc_request_init (rpcsvc_t *svc, rpc_transport_t *trans,
         req->msg[0] = progmsg;
         req->iobref = iobref_ref (msg->iobref);
         if (msg->vectored) {
-                /* msg->vector[2] is defined in structure. prevent a
+                /* msg->vector[MAX_IOVEC] is defined in structure. prevent a
                    out of bound access */
-                for (i = 1; i < min (msg->count, 2); i++) {
+                for (i = 1; i < min (msg->count, MAX_IOVEC); i++) {
                         req->msg[i] = msg->vector[i];
                 }
         }
@@ -418,13 +439,7 @@ rpcsvc_request_init (rpcsvc_t *svc, rpc_transport_t *trans,
          * been copied into the required sections of the req structure,
          * we just need to fill in the meta-data about it now.
          */
-        req->cred.flavour = rpc_call_cred_flavour (callmsg);
-        req->cred.datalen = rpc_call_cred_len (callmsg);
-        req->verf.flavour = rpc_call_verf_flavour (callmsg);
-        req->verf.datalen = rpc_call_verf_len (callmsg);
-
-        /* AUTH */
-        rpcsvc_auth_request_init (req);
+        rpcsvc_auth_request_init (req, callmsg);
         return req;
 }
 
@@ -471,9 +486,10 @@ rpcsvc_request_create (rpcsvc_t *svc, rpc_transport_t *trans,
         ret = -1;
         rpcsvc_request_init (svc, trans, &rpcmsg, progmsg, msg, req);
 
-        gf_log (GF_RPCSVC, GF_LOG_TRACE, "received rpc-message (XID: 0x%lx, "
-                "Ver: %ld, Program: %ld, ProgVers: %ld, Proc: %ld) from"
-                " rpc-transport (%s)", rpc_call_xid (&rpcmsg),
+        gf_log (GF_RPCSVC, GF_LOG_TRACE, "received rpc-message "
+		"(XID: 0x%" GF_PRI_RPC_XID ", Ver: %" GF_PRI_RPC_VERSION ", Program: %" GF_PRI_RPC_PROG_ID ", "
+		"ProgVers: %" GF_PRI_RPC_PROG_VERS ", Proc: %" GF_PRI_RPC_PROC ") "
+                "from rpc-transport (%s)", rpc_call_xid (&rpcmsg),
                 rpc_call_rpcvers (&rpcmsg), rpc_call_program (&rpcmsg),
                 rpc_call_progver (&rpcmsg), rpc_call_progproc (&rpcmsg),
                 trans->name);
@@ -489,8 +505,9 @@ rpcsvc_request_create (rpcsvc_t *svc, rpc_transport_t *trans,
                 /* LOG- TODO: print rpc version, also print the peerinfo
                    from transport */
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "RPC version not supported "
-                        "(XID: 0x%lx, Ver: %ld, Prog: %ld, ProgVers: %ld, "
-                        "Proc: %ld) from trans (%s)", rpc_call_xid (&rpcmsg),
+			"(XID: 0x%" GF_PRI_RPC_XID ", Ver: %" GF_PRI_RPC_VERSION ", Program: %" GF_PRI_RPC_PROG_ID ", "
+			"ProgVers: %" GF_PRI_RPC_PROG_VERS ", Proc: %" GF_PRI_RPC_PROC ") "
+			"from trans (%s)", rpc_call_xid (&rpcmsg),
                         rpc_call_rpcvers (&rpcmsg), rpc_call_program (&rpcmsg),
                         rpc_call_progver (&rpcmsg), rpc_call_progproc (&rpcmsg),
                         trans->name);
@@ -506,8 +523,9 @@ rpcsvc_request_create (rpcsvc_t *svc, rpc_transport_t *trans,
                  */
                 rpcsvc_request_seterr (req, AUTH_ERROR);
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "auth failed on request. "
-                        "(XID: 0x%lx, Ver: %ld, Prog: %ld, ProgVers: %ld, "
-                        "Proc: %ld) from trans (%s)", rpc_call_xid (&rpcmsg),
+			"(XID: 0x%" GF_PRI_RPC_XID ", Ver: %" GF_PRI_RPC_VERSION ", Program: %" GF_PRI_RPC_PROG_ID ", "
+			"ProgVers: %" GF_PRI_RPC_PROG_VERS ", Proc: %" GF_PRI_RPC_PROC ") "
+                        "from trans (%s)", rpc_call_xid (&rpcmsg),
                         rpc_call_rpcvers (&rpcmsg), rpc_call_program (&rpcmsg),
                         rpc_call_progver (&rpcmsg), rpc_call_progproc (&rpcmsg),
                         trans->name);
@@ -615,11 +633,13 @@ rpcsvc_handle_rpc_call (rpcsvc_t *svc, rpc_transport_t *trans,
 
         if (0 == svc->allow_insecure && unprivileged && !actor->unprivileged) {
                         /* Non-privileged user, fail request */
-                        gf_log ("glusterd", GF_LOG_ERROR,
+                        gf_log (GF_RPCSVC, GF_LOG_ERROR,
                                 "Request received from non-"
                                 "privileged port. Failing request");
-                        rpcsvc_request_destroy (req);
-                        return -1;
+                        req->rpc_status = MSG_DENIED;
+                        req->rpc_err = AUTH_ERROR;
+                        req->auth_err = RPCSVC_AUTH_REJECT;
+                        goto err_reply;
         }
 
         /* DRC */
@@ -627,37 +647,39 @@ rpcsvc_handle_rpc_call (rpcsvc_t *svc, rpc_transport_t *trans,
                 drc = req->svc->drc;
 
                 LOCK (&drc->lock);
-                reply = rpcsvc_drc_lookup (req);
+                {
+                        reply = rpcsvc_drc_lookup (req);
 
-                /* retransmission of completed request, send cached reply */
-                if (reply && reply->state == DRC_OP_CACHED) {
-                        gf_log (GF_RPCSVC, GF_LOG_INFO, "duplicate request:"
-                                " XID: 0x%x", req->xid);
-                        ret = rpcsvc_send_cached_reply (req, reply);
-                        drc->cache_hits++;
-                        UNLOCK (&drc->lock);
-                        goto out;
+                        /* retransmission of completed request, send cached reply */
+                        if (reply && reply->state == DRC_OP_CACHED) {
+                                gf_log (GF_RPCSVC, GF_LOG_INFO, "duplicate request:"
+                                        " XID: 0x%x", req->xid);
+                                ret = rpcsvc_send_cached_reply (req, reply);
+                                drc->cache_hits++;
+                                UNLOCK (&drc->lock);
+                                goto out;
 
-                } /* retransmitted request, original op in transit, drop it */
-                else if (reply && reply->state == DRC_OP_IN_TRANSIT) {
-                        gf_log (GF_RPCSVC, GF_LOG_INFO, "op in transit,"
-                                " discarding. XID: 0x%x", req->xid);
-                        ret = 0;
-                        drc->intransit_hits++;
-                        rpcsvc_request_destroy (req);
-                        UNLOCK (&drc->lock);
-                        goto out;
+                        } /* retransmitted request, original op in transit, drop it */
+                        else if (reply && reply->state == DRC_OP_IN_TRANSIT) {
+                                gf_log (GF_RPCSVC, GF_LOG_INFO, "op in transit,"
+                                        " discarding. XID: 0x%x", req->xid);
+                                ret = 0;
+                                drc->intransit_hits++;
+                                rpcsvc_request_destroy (req);
+                                UNLOCK (&drc->lock);
+                                goto out;
 
-                } /* fresh request, cache it as in-transit and proceed */
-                else {
-                        ret = rpcsvc_cache_request (req);
+                        } /* fresh request, cache it as in-transit and proceed */
+                        else {
+                                ret = rpcsvc_cache_request (req);
+                        }
                 }
                 UNLOCK (&drc->lock);
         }
 
         if (req->rpc_err == SUCCESS) {
                 /* Before going to xlator code, set the THIS properly */
-                THIS = svc->mydata;
+                THIS = svc->xl;
 
                 actor_fn = actor->actor;
 
@@ -852,15 +874,9 @@ err:
         return txrecord;
 }
 
-static inline int
-rpcsvc_get_callid (rpcsvc_t *rpc)
-{
-        return GF_UNIVERSAL_ANSWER;
-}
-
 int
 rpcsvc_fill_callback (int prognum, int progver, int procnum, int payload,
-                      uint64_t xid, struct rpc_msg *request)
+                      uint32_t xid, struct rpc_msg *request)
 {
         int   ret          = -1;
 
@@ -925,9 +941,9 @@ out:
         return txrecord;
 }
 
-struct iobuf *
+static struct iobuf *
 rpcsvc_callback_build_record (rpcsvc_t *rpc, int prognum, int progver,
-                              int procnum, size_t payload, uint64_t xid,
+                              int procnum, size_t payload, u_long xid,
                               struct iovec *recbuf)
 {
         struct rpc_msg           request     = {0, };
@@ -947,7 +963,7 @@ rpcsvc_callback_build_record (rpcsvc_t *rpc, int prognum, int progver,
                                     &request);
         if (ret == -1) {
                 gf_log ("rpcsvc", GF_LOG_WARNING, "cannot build a rpc-request "
-                        "xid (%"PRIu64")", xid);
+                        "xid (%" GF_PRI_RPC_XID ")", xid);
                 goto out;
         }
 
@@ -984,6 +1000,48 @@ out:
         return request_iob;
 }
 
+int rpcsvc_request_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
+                           rpcsvc_cbk_program_t *prog, int procnum,
+                           void *req, glusterfs_ctx_t *ctx,
+                           xdrproc_t xdrproc)
+{
+        int                     ret         = -1;
+        int                     count       = 0;
+        struct iovec            iov         = {0, };
+        struct iobuf            *iobuf      = NULL;
+        ssize_t                 xdr_size    = 0;
+
+        if (!req)
+                goto out;
+
+        xdr_size = xdr_sizeof (xdrproc, req);
+
+        iobuf = iobuf_get2 (ctx->iobuf_pool, xdr_size);
+        if (!iobuf)
+                goto out;
+
+        iov.iov_base = iobuf->ptr;
+        iov.iov_len  = iobuf_pagesize (iobuf);
+
+        ret = xdr_serialize_generic (iov, req, xdrproc);
+        if (ret == -1) {
+                gf_log (THIS->name, GF_LOG_WARNING,
+                        "failed to create XDR payload");
+                goto out;
+        }
+        iov.iov_len = ret;
+        count = 1;
+
+        ret = rpcsvc_callback_submit (rpc, trans, prog, procnum,
+                                      &iov, count);
+
+out:
+        if (iobuf)
+                iobuf_unref (iobuf);
+
+        return ret;
+}
+
 int
 rpcsvc_callback_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
                         rpcsvc_cbk_program_t *prog, int procnum,
@@ -994,7 +1052,6 @@ rpcsvc_callback_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
         rpc_transport_req_t    req;
         int                    ret         = -1;
         int                    proglen     = 0;
-        uint64_t               callid      = 0;
 
         if (!rpc) {
                 goto out;
@@ -1002,15 +1059,14 @@ rpcsvc_callback_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
 
         memset (&req, 0, sizeof (req));
 
-        callid = rpcsvc_get_callid (rpc);
-
         if (proghdr) {
                 proglen += iov_length (proghdr, proghdrcount);
         }
 
         request_iob = rpcsvc_callback_build_record (rpc, prog->prognum,
                                                     prog->progver, procnum,
-                                                    proglen, callid,
+                                                    proglen,
+                                                    GF_UNIVERSAL_ANSWER,
                                                     &rpchdr);
         if (!request_iob) {
                 gf_log ("rpcsvc", GF_LOG_WARNING,
@@ -1235,7 +1291,7 @@ rpcsvc_submit_generic (rpcsvc_request_t *req, struct iovec *proghdr,
         iobref_add (iobref, replyiob);
 
         /* cache the request in the duplicate request cache for appropriate ops */
-        if (req->reply) {
+        if ((req->reply) && (rpcsvc_need_drc (req))) {
                 drc = req->svc->drc;
 
                 LOCK (&drc->lock);
@@ -1255,14 +1311,14 @@ rpcsvc_submit_generic (rpcsvc_request_t *req, struct iovec *proghdr,
                         "rpc-transport (%s)", req->xid,
                         req->prog ? req->prog->progname : "(not matched)",
                         req->prog ? req->prog->progver : 0,
-                        req->procnum, trans->name);
+                        req->procnum, trans ? trans->name : "");
         } else {
                 gf_log (GF_RPCSVC, GF_LOG_TRACE,
                         "submitted reply for rpc-message (XID: 0x%x, "
                         "Program: %s, ProgVers: %d, Proc: %d) to rpc-transport "
                         "(%s)", req->xid, req->prog ? req->prog->progname: "-",
                         req->prog ? req->prog->progver : 0,
-                        req->procnum, trans->name);
+                        req->procnum, trans ? trans->name : "");
         }
 
 disconnect_exit:
@@ -1311,7 +1367,7 @@ rpcsvc_program_register_portmap (rpcsvc_program_t *newprog, uint32_t port)
         if (!(pmap_set (newprog->prognum, newprog->progver, IPPROTO_TCP,
                         port))) {
                 gf_log (GF_RPCSVC, GF_LOG_ERROR, "Could not register with"
-                        " portmap");
+                        " portmap %d %d %u", newprog->prognum, newprog->progver, port);
                 goto out;
         }
 
@@ -1490,9 +1546,15 @@ rpcsvc_program_unregister (rpcsvc_t *svc, rpcsvc_program_t *program)
         ret = 0;
 out:
         if (ret == -1) {
-                gf_log (GF_RPCSVC, GF_LOG_ERROR, "Program unregistration failed"
-                        ": %s, Num: %d, Ver: %d, Port: %d", program->progname,
-                        program->prognum, program->progver, program->progport);
+                if (program) {
+                        gf_log (GF_RPCSVC, GF_LOG_ERROR, "Program "
+                                "unregistration failed"
+                                ": %s, Num: %d, Ver: %d, Port: %d",
+                                program->progname, program->prognum,
+                                program->progver, program->progport);
+                } else {
+                        gf_log (GF_RPCSVC, GF_LOG_ERROR, "Program not found");
+                }
         }
 
         return ret;
@@ -1742,7 +1804,7 @@ rpcsvc_register_notify (rpcsvc_t *svc, rpcsvc_notify_t notify, void *mydata)
         if (!wrapper) {
                 goto out;
         }
-        svc->mydata   = mydata;  /* this_xlator */
+        svc->mydata   = mydata;
         wrapper->data = mydata;
         wrapper->notify = notify;
 
@@ -1865,6 +1927,32 @@ out:
         return ret;
 }
 
+static int
+rpcsvc_ping (rpcsvc_request_t *req)
+{
+        char          rsp_buf[8 * 1024] = {0,};
+        gf_common_rsp rsp               = {0,};
+        struct iovec  iov               = {0,};
+        int           ret               = -1;
+        uint32_t      ping_rsp_len      = 0;
+
+        ping_rsp_len = xdr_sizeof ((xdrproc_t) xdr_gf_common_rsp,
+                                   &rsp);
+
+        iov.iov_base = rsp_buf;
+        iov.iov_len  = ping_rsp_len;
+
+        ret = xdr_serialize_generic (iov, &rsp, (xdrproc_t)xdr_gf_common_rsp);
+        if (ret < 0) {
+                ret = RPCSVC_ACTOR_ERROR;
+        } else {
+                rsp.op_ret = 0;
+                rpcsvc_submit_generic (req, &iov, 1, NULL, 0, NULL);
+        }
+
+        return 0;
+}
+
 static int
 rpcsvc_dump (rpcsvc_request_t *req)
 {
@@ -1959,7 +2047,7 @@ rpcsvc_reconfigure_options (rpcsvc_t *svc, dict_t *options)
                 return (-1);
 
         /* Fetch the xlator from svc */
-        xlator = (xlator_t *) svc->mydata;
+        xlator = svc->xl;
         if (!xlator)
                 return (-1);
 
@@ -1973,13 +2061,14 @@ rpcsvc_reconfigure_options (rpcsvc_t *svc, dict_t *options)
                         return (-1);
                 }
 
-                /* key-string: rpc-auth.addr.<volname>.reject
+                /* key-string: rpc-auth.addr.<volname>.allow
                  *
                  * IMP: Delete the OLD key/value pair from dict.
                  * And set the NEW key/value pair IFF the option is SET
                  * in reconfigured volfile.
                  *
-                 * NB: No default value for reject key.
+                 * NB: If rpc-auth.addr.<volname>.allow is not SET explicitly,
+                 *     build_nfs_graph() sets it as "*" i.e. anonymous.
                  */
                 dict_del (svc->options, srchkey);
                 if (!dict_get_str (options, srchkey, &keyval)) {
@@ -2006,14 +2095,13 @@ rpcsvc_reconfigure_options (rpcsvc_t *svc, dict_t *options)
                         return (-1);
                 }
 
-                /* key-string: rpc-auth.addr.<volname>.allow
+                /* key-string: rpc-auth.addr.<volname>.reject
                  *
                  * IMP: Delete the OLD key/value pair from dict.
                  * And set the NEW key/value pair IFF the option is SET
                  * in reconfigured volfile.
                  *
-                 * NB: If rpc-auth.addr.<volname>.allow is not SET explicitly,
-                 *     build_nfs_graph() sets it as "*" i.e. anonymous.
+                 * NB: No default value for reject key.
                  */
                 dict_del (svc->options, srchkey);
                 if (!dict_get_str (options, srchkey, &keyval)) {
@@ -2130,6 +2218,52 @@ rpcsvc_set_outstanding_rpc_limit (rpcsvc_t *svc, dict_t *options, int defvalue)
         return (0);
 }
 
+/*
+ * Enable throttling for rpcsvc_t svc.
+ * Returns 0 on success, -1 otherwise.
+ */
+int
+rpcsvc_set_throttle_on (rpcsvc_t *svc)
+{
+
+        if (!svc)
+                return -1;
+
+        svc->throttle = _gf_true;
+
+        return 0;
+}
+
+/*
+ * Disable throttling for rpcsvc_t svc.
+ * Returns 0 on success, -1 otherwise.
+ */
+int
+rpcsvc_set_throttle_off (rpcsvc_t *svc)
+{
+
+        if (!svc)
+                return -1;
+
+        svc->throttle = _gf_false;
+
+        return 0;
+}
+
+/*
+ * Get throttle state for rpcsvc_t svc.
+ * Returns value of attribute throttle on success, _gf_false otherwise.
+ */
+gf_boolean_t
+rpcsvc_get_throttle (rpcsvc_t *svc)
+{
+
+        if (!svc)
+                return _gf_false;
+
+        return svc->throttle;
+}
+
 /* The global RPC service initializer.
  */
 rpcsvc_t *
@@ -2179,7 +2313,7 @@ rpcsvc_init (xlator_t *xl, glusterfs_ctx_t *ctx, dict_t *options,
         ret = -1;
         svc->options = options;
         svc->ctx = ctx;
-        svc->mydata = xl;
+        svc->xl = xl;
         gf_log (GF_RPCSVC, GF_LOG_DEBUG, "RPC service inited.");
 
         gluster_dump_prog.options = options;
@@ -2349,21 +2483,18 @@ rpcsvc_combine_allow_reject_volume_check (int allow, int reject)
 }
 
 int
-rpcsvc_auth_check (rpcsvc_t *svc, char *volname,
-                   rpc_transport_t *trans)
+rpcsvc_auth_check (rpcsvc_t *svc, char *volname, char *ipaddr)
 {
         int     ret                            = RPCSVC_AUTH_REJECT;
         int     accept                         = RPCSVC_AUTH_REJECT;
         int     reject                         = RPCSVC_AUTH_REJECT;
         char   *hostname                       = NULL;
-        char   *ip                             = NULL;
-        char    client_ip[RPCSVC_PEER_STRLEN]  = {0};
         char   *allow_str                      = NULL;
         char   *reject_str                     = NULL;
         char   *srchstr                        = NULL;
         dict_t *options                        = NULL;
 
-        if (!svc || !volname || !trans)
+        if (!svc || !volname || !ipaddr)
                 return ret;
 
         /* Fetch the options from svc struct and validate */
@@ -2371,13 +2502,6 @@ rpcsvc_auth_check (rpcsvc_t *svc, char *volname,
         if (!options)
                 return ret;
 
-        ret = rpcsvc_transport_peername (trans, client_ip, RPCSVC_PEER_STRLEN);
-        if (ret != 0) {
-                gf_log (GF_RPCSVC, GF_LOG_ERROR, "Failed to get remote addr: "
-                        "%s", gai_strerror (ret));
-                return RPCSVC_AUTH_REJECT;
-        }
-
         /* Accept if its the default case: Allow all, Reject none
          * The default volfile always contains a 'allow *' rule
          * for each volume. If allow rule is missing (which implies
@@ -2418,13 +2542,9 @@ rpcsvc_auth_check (rpcsvc_t *svc, char *volname,
                         return RPCSVC_AUTH_ACCEPT;
         }
 
-        /* Non-default rule, authenticate */
-        if (!get_host_name (client_ip, &ip))
-                ip = client_ip;
-
         /* addr-namelookup check */
         if (svc->addr_namelookup == _gf_true) {
-                ret = gf_get_hostname_from_ip (ip, &hostname);
+                ret = gf_get_hostname_from_ip (ipaddr, &hostname);
                 if (ret) {
                         if (hostname)
                                 GF_FREE (hostname);
@@ -2437,10 +2557,10 @@ rpcsvc_auth_check (rpcsvc_t *svc, char *volname,
         }
 
         accept = rpcsvc_transport_peer_check_allow (options, volname,
-                                                    ip, hostname);
+                                                    ipaddr, hostname);
 
         reject = rpcsvc_transport_peer_check_reject (options, volname,
-                                                     ip, hostname);
+                                                     ipaddr, hostname);
 
         if (hostname)
                 GF_FREE (hostname);
@@ -2448,32 +2568,16 @@ rpcsvc_auth_check (rpcsvc_t *svc, char *volname,
 }
 
 int
-rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
-                                 rpc_transport_t *trans)
+rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname, uint16_t port)
 {
-        union gf_sock_union     sock_union;
         int                     ret = RPCSVC_AUTH_REJECT;
-        socklen_t               sinsize = sizeof (&sock_union.sin);
         char                    *srchstr = NULL;
         char                    *valstr = NULL;
-        uint16_t                port = 0;
         gf_boolean_t            insecure = _gf_false;
 
-        memset (&sock_union, 0, sizeof (sock_union));
-
-        if ((!svc) || (!volname) || (!trans))
+        if ((!svc) || (!volname))
                 return ret;
 
-        ret = rpcsvc_transport_peeraddr (trans, NULL, 0, &sock_union.storage,
-                                         sinsize);
-        if (ret != 0) {
-                gf_log (GF_RPCSVC, GF_LOG_ERROR, "Failed to get peer addr: %s",
-                        gai_strerror (ret));
-                ret = RPCSVC_AUTH_REJECT;
-                goto err;
-        }
-
-        port = ntohs (sock_union.sin.sin_port);
         gf_log (GF_RPCSVC, GF_LOG_TRACE, "Client port: %d", (int)port);
         /* If the port is already a privileged one, dont bother with checking
          * options.
@@ -2600,7 +2704,6 @@ rpcsvc_match_subnet_v4 (const char *addrtok, const char *ipaddr)
                         mask.sin_addr.s_addr)) {
                 ret = 0; /* SUCCESS */
         }
-
 out:
         GF_FREE (netaddr);
         return ret;
@@ -2610,6 +2713,7 @@ out:
 rpcsvc_actor_t gluster_dump_actors[GF_DUMP_MAXVALUE] = {
         [GF_DUMP_NULL]      = {"NULL",     GF_DUMP_NULL,     NULL,        NULL, 0, DRC_NA},
         [GF_DUMP_DUMP]      = {"DUMP",     GF_DUMP_DUMP,     rpcsvc_dump, NULL, 0, DRC_NA},
+        [GF_DUMP_PING]      = {"PING",     GF_DUMP_PING,     rpcsvc_ping, NULL, 0, DRC_NA},
 };
 
 
