@@ -30,6 +30,10 @@ enum gf_dht_mem_types_ {
         gf_defrag_info_mt,
         gf_dht_mt_inode_ctx_t,
         gf_dht_mt_ctx_stat_time_t,
+        gf_dht_mt_dirent_t,
+        gf_dht_mt_container_t,
+        gf_dht_mt_octx_t,
+        gf_dht_mt_miginfo_t,
         gf_dht_mt_end
 };
 #endif
