@@ -15,9 +15,34 @@
 #endif
 
 /* TODO: add NS locking */
-
 #include "statedump.h"
 #include "dht-common.h"
+#include "dht-messages.h"
+
+#ifndef MAX
+#define MAX(a, b) (((a) > (b))?(a):(b))
+#endif
+
+#define GF_DECIDE_DEFRAG_THROTTLE_COUNT(throttle_count, conf) {         \
+                                                                        \
+                pthread_mutex_lock (&conf->defrag->dfq_mutex);          \
+                                                                        \
+                if (!strcasecmp (conf->dthrottle, "lazy"))              \
+                        conf->defrag->recon_thread_count = 1;           \
+                                                                        \
+                throttle_count =                                        \
+                    MAX ((sysconf(_SC_NPROCESSORS_ONLN) - 4), 4);       \
+                                                                        \
+                if (!strcasecmp (conf->dthrottle, "normal"))            \
+                        conf->defrag->recon_thread_count =              \
+                                                 (throttle_count / 2);  \
+                                                                        \
+                if (!strcasecmp (conf->dthrottle, "aggressive"))        \
+                        conf->defrag->recon_thread_count =              \
+                                                 throttle_count;        \
+                                                                        \
+                pthread_mutex_unlock (&conf->defrag->dfq_mutex);        \
+        }                                                               \
 
 /* TODO:
    - use volumename in xattr instead of "dht"
@@ -27,6 +52,8 @@
 */
 struct volume_options options[];
 
+extern dht_methods_t dht_methods;
+
 void
 dht_layout_dump (dht_layout_t  *layout, const char *prefix)
 {
@@ -103,20 +130,20 @@ dht_priv_dump (xlator_t *this)
                                this->name);
         gf_proc_dump_write("subvol_cnt","%d", conf->subvolume_cnt);
         for (i = 0; i < conf->subvolume_cnt; i++) {
-                sprintf (key, "subvolumes[%d]", i);
+                snprintf (key, sizeof (key), "subvolumes[%d]", i);
                 gf_proc_dump_write(key, "%s.%s", conf->subvolumes[i]->type,
                                    conf->subvolumes[i]->name);
                 if (conf->file_layouts && conf->file_layouts[i]){
-                        sprintf (key, "file_layouts[%d]", i);
+                        snprintf (key, sizeof (key), "file_layouts[%d]", i);
                         dht_layout_dump(conf->file_layouts[i], key);
                 }
                 if (conf->dir_layouts && conf->dir_layouts[i]) {
-                        sprintf (key, "dir_layouts[%d]", i);
+                        snprintf (key, sizeof (key), "dir_layouts[%d]", i);
                         dht_layout_dump(conf->dir_layouts[i], key);
                 }
                 if (conf->subvolume_status) {
 
-                        sprintf (key, "subvolume_status[%d]", i);
+                        snprintf (key, sizeof (key), "subvolume_status[%d]", i);
                         gf_proc_dump_write(key, "%d",
                                            (int)conf->subvolume_status[i]);
                 }
@@ -130,14 +157,35 @@ dht_priv_dump (xlator_t *this)
         gf_proc_dump_write("disk_unit", "%c", conf->disk_unit);
         gf_proc_dump_write("refresh_interval", "%d", conf->refresh_interval);
         gf_proc_dump_write("unhashed_sticky_bit", "%d", conf->unhashed_sticky_bit);
-        if (conf ->du_stats) {
-                gf_proc_dump_write("du_stats.avail_percent", "%lf",
-                                   conf->du_stats->avail_percent);
-                gf_proc_dump_write("du_stats.avail_space", "%lu",
-                                   conf->du_stats->avail_space);
-		gf_proc_dump_write("du_stats.avail_inodes", "%lf",
-                                   conf->du_stats->avail_inodes);
-                gf_proc_dump_write("du_stats.log", "%lu", conf->du_stats->log);
+
+        if (conf->du_stats) {
+                for (i = 0; i < conf->subvolume_cnt; i++) {
+                        if (!conf->subvolume_status[i])
+                                continue;
+
+                        snprintf (key, sizeof (key), "subvolumes[%d]", i);
+                        gf_proc_dump_write (key, "%s",
+                                            conf->subvolumes[i]->name);
+
+                        snprintf (key, sizeof (key),
+                                  "du_stats[%d].avail_percent", i);
+                        gf_proc_dump_write (key, "%lf",
+                                            conf->du_stats[i].avail_percent);
+
+                        snprintf (key, sizeof (key), "du_stats[%d].avail_space",
+                                  i);
+                        gf_proc_dump_write (key, "%lu",
+                                            conf->du_stats[i].avail_space);
+
+                        snprintf (key, sizeof (key),
+                                  "du_stats[%d].avail_inodes", i);
+                        gf_proc_dump_write (key, "%lf",
+                                            conf->du_stats[i].avail_inodes);
+
+                        snprintf (key, sizeof (key), "du_stats[%d].log", i);
+                        gf_proc_dump_write (key, "%lu",
+                                            conf->du_stats[i].log);
+                }
         }
 
         if (conf->last_stat_fetch.tv_sec)
@@ -191,10 +239,15 @@ dht_fini (xlator_t *this)
                         GF_FREE (conf->file_layouts);
                 }
 
+                dict_destroy(conf->leaf_to_subvol);
+
                 GF_FREE (conf->subvolumes);
 
                 GF_FREE (conf->subvolume_status);
 
+                if (conf->lock_pool)
+                        mem_pool_destroy (conf->lock_pool);
+
                 GF_FREE (conf);
         }
 out:
@@ -211,8 +264,9 @@ mem_acct_init (xlator_t *this)
         ret = xlator_mem_acct_init (this, gf_dht_mt_end + 1);
 
         if (ret != 0) {
-                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
-                        "failed");
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        DHT_MSG_NO_MEMORY,
+                        "Memory accounting init failed");
                 return ret;
         }
 out:
@@ -241,7 +295,8 @@ dht_parse_decommissioned_bricks (xlator_t *this, dht_conf_t *conf,
                                 conf->decommissioned_bricks[i] =
                                         conf->subvolumes[i];
                                         conf->decommission_subvols_cnt++;
-                                gf_log (this->name, GF_LOG_INFO,
+                                gf_msg (this->name, GF_LOG_INFO, 0,
+                                        DHT_MSG_SUBVOL_DECOMMISSION_INFO,
                                         "decommissioning subvolume %s",
                                         conf->subvolumes[i]->name);
                                 break;
@@ -262,7 +317,6 @@ out:
         return ret;
 }
 
-
 int
 dht_decommissioned_remove (xlator_t *this, dht_conf_t *conf)
 {
@@ -307,16 +361,38 @@ dht_init_regex (xlator_t *this, dict_t *odict, char *name,
         }
 
         if (regcomp(re,temp_str,REG_EXTENDED) == 0) {
-                gf_log (this->name, GF_LOG_INFO,
-                        "using regex %s = %s", name, temp_str);
+                gf_msg_debug (this->name, 0,
+                              "using regex %s = %s", name, temp_str);
                 *re_valid = _gf_true;
         }
         else {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_REGEX_INFO,
                         "compiling regex %s failed", temp_str);
         }
 }
 
+int
+dht_set_subvol_range(xlator_t *this)
+{
+        int ret = -1;
+        dht_conf_t *conf = NULL;
+
+        conf = this->private;
+
+        if (!conf)
+                goto out;
+
+        conf->leaf_to_subvol = dict_new();
+        if (!conf->leaf_to_subvol)
+                goto out;
+
+        ret = glusterfs_reachable_leaves(this, conf->leaf_to_subvol);
+
+out:
+        return ret;
+}
+
 int
 dht_reconfigure (xlator_t *this, dict_t *options)
 {
@@ -324,6 +400,7 @@ dht_reconfigure (xlator_t *this, dict_t *options)
         char            *temp_str = NULL;
         gf_boolean_t     search_unhashed;
         int              ret = -1;
+        int              throttle_count = 0;
 
         GF_VALIDATE_OR_GOTO ("dht", this, out);
         GF_VALIDATE_OR_GOTO ("dht", options, out);
@@ -336,26 +413,30 @@ dht_reconfigure (xlator_t *this, dict_t *options)
                 /* If option is not "auto", other options _should_ be boolean*/
                 if (strcasecmp (temp_str, "auto")) {
                         if (!gf_string2boolean (temp_str, &search_unhashed)) {
-                                gf_log(this->name, GF_LOG_DEBUG, "Reconfigure:"
-                                       " lookup-unhashed reconfigured (%s)",
-                                       temp_str);
+                                gf_msg_debug(this->name, 0, "Reconfigure: "
+                                             "lookup-unhashed reconfigured(%s)",
+                                             temp_str);
                                 conf->search_unhashed = search_unhashed;
                         } else {
-                                gf_log(this->name, GF_LOG_ERROR, "Reconfigure:"
-                                       " lookup-unhashed should be boolean,"
+                                gf_msg(this->name, GF_LOG_ERROR, 0,
+                                       DHT_MSG_INVALID_OPTION,
+                                       "Invalid option: Reconfigure: "
+                                       "lookup-unhashed should be boolean,"
                                        " not (%s), defaulting to (%d)",
                                        temp_str, conf->search_unhashed);
-                                //return -1;
                                 ret = -1;
                                 goto out;
                         }
                 } else {
-                        gf_log(this->name, GF_LOG_DEBUG, "Reconfigure:"
-                               " lookup-unhashed reconfigured auto ");
+                        gf_msg_debug(this->name, 0, "Reconfigure:"
+                                     " lookup-unhashed reconfigured auto ");
                         conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
                 }
         }
 
+        GF_OPTION_RECONF ("lookup-optimize", conf->lookup_optimize, options,
+                          bool, out);
+
 	GF_OPTION_RECONF ("min-free-disk", conf->min_free_disk, options,
                           percent_or_size, out);
         /* option can be any one of percent or bytes */
@@ -371,6 +452,22 @@ dht_reconfigure (xlator_t *this, dict_t *options)
 
         GF_OPTION_RECONF ("readdir-optimize", conf->readdir_optimize, options,
                           bool, out);
+        GF_OPTION_RECONF ("randomize-hash-range-by-gfid",
+                          conf->randomize_by_gfid,
+                          options, bool, out);
+
+        GF_OPTION_RECONF ("rebal-throttle", conf->dthrottle, options,
+                          str, out);
+
+        if (conf->defrag) {
+                GF_DECIDE_DEFRAG_THROTTLE_COUNT (throttle_count, conf);
+                gf_msg ("DHT", GF_LOG_INFO, 0,
+                        DHT_MSG_REBAL_THROTTLE_INFO,
+                        "conf->dthrottle: %s, "
+                        "conf->defrag->recon_thread_count: %d",
+                         conf->dthrottle, conf->defrag->recon_thread_count);
+        }
+
         if (conf->defrag) {
                 GF_OPTION_RECONF ("rebalance-stats", conf->defrag->stats,
                                   options, bool, out);
@@ -391,6 +488,9 @@ dht_reconfigure (xlator_t *this, dict_t *options)
         dht_init_regex (this, options, "extra-hash-regex",
                         &conf->extra_regex, &conf->extra_regex_valid);
 
+        GF_OPTION_RECONF ("weighted-rebalance", conf->do_weighting, options,
+                          bool, out);
+
         ret = 0;
 out:
         return ret;
@@ -428,13 +528,15 @@ gf_defrag_pattern_list_fill (xlator_t *this, gf_defrag_info_t *defrag, char *dat
                 if (!pattern)
                         goto out;
                 if (!num) {
-                        if (gf_string2bytesize(pattern, &pattern_list->size)
+                        if (gf_string2bytesize_uint64(pattern, &pattern_list->size)
                              == 0) {
                                 pattern = "*";
                         }
-                } else if (gf_string2bytesize (num, &pattern_list->size) != 0) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "invalid number format \"%s\"", num);
+                } else if (gf_string2bytesize_uint64 (num, &pattern_list->size) != 0) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                DHT_MSG_INVALID_OPTION,
+                                "Invalid option. Defrag pattern:"
+                                " Invalid number format \"%s\"", num);
                         goto out;
                 }
                 memcpy (pattern_list->path_pattern, pattern, strlen (dup_str));
@@ -474,18 +576,21 @@ dht_init (xlator_t *this)
         gf_defrag_info_t                *defrag         = NULL;
         int                              cmd            = 0;
         char                            *node_uuid      = NULL;
-
+        int                              throttle_count = 0;
+        uint32_t                         commit_hash    = 0;
 
         GF_VALIDATE_OR_GOTO ("dht", this, err);
 
         if (!this->children) {
-                gf_log (this->name, GF_LOG_CRITICAL,
+                gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        DHT_MSG_INVALID_CONFIGURATION,
                         "Distribute needs more than one subvolume");
                 return -1;
         }
 
         if (!this->parents) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_INVALID_CONFIGURATION,
                         "dangling volume. check volfile");
         }
 
@@ -494,6 +599,16 @@ dht_init (xlator_t *this)
                 goto err;
         }
 
+        /* We get the commit-hash to set only for rebalance process */
+        if (dict_get_uint32 (this->options,
+                             "commit-hash", &commit_hash) == 0) {
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        DHT_MSG_COMMIT_HASH_INFO, "%s using commit hash %u",
+                        __func__, commit_hash);
+                conf->vol_commit_hash = commit_hash;
+                conf->vch_forced = _gf_true;
+        }
+
         ret = dict_get_int32 (this->options, "rebalance-cmd", &cmd);
 
         if (cmd) {
@@ -510,31 +625,59 @@ dht_init (xlator_t *this)
 
                 ret = dict_get_str (this->options, "node-uuid", &node_uuid);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_ERROR, "node-uuid not "
-                                "specified");
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                DHT_MSG_INVALID_CONFIGURATION,
+                                "Invalid volume configuration: "
+                                "node-uuid not specified");
                         goto err;
                 }
 
-                if (uuid_parse (node_uuid, defrag->node_uuid)) {
-                        gf_log (this->name, GF_LOG_ERROR, "Cannot parse "
-                                "glusterd node uuid");
+                if (gf_uuid_parse (node_uuid, defrag->node_uuid)) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                DHT_MSG_INVALID_OPTION, "Invalid option:"
+                                " Cannot parse glusterd node uuid");
                         goto err;
                 }
 
                 defrag->cmd = cmd;
 
                 defrag->stats = _gf_false;
+
+                defrag->queue = NULL;
+
+                defrag->crawl_done = 0;
+
+                defrag->global_error = 0;
+
+                defrag->q_entry_count = 0;
+
+                defrag->wakeup_crawler = 0;
+
+                synclock_init (&defrag->link_lock, SYNC_LOCK_DEFAULT);
+                pthread_mutex_init (&defrag->dfq_mutex, 0);
+                pthread_cond_init  (&defrag->parallel_migration_cond, 0);
+                pthread_cond_init  (&defrag->rebalance_crawler_alarm, 0);
+                pthread_cond_init  (&defrag->df_wakeup_thread, 0);
+
+                defrag->global_error = 0;
+
         }
 
         conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_ON;
         if (dict_get_str (this->options, "lookup-unhashed", &temp_str) == 0) {
                 /* If option is not "auto", other options _should_ be boolean */
-                if (strcasecmp (temp_str, "auto"))
-                        gf_string2boolean (temp_str, &conf->search_unhashed);
+                if (strcasecmp (temp_str, "auto")) {
+                        ret = gf_string2boolean (temp_str,
+                                                 &conf->search_unhashed);
+                        if (ret == -1)
+                                goto err;
+                }
                 else
                         conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
         }
 
+        GF_OPTION_INIT ("lookup-optimize", conf->lookup_optimize, bool, err);
+
         GF_OPTION_INIT ("unhashed-sticky-bit", conf->unhashed_sticky_bit, bool,
                         err);
 
@@ -561,8 +704,12 @@ dht_init (xlator_t *this)
                     == 0) {
                         if (gf_defrag_pattern_list_fill (this, defrag, temp_str)
                             == -1) {
-                                gf_log (this->name, GF_LOG_ERROR, "Cannot parse"
-                                        " rebalance-filter (%s)", temp_str);
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        DHT_MSG_INVALID_OPTION,
+                                        "Invalid option:"
+                                        " Cannot parse rebalance-filter (%s)",
+                                        temp_str);
+
                                 goto err;
                         }
                 }
@@ -578,6 +725,16 @@ dht_init (xlator_t *this)
                 goto err;
         }
 
+        if (cmd) {
+                ret = dht_init_local_subvolumes (this, conf);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                DHT_MSG_INIT_LOCAL_SUBVOL_FAILED,
+                                "dht_init_local_subvolumes failed");
+                        goto err;
+                }
+        }
+
         if (dict_get_str (this->options, "decommissioned-bricks", &temp_str) == 0) {
                 ret = dht_parse_decommissioned_bricks (this, conf, temp_str);
                 if (ret == -1)
@@ -601,21 +758,55 @@ dht_init (xlator_t *this)
 
         this->local_pool = mem_pool_new (dht_local_t, 512);
         if (!this->local_pool) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                        DHT_MSG_NO_MEMORY,
+                        " DHT initialisation failed. "
                         "failed to create local_t's memory pool");
                 goto err;
         }
 
+        GF_OPTION_INIT ("randomize-hash-range-by-gfid",
+                        conf->randomize_by_gfid, bool, err);
+
+        if (defrag) {
+                GF_OPTION_INIT ("rebal-throttle",
+                                 conf->dthrottle, str, err);
+
+                GF_DECIDE_DEFRAG_THROTTLE_COUNT(throttle_count, conf);
+
+                gf_msg_debug ("DHT", 0, "conf->dthrottle: %s, "
+                              "conf->defrag->recon_thread_count: %d",
+                              conf->dthrottle,
+                              conf->defrag->recon_thread_count);
+        }
+
         GF_OPTION_INIT ("xattr-name", conf->xattr_name, str, err);
         gf_asprintf (&conf->link_xattr_name, "%s."DHT_LINKFILE_STR,
                      conf->xattr_name);
+        gf_asprintf (&conf->commithash_xattr_name, "%s."DHT_COMMITHASH_STR,
+                     conf->xattr_name);
         gf_asprintf (&conf->wild_xattr_name, "%s*", conf->xattr_name);
         if (!conf->link_xattr_name || !conf->wild_xattr_name) {
                 goto err;
         }
 
+        GF_OPTION_INIT ("weighted-rebalance", conf->do_weighting, bool, err);
+
+        conf->lock_pool = mem_pool_new (dht_lock_t, 512);
+        if (!conf->lock_pool) {
+                gf_msg (this->name, GF_LOG_ERROR, 0, DHT_MSG_INIT_FAILED,
+                        "failed to create lock mem_pool, failing "
+                        "initialization");
+                goto err;
+        }
+
         this->private = conf;
 
+        if (dht_set_subvol_range(this))
+                goto err;
+
+        conf->methods = &dht_methods;
+
         return 0;
 
 err:
@@ -639,6 +830,9 @@ err:
                 GF_FREE (conf->link_xattr_name);
                 GF_FREE (conf->wild_xattr_name);
 
+                if (conf->lock_pool)
+                        mem_pool_destroy (conf->lock_pool);
+
                 GF_FREE (conf);
         }
 
@@ -657,6 +851,14 @@ struct volume_options options[] = {
           "from the hash subvolume. If set to OFF, it does not do a lookup "
           "on the remaining subvolumes."
         },
+        { .key = {"lookup-optimize"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "off",
+          .description = "This option if set to ON enables the optimization "
+          "of -ve lookups, by not doing a lookup on non-hashed subvolumes for "
+          "files, in case the hashed subvolume does not return any result. "
+          "This option disregards the lookup-unhashed setting, when enabled."
+        },
         { .key  = {"min-free-disk"},
           .type = GF_OPTION_TYPE_PERCENT_OR_SIZET,
           .default_value = "10%",
@@ -702,6 +904,9 @@ struct volume_options options[] = {
         { .key  = {"rebalance-cmd"},
           .type = GF_OPTION_TYPE_INT,
         },
+        { .key = {"commit-hash"},
+          .type = GF_OPTION_TYPE_INT,
+        },
         { .key = {"node-uuid"},
           .type = GF_OPTION_TYPE_STR,
         },
@@ -746,15 +951,71 @@ struct volume_options options[] = {
           "below it."
         },
 
+        { .key = {"weighted-rebalance"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "on",
+          .description = "When enabled, files will be allocated to bricks "
+          "with a probability proportional to their size.  Otherwise, all "
+          "bricks will have the same probability (legacy behavior)."
+        },
+
         /* NUFA option */
         { .key  = {"local-volume-name"},
           .type = GF_OPTION_TYPE_XLATOR
         },
 
+        /* tier options */
+        { .key  = {"tier-promote-frequency"},
+          .type = GF_OPTION_TYPE_INT,
+          .default_value = "120",
+          .description = "Frequency to promote files to fast tier"
+        },
+
+        { .key  = {"tier-demote-frequency"},
+          .type = GF_OPTION_TYPE_INT,
+          .default_value = "120",
+          .description = "Frequency to demote files to slow tier"
+        },
+
+        { .key  = {"write-freq-threshold"},
+          .type = GF_OPTION_TYPE_INT,
+          .default_value = "0",
+          .description = "Defines the write fequency "
+                        "that would be considered hot"
+        },
+
+        { .key  = {"read-freq-threshold"},
+          .type = GF_OPTION_TYPE_INT,
+          .default_value = "0",
+          .description = "Defines the read fequency "
+                        "that would be considered hot"
+        },
+
         /* switch option */
         { .key  = {"pattern.switch.case"},
           .type = GF_OPTION_TYPE_ANY
         },
 
+        { .key =  {"randomize-hash-range-by-gfid"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "off",
+          .description = "Use gfid of directory to determine the subvolume "
+          "from which hash ranges are allocated starting with 0. "
+          "Note that we still use a directory/file's name to determine the "
+          "subvolume to which it hashes"
+        },
+
+        { .key =  {"rebal-throttle"},
+          .type = GF_OPTION_TYPE_STR,
+          .default_value = "normal",
+          .description = " Sets the maximum number of parallel file migrations "
+                         "allowed on a node during the rebalance operation. The"
+                         " default value is normal and allows a max of "
+                         "[($(processing units) - 4) / 2), 2]  files to be "
+                         "migrated at a time. Lazy will allow only one file to "
+                         "be migrated at a time and aggressive will allow "
+                         "max of [($(processing units) - 4) / 2), 4]"
+        },
+
         { .key  = {NULL} },
 };
