@@ -44,6 +44,7 @@
 #endif
 
 #include "quota.h"
+#include "quota-messages.h"
 
 extern struct rpc_clnt_program quota_enforcer_clnt;
 
@@ -120,22 +121,26 @@ int
 quota_enforcer_lookup_cbk (struct rpc_req *req, struct iovec *iov,
                            int count, void *myframe)
 {
-        quota_local_t    *local      = NULL;
-        call_frame_t     *frame      = NULL;
-        int               ret        = 0;
-        gfs3_lookup_rsp   rsp        = {0,};
-        struct iatt       stbuf      = {0,};
-        struct iatt       postparent = {0,};
-        int               op_errno   = EINVAL;
-        dict_t           *xdata      = NULL;
-        inode_t          *inode      = NULL;
-        xlator_t         *this       = NULL;
+        quota_local_t    *local       = NULL;
+        call_frame_t     *frame       = NULL;
+        int               ret         = 0;
+        gfs3_lookup_rsp   rsp         = {0,};
+        struct iatt       stbuf       = {0,};
+        struct iatt       postparent  = {0,};
+        int               op_errno    = EINVAL;
+        dict_t           *xdata       = NULL;
+        inode_t          *inode       = NULL;
+        xlator_t         *this        = NULL;
+        quota_priv_t     *priv        = NULL;
+        struct timespec   retry_delay = {0,};
+        gf_timer_t       *timer       = NULL;
 
         this = THIS;
 
         frame = myframe;
         local = frame->local;
         inode = local->validate_loc.inode;
+        priv  = this->private;
 
         if (-1 == req->rpc_status) {
                 rsp.op_ret   = -1;
@@ -145,7 +150,9 @@ quota_enforcer_lookup_cbk (struct rpc_req *req, struct iovec *iov,
 
         ret = xdr_to_generic (*iov, &rsp, (xdrproc_t)xdr_gfs3_lookup_rsp);
         if (ret < 0) {
-                gf_log (this->name, GF_LOG_ERROR, "XDR decoding failed");
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        Q_MSG_XDR_DECODING_FAILED,
+                        "XDR decoding failed");
                 rsp.op_ret   = -1;
                 op_errno = EINVAL;
                 goto out;
@@ -164,10 +171,10 @@ quota_enforcer_lookup_cbk (struct rpc_req *req, struct iovec *iov,
                                       (rsp.xdata.xdata_len), rsp.op_ret,
                                       op_errno, out);
 
-        if ((!uuid_is_null (inode->gfid))
-            && (uuid_compare (stbuf.ia_gfid, inode->gfid) != 0)) {
-                gf_log (frame->this->name, GF_LOG_DEBUG,
-                        "gfid changed for %s", local->validate_loc.path);
+        if ((!gf_uuid_is_null (inode->gfid))
+            && (gf_uuid_compare (stbuf.ia_gfid, inode->gfid) != 0)) {
+                gf_msg_debug (frame->this->name, ESTALE,
+                              "gfid changed for %s", local->validate_loc.path);
                 rsp.op_ret = -1;
                 op_errno = ESTALE;
                 goto out;
@@ -177,23 +184,69 @@ quota_enforcer_lookup_cbk (struct rpc_req *req, struct iovec *iov,
 
 out:
         rsp.op_errno = op_errno;
+
+        /* We need to retry connecting to quotad on ENOTCONN error.
+         * Suppose if there are two volumes vol1 and vol2,
+         * and quota is enabled and limit is set on vol1.
+         * Now if IO is happening on vol1 and quota is enabled/disabled
+         * on vol2, quotad gets restarted and client will receive
+         * ENOTCONN in the IO path of vol1
+         */
+        if (rsp.op_ret == -1 && rsp.op_errno == ENOTCONN) {
+                if (local->quotad_conn_retry >= 12) {
+                        priv->quotad_conn_status = 1;
+                        gf_log (this->name, GF_LOG_WARNING, "failed to connect "
+                                "to quotad after retry count %d)",
+                                local->quotad_conn_retry);
+                } else {
+                        local->quotad_conn_retry++;
+                }
+
+                if (priv->quotad_conn_status == 0) {
+                        /* retry connecting after 5secs for 12 retries
+                         * (upto 60sec).
+                         */
+                        gf_log (this->name, GF_LOG_DEBUG, "retry connecting to "
+                                "quotad (retry count %d)",
+                                local->quotad_conn_retry);
+
+                        retry_delay.tv_sec = 5;
+                        retry_delay.tv_nsec = 0;
+                        timer = gf_timer_call_after (this->ctx, retry_delay,
+                                                     _quota_enforcer_lookup,
+                                                     (void *) frame);
+                        if (timer == NULL) {
+                                gf_log (this->name, GF_LOG_WARNING, "failed to "
+                                        "set quota_enforcer_lookup with timer");
+                        } else {
+                                goto clean;
+                        }
+                }
+        } else {
+                priv->quotad_conn_status = 0;
+        }
+
         if (rsp.op_ret == -1) {
                 /* any error other than ENOENT */
                 if (rsp.op_errno != ENOENT)
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "remote operation failed: %s. Path: %s (%s)",
-                                strerror (rsp.op_errno),
-                                local->validate_loc.path,
+                        gf_msg (this->name, GF_LOG_WARNING, rsp.op_errno,
+                                Q_MSG_LOOKUP_FAILED,
+                                "Getting cluster-wide size of directory failed "
+                                "(path: %s gfid:%s)", local->validate_loc.path,
                                 loc_gfid_utoa (&local->validate_loc));
                 else
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "not found on remote node");
+                        gf_msg_trace (this->name, ENOENT,
+                                      "not found on remote node");
 
+        } else if (local->quotad_conn_retry) {
+                gf_log (this->name, GF_LOG_DEBUG, "connected to quotad after "
+                        "retry count %d", local->quotad_conn_retry);
         }
 
         local->validate_cbk (frame, NULL, this, rsp.op_ret, rsp.op_errno, inode,
                              &stbuf, xdata, &postparent);
 
+clean:
         if (xdata)
                 dict_unref (xdata);
 
@@ -202,34 +255,36 @@ out:
         return 0;
 }
 
-int
-quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
-                       dict_t *xdata, fop_lookup_cbk_t validate_cbk)
+void
+_quota_enforcer_lookup (void *data)
 {
         quota_local_t          *local      = NULL;
         gfs3_lookup_req         req        = {{0,},};
         int                     ret        = 0;
         int                     op_errno   = ESTALE;
         quota_priv_t           *priv       = NULL;
+        call_frame_t           *frame      = NULL;
+        loc_t                  *loc        = NULL;
+        xlator_t               *this       = NULL;
+        char                   *dir_path       = NULL;
 
-        if (!frame || !this || !loc)
-                goto unwind;
-
+        frame = data;
         local = frame->local;
-        local->validate_cbk = validate_cbk;
+        this  = local->this;
+        loc   = &local->validate_loc;
 
         priv = this->private;
 
         if (!(loc && loc->inode))
                 goto unwind;
 
-        if (!uuid_is_null (loc->inode->gfid))
+        if (!gf_uuid_is_null (loc->inode->gfid))
                 memcpy (req.gfid, loc->inode->gfid, 16);
         else
                 memcpy (req.gfid, loc->gfid, 16);
 
-        if (xdata) {
-                GF_PROTOCOL_DICT_SERIALIZE (this, xdata,
+        if (local->validate_xdata) {
+                GF_PROTOCOL_DICT_SERIALIZE (this, local->validate_xdata,
                                             (&req.xdata.xdata_val),
                                             req.xdata.xdata_len,
                                             op_errno, unwind);
@@ -240,6 +295,11 @@ quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
         else
                 req.bname = "";
 
+        if (loc->path)
+                dir_path = (char *)loc->path;
+        else
+                dir_path = "";
+
         ret = quota_enforcer_submit_request (&req, frame,
                                              priv->quota_enforcer,
                                              GF_AGGREGATOR_LOOKUP,
@@ -248,17 +308,45 @@ quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
                                              (xdrproc_t)xdr_gfs3_lookup_req);
 
         if (ret) {
-                gf_log (this->name, GF_LOG_WARNING, "failed to send the fop");
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+			Q_MSG_RPC_SUBMIT_FAILED, "Couldn't send the request to "
+                        "fetch cluster wide size of directory (path:%s gfid:%s)"
+                        , dir_path, req.gfid);
         }
 
         GF_FREE (req.xdata.xdata_val);
 
+        return;
+
+unwind:
+        local->validate_cbk (frame, NULL, this, -1, op_errno, NULL, NULL, NULL,
+                             NULL);
+
+        GF_FREE (req.xdata.xdata_val);
+
+        return;
+}
+
+int
+quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, dict_t *xdata,
+                       fop_lookup_cbk_t validate_cbk)
+{
+        quota_local_t          *local      = NULL;
+
+        if (!frame || !this)
+                goto unwind;
+
+        local = frame->local;
+        local->this = this;
+        local->validate_cbk = validate_cbk;
+        local->validate_xdata = dict_ref (xdata);
+
+        _quota_enforcer_lookup (frame);
+
         return 0;
 
 unwind:
-        validate_cbk (frame, NULL, this, -1, op_errno, NULL, NULL, NULL, NULL);
-
-        GF_FREE (req.xdata.xdata_val);
+        validate_cbk (frame, NULL, this, -1, ESTALE, NULL, NULL, NULL, NULL);
 
         return 0;
 }
@@ -275,19 +363,19 @@ quota_enforcer_notify (struct rpc_clnt *rpc, void *mydata,
         switch (event) {
         case RPC_CLNT_CONNECT:
         {
-                gf_log (this->name, GF_LOG_TRACE, "got RPC_CLNT_CONNECT");
+                gf_msg_trace (this->name, 0, "got RPC_CLNT_CONNECT");
                 break;
         }
 
         case RPC_CLNT_DISCONNECT:
         {
-                gf_log (this->name, GF_LOG_TRACE, "got RPC_CLNT_DISCONNECT");
+                gf_msg_trace (this->name, 0, "got RPC_CLNT_DISCONNECT");
                 break;
         }
 
         default:
-                gf_log (this->name, GF_LOG_TRACE,
-                        "got some other RPC event %d", event);
+                gf_msg_trace (this->name, 0,
+                              "got some other RPC event %d", event);
                 ret = 0;
                 break;
         }
@@ -321,7 +409,8 @@ quota_enforcer_blocking_connect (rpc_clnt_t *rpc)
 
         ret = 0;
 out:
-        dict_unref (options);
+        if (options)
+                dict_unref (options);
 
         return ret;
 }
@@ -336,16 +425,18 @@ quota_enforcer_init (xlator_t *this, dict_t *options)
         int              ret  = -1;
 
         priv = this->private;
-        if (priv->rpc_clnt) {
-                gf_log (this->name, GF_LOG_TRACE, "quota enforcer clnt already "
-                        "inited");
-                //Turns out to be a NOP if the clnt is already connected.
-                ret = quota_enforcer_blocking_connect (priv->rpc_clnt);
-                if (ret)
-                        goto out;
 
-                return priv->rpc_clnt;
+        LOCK (&priv->lock);
+        {
+                if (priv->rpc_clnt) {
+                        ret = 0;
+                        rpc = priv->rpc_clnt;
+                }
         }
+        UNLOCK (&priv->lock);
+
+        if (rpc)
+                goto out;
 
         priv->quota_enforcer = &quota_enforcer_clnt;
 
@@ -358,7 +449,7 @@ quota_enforcer_init (xlator_t *this, dict_t *options)
                 goto out;
 
         ret = dict_set_str (options, "transport.socket.connect-path",
-                            "/tmp/quotad.socket");
+                            "/var/run/gluster/quotad.socket");
         if (ret)
                 goto out;
 
@@ -370,7 +461,9 @@ quota_enforcer_init (xlator_t *this, dict_t *options)
 
         ret = rpc_clnt_register_notify (rpc, quota_enforcer_notify, this);
         if (ret) {
-                gf_log ("cli", GF_LOG_ERROR, "failed to register notify");
+                gf_msg ("quota", GF_LOG_ERROR, 0,
+                        Q_MSG_RPCCLNT_REGISTER_NOTIFY_FAILED,
+                        "failed to register notify");
                 goto out;
         }
 
@@ -387,7 +480,7 @@ out:
         }
 
         return rpc;
-}
+        }
 
 struct rpc_clnt_procedure quota_enforcer_actors[GF_AGGREGATOR_MAXVALUE] = {
         [GF_AGGREGATOR_NULL]     = {"NULL", NULL},
