@@ -25,7 +25,6 @@
 #include "logging.h"
 #include "dict.h"
 #include "stack.h"
-#include "common-utils.h"
 #include "event.h"
 #include "globals.h"
 #include "rpcsvc.h"
@@ -36,6 +35,8 @@
 #include "xdr-generic.h"
 #include "compat-errno.h"
 #include "protocol-common.h"
+#include "quota-common-utils.h"
+#include "quota-messages.h"
 
 #define DIRTY                   "dirty"
 #define SIZE                    "size"
@@ -51,6 +52,12 @@
         if (!is_quota_on)                       \
                 goto label;
 
+#define QUOTA_WIND_FOR_INTERNAL_FOP(xdata, label)                          \
+        do {                                                               \
+                if (xdata && dict_get (xdata, GLUSTERFS_INTERNAL_FOP_KEY)) \
+                goto label;                                                \
+        } while (0)
+
 #define DID_REACH_LIMIT(lim, prev_size, cur_size)               \
         ((cur_size) >= (lim) && (prev_size) < (lim))
 
@@ -73,13 +80,31 @@
                 var = GF_CALLOC (sizeof (type), 1,      \
                                  gf_quota_mt_##type);   \
                 if (!var) {                             \
-                        gf_log ("", GF_LOG_ERROR,       \
-                                "out of memory");    \
+                        gf_msg ("", GF_LOG_ERROR,       \
+                                ENOMEM, Q_MSG_ENOMEM,   \
+				"out of memory");       \
                         ret = -1;                       \
                         goto label;                     \
                 }                                       \
         } while (0);
 
+#define QUOTA_STACK_WIND_TAIL(frame, params...)                         \
+        do {                                                            \
+                quota_local_t *_local = NULL;                           \
+                xlator_t      *_this  = NULL;                           \
+                                                                        \
+                if (frame) {                                            \
+                        _local = frame->local;                          \
+                        _this  = frame->this;                           \
+                        frame->local = NULL;                            \
+                }                                                       \
+                                                                        \
+                STACK_WIND_TAIL (frame, params);                        \
+                                                                        \
+                if (_local)                                             \
+                        quota_local_cleanup (_local);                   \
+        } while (0)
+
 #define QUOTA_STACK_UNWIND(fop, frame, params...)                       \
         do {                                                            \
                 quota_local_t *_local = NULL;                           \
@@ -90,7 +115,7 @@
                         frame->local = NULL;                            \
                 }                                                       \
                 STACK_UNWIND_STRICT (fop, frame, params);               \
-                quota_local_cleanup (_this, _local);                    \
+                quota_local_cleanup (_local);                           \
         } while (0)
 
 #define QUOTA_FREE_CONTRIBUTION_NODE(_contribution)     \
@@ -103,7 +128,7 @@
         do {                                                    \
                 char _gfid_unparsed[40];                        \
                 if (_gfid != NULL) {                            \
-                        uuid_unparse (_gfid, _gfid_unparsed);   \
+                        gf_uuid_unparse (_gfid, _gfid_unparsed);\
                         _ret = gf_asprintf (var, QUOTA_XATTR_PREFIX     \
                                             "%s.%s." CONTRIBUTION,      \
                                             _vol_name, _gfid_unparsed); \
@@ -146,27 +171,28 @@ struct quota_inode_ctx {
         int64_t          size;
         int64_t          hard_lim;
         int64_t          soft_lim;
+        int64_t          file_count;
+        int64_t          dir_count;
+        int64_t          object_hard_lim;
+        int64_t          object_soft_lim;
         struct iatt      buf;
         struct list_head parents;
         struct timeval   tv;
         struct timeval   prev_log;
+        gf_boolean_t     ancestry_built;
         gf_lock_t        lock;
 };
 typedef struct quota_inode_ctx quota_inode_ctx_t;
 
-struct quota_limit {
-        int64_t hard_lim;
-        int64_t soft_lim_percent;
-} __attribute__ ((packed));
-typedef struct quota_limit quota_limit_t;
-
 typedef void
 (*quota_ancestry_built_t) (struct list_head *parents, inode_t *inode,
                            int32_t op_ret, int32_t op_errno, void *data);
 
+typedef void
+(*quota_fop_continue_t) (call_frame_t *frame);
+
 struct quota_local {
         gf_lock_t               lock;
-        uint32_t                validate_count;
         uint32_t                link_count;
         loc_t                   loc;
         loc_t                   oldloc;
@@ -176,16 +202,23 @@ struct quota_local {
         int32_t                 op_ret;
         int32_t                 op_errno;
         int64_t                 size;
-        gf_boolean_t            skip_check;
         char                    just_validated;
         fop_lookup_cbk_t        validate_cbk;
+        quota_fop_continue_t    fop_continue_cbk;
         inode_t                *inode;
+        uuid_t                  common_ancestor; /* Used by quota_rename */
         call_stub_t            *stub;
         struct iobref          *iobref;
-        quota_limit_t           limit;
+        quota_limits_t          limit;
+        quota_limits_t          object_limit;
         int64_t                 space_available;
         quota_ancestry_built_t  ancestry_cbk;
         void                   *ancestry_data;
+        dict_t                 *xdata;
+        dict_t                 *validate_xdata;
+        int32_t                 quotad_conn_retry;
+        xlator_t               *this;
+        call_frame_t           *par_frame;
 };
 typedef struct quota_local      quota_local_t;
 
@@ -204,12 +237,17 @@ struct quota_priv {
         inode_table_t         *itable;
         char                  *volume_uuid;
         uint64_t               validation_count;
+        int32_t                quotad_conn_status;
 };
 typedef struct quota_priv      quota_priv_t;
 
 int
-quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
-                       dict_t *xdata, fop_lookup_cbk_t cbk);
+quota_enforcer_lookup (call_frame_t *frame, xlator_t *this, dict_t *xdata,
+                       fop_lookup_cbk_t cbk);
+
+void
+_quota_enforcer_lookup (void *data);
+
 struct rpc_clnt *
 quota_enforcer_init (xlator_t *this, dict_t *options);
 
@@ -217,4 +255,33 @@ void
 quota_log_usage (xlator_t *this, quota_inode_ctx_t *ctx, inode_t *inode,
                  int64_t delta);
 
+int
+quota_build_ancestry (inode_t *inode, quota_ancestry_built_t ancestry_cbk,
+                      void *data);
+
+void
+quota_get_limit_dir (call_frame_t *frame, inode_t *cur_inode, xlator_t *this);
+
+int32_t
+quota_check_limit (call_frame_t *frame, inode_t *inode, xlator_t *this,
+                   char *name, uuid_t par);
+
+inode_t *
+do_quota_check_limit (call_frame_t *frame, inode_t *inode, xlator_t *this,
+                      quota_dentry_t *dentry);
+int
+quota_fill_inodectx (xlator_t *this, inode_t *inode, dict_t *dict,
+                     loc_t *loc, struct iatt *buf, int32_t *op_errno);
+
+int32_t
+quota_check_size_limit (call_frame_t *frame, quota_inode_ctx_t *ctx,
+                          quota_priv_t *priv, inode_t *_inode, xlator_t *this,
+                          int32_t *op_errno, int just_validated, int64_t delta,
+                          quota_local_t *local, gf_boolean_t *skip_check);
+
+int32_t
+quota_check_object_limit (call_frame_t *frame, quota_inode_ctx_t *ctx,
+                          quota_priv_t *priv, inode_t *_inode, xlator_t *this,
+                          int32_t *op_errno, int just_validated,
+                          quota_local_t *local, gf_boolean_t *skip_check);
 #endif
