@@ -29,9 +29,11 @@
 #include "glusterd-sm.h"
 #include "glusterd-op-sm.h"
 #include "glusterd-utils.h"
+#include "glusterd-messages.h"
 #include "glusterd-store.h"
 #include "run.h"
 #include "glusterd-volgen.h"
+#include "glusterd-messages.h"
 
 #include "syscall.h"
 #include "cli1-xdr.h"
@@ -53,7 +55,7 @@ glusterd_defrag_start_validate (glusterd_volinfo_t *volinfo, char *op_errstr,
         /* Check only if operation is not remove-brick */
         if ((GD_OP_REMOVE_BRICK != op) &&
             !gd_is_remove_brick_committed (volinfo)) {
-                gf_log (this->name, GF_LOG_DEBUG, "A remove-brick task on "
+                gf_msg_debug (this->name, 0, "A remove-brick task on "
                         "volume %s is not yet committed", volinfo->volname);
                 snprintf (op_errstr, len, "A remove-brick task on volume %s is"
                           " not yet committed. Either commit or stop the "
@@ -62,7 +64,7 @@ glusterd_defrag_start_validate (glusterd_volinfo_t *volinfo, char *op_errstr,
         }
 
         if (glusterd_is_defrag_on (volinfo)) {
-                gf_log (this->name, GF_LOG_DEBUG,
+                gf_msg_debug (this->name, 0,
                         "rebalance on volume %s already started",
                         volinfo->volname);
                 snprintf (op_errstr, len, "Rebalance on %s is already started",
@@ -70,18 +72,9 @@ glusterd_defrag_start_validate (glusterd_volinfo_t *volinfo, char *op_errstr,
                 goto out;
         }
 
-        if (glusterd_is_rb_started (volinfo) ||
-            glusterd_is_rb_paused (volinfo)) {
-                gf_log (this->name, GF_LOG_DEBUG,
-                        "Rebalance failed as replace brick is in progress on volume %s",
-                        volinfo->volname);
-                snprintf (op_errstr, len, "Rebalance failed as replace brick is in progress on "
-                          "volume %s", volinfo->volname);
-                goto out;
-        }
         ret = 0;
 out:
-        gf_log (this->name, GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug (this->name, 0, "Returning %d", ret);
         return ret;
 }
 
@@ -95,8 +88,13 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
         int                     ret      = 0;
         char                    pidfile[PATH_MAX];
         glusterd_conf_t        *priv    = NULL;
+        xlator_t               *this    = NULL;
 
-        priv = THIS->private;
+        this = THIS;
+        if (!this)
+                return 0;
+
+        priv = this->private;
         if (!priv)
                 return 0;
 
@@ -125,8 +123,8 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
                 }
                 UNLOCK (&defrag->lock);
 
-               gf_log ("", GF_LOG_DEBUG, "%s got RPC_CLNT_CONNECT",
-                        rpc->conn.trans->name);
+               gf_msg_debug (this->name, 0, "%s got RPC_CLNT_CONNECT",
+                        rpc->conn.name);
                break;
         }
 
@@ -151,24 +149,23 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
 
                 glusterd_store_perform_node_state_store (volinfo);
 
-                if (defrag->rpc) {
-                        glusterd_rpc_clnt_unref (priv, defrag->rpc);
-                        defrag->rpc = NULL;
-                }
+                glusterd_defrag_rpc_put (defrag);
                 if (defrag->cbk_fn)
                         defrag->cbk_fn (volinfo,
                                         volinfo->rebal.defrag_status);
 
                 GF_FREE (defrag);
-                gf_log ("", GF_LOG_DEBUG, "%s got RPC_CLNT_DISCONNECT",
-                        rpc->conn.trans->name);
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        GD_MSG_REBALANCE_DISCONNECTED,
+                        "Rebalance process for volume %s has disconnected.",
+                        volinfo->volname);
                 break;
         }
         case RPC_CLNT_DESTROY:
                 glusterd_volinfo_unref (volinfo);
                 break;
         default:
-                gf_log ("", GF_LOG_TRACE,
+                gf_msg_trace (this->name, 0,
                         "got some other RPC event %d", event);
                 ret = 0;
                 break;
@@ -198,6 +195,7 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         char                   sockfile[PATH_MAX] = {0,};
         char                   pidfile[PATH_MAX] = {0,};
         char                   logfile[PATH_MAX] = {0,};
+        char                   volname[PATH_MAX] = {0,};
         char                   valgrind_logfile[PATH_MAX] = {0,};
 
         priv    = THIS->private;
@@ -205,6 +203,10 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         GF_ASSERT (volinfo);
         GF_ASSERT (op_errstr);
 
+        if ((cmd == GF_OP_CMD_DETACH_START) &&
+            (volinfo->rebal.defrag_status == GF_DEFRAG_STATUS_STARTED))
+                return 0;
+
         ret = glusterd_defrag_start_validate (volinfo, op_errstr, len, op);
         if (ret)
                 goto out;
@@ -232,15 +234,17 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         GLUSTERD_GET_DEFRAG_DIR (defrag_path, volinfo, priv);
         ret = mkdir_p (defrag_path, 0777, _gf_true);
         if (ret) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Failed to create "
+                gf_msg (THIS->name, GF_LOG_ERROR, errno,
+                        GD_MSG_CREATE_DIR_FAILED, "Failed to create "
                         "directory %s", defrag_path);
                 goto out;
         }
 
-        GLUSTERD_GET_DEFRAG_SOCK_FILE (sockfile, volinfo, priv);
+        GLUSTERD_GET_DEFRAG_SOCK_FILE (sockfile, volinfo);
         GLUSTERD_GET_DEFRAG_PID_FILE (pidfile, volinfo, priv);
-        snprintf (logfile, PATH_MAX, "%s/%s-rebalance.log",
-                    DEFAULT_LOG_FILE_DIRECTORY, volinfo->volname);
+        snprintf (logfile, PATH_MAX, "%s/%s-%s.log",
+                    DEFAULT_LOG_FILE_DIRECTORY, volinfo->volname,
+                    (cmd == GF_DEFRAG_CMD_START_TIER ? "tier":"rebalance"));
         runinit (&runner);
 
         if (priv->valgrind) {
@@ -255,8 +259,9 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
                 runner_argprintf (&runner, "--log-file=%s", valgrind_logfile);
         }
 
+        snprintf (volname, sizeof(volname), "rebalance/%s", volinfo->volname);
         runner_add_args (&runner, SBIN_DIR"/glusterfs",
-                         "-s", "localhost", "--volfile-id", volinfo->volname,
+                         "-s", "localhost", "--volfile-id", volname,
                          "--xlator-option", "*dht.use-readdirp=yes",
                          "--xlator-option", "*dht.lookup-unhashed=yes",
                          "--xlator-option", "*dht.assert-no-child-down=yes",
@@ -264,13 +269,22 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
                          "--xlator-option",
                          "*replicate*.metadata-self-heal=off",
                          "--xlator-option", "*replicate*.entry-self-heal=off",
-                         "--xlator-option", "*replicate*.readdir-failover=off",
                          "--xlator-option", "*dht.readdir-optimize=on",
                          NULL);
+
+        if (volinfo->type == GF_CLUSTER_TYPE_TIER) {
+                runner_add_arg (&runner, "--xlator-option");
+                runner_argprintf (&runner,
+                                  "*tier-dht.xattr-name=trusted.tier-gfid");
+        }
+
         runner_add_arg (&runner, "--xlator-option");
         runner_argprintf ( &runner, "*dht.rebalance-cmd=%d",cmd);
         runner_add_arg (&runner, "--xlator-option");
         runner_argprintf (&runner, "*dht.node-uuid=%s", uuid_utoa(MY_UUID));
+        runner_add_arg (&runner, "--xlator-option");
+        runner_argprintf (&runner, "*dht.commit-hash=%u",
+                          volinfo->rebal.commit_hash);
         runner_add_arg (&runner, "--socket-file");
         runner_argprintf (&runner, "%s",sockfile);
         runner_add_arg (&runner, "--pid-file");
@@ -282,13 +296,13 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
 
         ret = runner_run_nowait (&runner);
         if (ret) {
-                gf_log ("glusterd", GF_LOG_DEBUG, "rebalance command failed");
+                gf_msg_debug ("glusterd", 0, "rebalance command failed");
                 goto out;
         }
 
         sleep (5);
 
-        ret = glusterd_rebalance_rpc_create (volinfo);
+        ret = glusterd_rebalance_rpc_create (volinfo, _gf_false);
 
         //FIXME: this cbk is passed as NULL in all occurrences. May be
         //we never needed it.
@@ -296,30 +310,63 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
                 defrag->cbk_fn = cbk;
 
 out:
-        gf_log ("", GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug ("glusterd", 0, "Returning %d", ret);
         return ret;
 }
 
 
 int
-glusterd_rebalance_rpc_create (glusterd_volinfo_t *volinfo)
+glusterd_rebalance_rpc_create (glusterd_volinfo_t *volinfo,
+                               gf_boolean_t reconnect)
 {
         dict_t                  *options = NULL;
         char                     sockfile[PATH_MAX] = {0,};
         int                      ret = -1;
         glusterd_defrag_info_t  *defrag = volinfo->rebal.defrag;
-        glusterd_conf_t         *priv = THIS->private;
+        glusterd_conf_t         *priv = NULL;
+        xlator_t                *this = NULL;
+        struct stat             buf = {0,};
+
+        this = THIS;
+        GF_ASSERT (this);
+        priv = this->private;
+        GF_ASSERT (priv);
 
         //rebalance process is not started
         if (!defrag)
                 goto out;
 
         //rpc obj for rebalance process already in place.
-        if (defrag->rpc) {
+        if (glusterd_defrag_rpc_get (defrag)) {
                 ret = 0;
+                glusterd_defrag_rpc_put (defrag);
                 goto out;
         }
-        GLUSTERD_GET_DEFRAG_SOCK_FILE (sockfile, volinfo, priv);
+        GLUSTERD_GET_DEFRAG_SOCK_FILE (sockfile, volinfo);
+        /* If reconnecting check if defrag sockfile exists in the new location
+         * in /var/run/ , if it does not try the old location
+         */
+        if (reconnect) {
+                ret = sys_stat (sockfile, &buf);
+                /* TODO: Remove this once we don't need backward compatibility
+                 * with the older path
+                 */
+                if (ret && (errno == ENOENT)) {
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                GD_MSG_FILE_OP_FAILED, "Rebalance sockfile "
+                                "%s does not exist. Trying old path.",
+                                sockfile);
+                        GLUSTERD_GET_DEFRAG_SOCK_FILE_OLD (sockfile, volinfo,
+                                                           priv);
+                        ret =sys_stat (sockfile, &buf);
+                        if (ret && (ENOENT == errno)) {
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_REBAL_NO_SOCK_FILE, "Rebalance "
+                                        "sockfile %s does not exist", sockfile);
+                                goto out;
+                        }
+                }
+        }
 
         /* Setting frame-timeout to 10mins (600seconds).
          * Unix domain sockets ensures that the connection is reliable. The
@@ -328,17 +375,17 @@ glusterd_rebalance_rpc_create (glusterd_volinfo_t *volinfo)
          */
         ret = rpc_transport_unix_options_build (&options, sockfile, 600);
         if (ret) {
-                gf_log (THIS->name, GF_LOG_ERROR, "Unix options build failed");
+                gf_msg (THIS->name, GF_LOG_ERROR, 0, GD_MSG_UNIX_OP_BUILD_FAIL,
+                        "Unix options build failed");
                 goto out;
         }
 
         glusterd_volinfo_ref (volinfo);
-        synclock_unlock (&priv->big_lock);
         ret = glusterd_rpc_create (&defrag->rpc, options,
                                    glusterd_defrag_notify, volinfo);
-        synclock_lock (&priv->big_lock);
         if (ret) {
-                gf_log (THIS->name, GF_LOG_ERROR, "RPC create failed");
+                gf_msg (THIS->name, GF_LOG_ERROR, 0, GD_MSG_RPC_CREATE_FAIL,
+                        "Glusterd RPC creation failed");
                 goto out;
         }
         ret = 0;
@@ -354,14 +401,16 @@ glusterd_rebalance_cmd_validate (int cmd, char *volname,
         int ret = -1;
 
         if (glusterd_volinfo_find(volname, volinfo)) {
-                gf_log ("glusterd", GF_LOG_ERROR, "Received rebalance on invalid"
+                gf_msg ("glusterd", GF_LOG_ERROR, EINVAL,
+                        GD_MSG_VOL_NOT_FOUND, "Received rebalance on invalid"
                         " volname %s", volname);
                 snprintf (op_errstr, len, "Volume %s does not exist",
                           volname);
                 goto out;
         }
         if ((*volinfo)->brick_count <= (*volinfo)->dist_leaf_count) {
-                gf_log ("glusterd", GF_LOG_ERROR, "Volume %s is not a "
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_VOL_NOT_DISTRIBUTE, "Volume %s is not a "
                 "distribute type or contains only 1 brick", volname);
                 snprintf (op_errstr, len, "Volume %s is not a distribute "
                           "volume or contains only 1 brick.\n"
@@ -370,17 +419,29 @@ glusterd_rebalance_cmd_validate (int cmd, char *volname,
         }
 
         if ((*volinfo)->status != GLUSTERD_STATUS_STARTED) {
-                gf_log ("glusterd", GF_LOG_ERROR, "Received rebalance on stopped"
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_VOL_STOPPED, "Received rebalance on stopped"
                         " volname %s", volname);
                 snprintf (op_errstr, len, "Volume %s needs to "
                           "be started to perform rebalance", volname);
                 goto out;
         }
 
+        ret = glusterd_disallow_op_for_tier (*volinfo, GD_OP_REBALANCE, cmd);
+        if (ret) {
+                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                        GD_MSG_REBALANCE_CMD_IN_TIER_VOL,
+                        "Received rebalance command "
+                        "on Tier volume %s", volname);
+                snprintf (op_errstr, len, "Rebalance operations are not "
+                          "supported on a tiered volume");
+                goto out;
+        }
+
         ret = 0;
 
 out:
-        gf_log ("glusterd", GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug ("glusterd", 0, "Returning %d", ret);
         return ret;
 }
 
@@ -418,7 +479,8 @@ __glusterd_handle_defrag_volume (rpcsvc_request_t *req)
                                         cli_req.dict.dict_len,
                                         &dict);
                 if (ret < 0) {
-                        gf_log (this->name, GF_LOG_ERROR, "failed to "
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                GD_MSG_DICT_UNSERIALIZE_FAIL, "failed to "
                                 "unserialize req-buffer to dictionary");
                         snprintf (msg, sizeof (msg), "Unable to decode the "
                                   "command");
@@ -429,14 +491,16 @@ __glusterd_handle_defrag_volume (rpcsvc_request_t *req)
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
                 snprintf (msg, sizeof (msg), "Failed to get volume name");
-                gf_log (this->name, GF_LOG_ERROR, "%s", msg);
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "%s", msg);
                 goto out;
         }
 
         ret = dict_get_int32 (dict, "rebalance-command", (int32_t*)&cmd);
         if (ret) {
                 snprintf (msg, sizeof (msg), "Failed to get command");
-                gf_log (this->name, GF_LOG_ERROR, "%s", msg);
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "%s", msg);
                 goto out;
         }
 
@@ -445,6 +509,8 @@ __glusterd_handle_defrag_volume (rpcsvc_request_t *req)
                 goto out;
 
         if ((cmd == GF_DEFRAG_CMD_STATUS) ||
+            (cmd == GF_DEFRAG_CMD_STATUS_TIER) ||
+            (cmd == GF_DEFRAG_CMD_STOP_DETACH_TIER) ||
               (cmd == GF_DEFRAG_CMD_STOP)) {
                 ret = glusterd_op_begin (req, GD_OP_DEFRAG_BRICK_VOLUME,
                                          dict, msg, sizeof (msg));
@@ -476,49 +542,110 @@ glusterd_handle_defrag_volume (rpcsvc_request_t *req)
         return glusterd_big_locked_handler (req, __glusterd_handle_defrag_volume);
 }
 
+static int
+glusterd_brick_validation  (dict_t *dict, char *key, data_t *value,
+                            void *data)
+{
+        int32_t                   ret              = -1;
+        xlator_t                 *this             = NULL;
+        glusterd_volinfo_t       *volinfo          = data;
+        glusterd_brickinfo_t     *brickinfo        = NULL;
+
+        this = THIS;
+        GF_ASSERT (this);
+
+        ret = glusterd_volume_brickinfo_get_by_brick (value->data, volinfo,
+                                                      &brickinfo);
+        if (ret) {
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        GD_MSG_BRICK_NOT_FOUND,
+                        "Incorrect brick %s for "
+                        "volume %s", value->data, volinfo->volname);
+                return ret;
+        }
+
+        if (!brickinfo->decommissioned) {
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        GD_MSG_BRICK_NOT_FOUND, "Incorrect brick %s for "
+                        "volume %s", value->data, volinfo->volname);
+                ret = -1;
+                return ret;
+        }
+
+        return ret;
+}
 
 int
 glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
 {
-        char                    *volname = NULL;
-        int                     ret = 0;
-        int32_t                 cmd = 0;
-        char                    msg[2048] = {0};
-        glusterd_volinfo_t      *volinfo = NULL;
+        char                    *volname     = NULL;
+        char                    *cmd_str     = NULL;
+        int                     ret          = 0;
+        int32_t                 cmd          = 0;
+        char                    msg[2048]    = {0};
+        glusterd_volinfo_t      *volinfo     = NULL;
         char                    *task_id_str = NULL;
-        dict_t                  *op_ctx = NULL;
-        xlator_t                *this = 0;
+        dict_t                  *op_ctx      = NULL;
+        xlator_t                *this        = 0;
 
         this = THIS;
         GF_ASSERT (this);
 
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "volname not found");
+                gf_msg_debug (this->name, 0, "volname not found");
                 goto out;
         }
 
         ret = dict_get_int32 (dict, "rebalance-command", &cmd);
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "cmd not found");
+                gf_msg_debug (this->name, 0, "cmd not found");
                 goto out;
         }
 
         ret = glusterd_rebalance_cmd_validate (cmd, volname, &volinfo,
                                                msg, sizeof (msg));
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "failed to validate");
+                gf_msg_debug (this->name, 0, "failed to validate");
                 goto out;
         }
         switch (cmd) {
+        case GF_DEFRAG_CMD_START_TIER:
+                if (volinfo->type != GF_CLUSTER_TYPE_TIER) {
+                        gf_asprintf (op_errstr, "volume %s is not a tier "
+                                     "volume.", volinfo->volname);
+                        ret = -1;
+                        goto out;
+                }
         case GF_DEFRAG_CMD_START:
         case GF_DEFRAG_CMD_START_LAYOUT_FIX:
+                /* Check if the connected clients are all of version
+                 * glusterfs-3.6 and higher. This is needed to prevent some data
+                 * loss issues that could occur when older clients are connected
+                 * when rebalance is run. This check can be bypassed by using
+                 * 'force'
+                 */
+                ret = glusterd_check_client_op_version_support
+                        (volname, GD_OP_VERSION_3_6_0, NULL);
+                if (ret) {
+                        ret = gf_asprintf (op_errstr, "Volume %s has one or "
+                                           "more connected clients of a version"
+                                           " lower than GlusterFS-v3.6.0. "
+                                           "Starting rebalance in this state "
+                                           "could lead to data loss.\nPlease "
+                                           "disconnect those clients before "
+                                           "attempting this command again.",
+                                           volname);
+                        goto out;
+                }
+
         case GF_DEFRAG_CMD_START_FORCE:
-                if (is_origin_glusterd ()) {
+                if (is_origin_glusterd (dict)) {
                         op_ctx = glusterd_op_get_ctx ();
                         if (!op_ctx) {
                                 ret = -1;
-                                gf_log (this->name, GF_LOG_ERROR,
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_OPCTX_GET_FAIL,
                                         "Failed to get op_ctx");
                                 goto out;
                         }
@@ -526,7 +653,8 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                         ret = glusterd_generate_and_set_task_id
                                 (op_ctx, GF_REBALANCE_TID_KEY);
                         if (ret) {
-                                gf_log (this->name, GF_LOG_ERROR,
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_TASKID_GEN_FAIL,
                                         "Failed to generate task-id");
                                 goto out;
                         }
@@ -536,7 +664,8 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                         if (ret) {
                                 snprintf (msg, sizeof (msg),
                                           "Missing rebalance-id");
-                                gf_log (this->name, GF_LOG_WARNING, "%s", msg);
+                                gf_msg (this->name, GF_LOG_WARNING, 0,
+                                        GD_MSG_REBALANCE_ID_MISSING, "%s", msg);
                                 ret = 0;
                         }
                 }
@@ -544,13 +673,56 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                                                       sizeof (msg),
                                                       GD_OP_REBALANCE);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_DEBUG,
+                        gf_msg_debug (this->name, 0,
                                         "start validate failed");
                         goto out;
                 }
                 break;
+        case GF_DEFRAG_CMD_STATUS_TIER:
         case GF_DEFRAG_CMD_STATUS:
         case GF_DEFRAG_CMD_STOP:
+        case GF_DEFRAG_CMD_STOP_DETACH_TIER:
+                ret = dict_get_str (dict, "cmd-str", &cmd_str);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                GD_MSG_DICT_GET_FAILED, "Failed to get "
+                                "command string");
+                        ret = -1;
+                        goto out;
+                }
+                if ((strstr(cmd_str, "rebalance") != NULL) &&
+                    (volinfo->rebal.op != GD_OP_REBALANCE)) {
+                        snprintf (msg, sizeof(msg), "Rebalance not started.");
+                        ret = -1;
+                        goto out;
+                }
+                if (strstr(cmd_str, "remove-brick") != NULL) {
+                        if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
+                                snprintf (msg, sizeof(msg), "remove-brick not "
+                                          "started.");
+                                ret = -1;
+                                goto out;
+                        }
+
+                        /* For remove-brick status/stop command check whether
+                         * given input brick is part of volume or not.*/
+
+                        ret = dict_foreach_fnmatch (dict, "brick*",
+                                                    glusterd_brick_validation,
+                                                    volinfo);
+                        if (ret == -1) {
+                                snprintf (msg, sizeof (msg), "Incorrect brick"
+                                          " for volume %s", volinfo->volname);
+                                goto out;
+                        }
+                }
+                if ((strstr(cmd_str, "detach-tier") != NULL) &&
+                    (volinfo->rebal.op != GD_OP_REMOVE_BRICK)) {
+                        snprintf (msg, sizeof(msg), "Detach-tier not started.");
+                        ret = -1;
+                        goto out;
+                }
+
                 break;
         default:
                 break;
@@ -564,7 +736,6 @@ out:
         return ret;
 }
 
-
 int
 glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
 {
@@ -580,6 +751,7 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         char                    *task_id_str = NULL;
         dict_t                  *ctx = NULL;
         xlator_t                *this = NULL;
+        uint32_t                commit_hash;
 
         this = THIS;
         GF_ASSERT (this);
@@ -587,13 +759,13 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
 
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "volname not given");
+                gf_msg_debug (this->name, 0, "volname not given");
                 goto out;
         }
 
         ret = dict_get_int32 (dict, "rebalance-command", &cmd);
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "command not given");
+                gf_msg_debug (this->name, 0, "command not given");
                 goto out;
         }
 
@@ -601,18 +773,21 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         ret = glusterd_rebalance_cmd_validate (cmd, volname, &volinfo,
                                                msg, sizeof (msg));
         if (ret) {
-                gf_log (this->name, GF_LOG_DEBUG, "cmd validate failed");
+                gf_msg_debug (this->name, 0, "cmd validate failed");
                 goto out;
         }
 
         /* Set task-id, if available, in op_ctx dict for operations other than
          * start
          */
-        if (cmd == GF_DEFRAG_CMD_STATUS || cmd == GF_DEFRAG_CMD_STOP) {
-                if (!uuid_is_null (volinfo->rebal.rebalance_id)) {
+        if (cmd == GF_DEFRAG_CMD_STATUS ||
+            cmd == GF_DEFRAG_CMD_STOP ||
+            cmd == GF_DEFRAG_CMD_STATUS_TIER) {
+                if (!gf_uuid_is_null (volinfo->rebal.rebalance_id)) {
                         ctx = glusterd_op_get_ctx ();
                         if (!ctx) {
-                                gf_log (this->name, GF_LOG_ERROR,
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_OPCTX_GET_FAIL,
                                         "Failed to get op_ctx");
                                 ret = -1;
                                 goto out;
@@ -627,7 +802,8 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
                                         (volinfo->rebal.rebalance_id, ctx,
                                          GF_REBALANCE_TID_KEY);
                         if (ret) {
-                                gf_log (this->name, GF_LOG_ERROR,
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_TASKID_GEN_FAIL,
                                         "Failed to set task-id");
                                 goto out;
                         }
@@ -638,31 +814,52 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         case GF_DEFRAG_CMD_START:
         case GF_DEFRAG_CMD_START_LAYOUT_FIX:
         case GF_DEFRAG_CMD_START_FORCE:
+        case GF_DEFRAG_CMD_START_TIER:
+                /* Reset defrag status to 'NOT STARTED' whenever a
+                 * remove-brick/rebalance command is issued to remove
+                 * stale information from previous run.
+                 */
+                volinfo->rebal.defrag_status = GF_DEFRAG_STATUS_NOT_STARTED;
+
                 ret = dict_get_str (dict, GF_REBALANCE_TID_KEY, &task_id_str);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_DEBUG, "Missing rebalance "
+                        gf_msg_debug (this->name, 0, "Missing rebalance "
                                 "id");
                         ret = 0;
                 } else {
-                        uuid_parse (task_id_str, volinfo->rebal.rebalance_id) ;
+                        gf_uuid_parse (task_id_str, volinfo->rebal.rebalance_id) ;
                         volinfo->rebal.op = GD_OP_REBALANCE;
                 }
-                if (!gd_should_i_start_rebalance (volinfo))
+                if (!gd_should_i_start_rebalance (volinfo)) {
+                        /* Store the rebalance-id and rebalance command even if
+                         * the peer isn't starting a rebalance process. On peers
+                         * where a rebalance process is started,
+                         * glusterd_handle_defrag_start performs the storing.
+                         *
+                         * Storing this is needed for having 'volume status'
+                         * work correctly.
+                         */
+                        glusterd_store_perform_node_state_store (volinfo);
                         break;
+                }
+                if (dict_get_uint32 (dict, "commit-hash", &commit_hash) == 0) {
+                        volinfo->rebal.commit_hash = commit_hash;
+                }
                 ret = glusterd_handle_defrag_start (volinfo, msg, sizeof (msg),
                                                     cmd, NULL, GD_OP_REBALANCE);
                 break;
         case GF_DEFRAG_CMD_STOP:
+        case GF_DEFRAG_CMD_STOP_DETACH_TIER:
                 /* Clear task-id only on explicitly stopping rebalance.
                  * Also clear the stored operation, so it doesn't cause trouble
                  * with future rebalance/remove-brick starts
                  */
-                uuid_clear (volinfo->rebal.rebalance_id);
+                gf_uuid_clear (volinfo->rebal.rebalance_id);
                 volinfo->rebal.op = GD_OP_NONE;
 
                 /* Fall back to the old volume file in case of decommission*/
-                list_for_each_entry_safe (brickinfo, tmp, &volinfo->bricks,
-                                          brick_list) {
+                cds_list_for_each_entry_safe (brickinfo, tmp, &volinfo->bricks,
+                                              brick_list) {
                         if (!brickinfo->decommissioned)
                                 continue;
                         brickinfo->decommissioned = 0;
@@ -676,7 +873,8 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
 
                 ret = glusterd_create_volfiles_and_notify_services (volinfo);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                GD_MSG_VOLFILE_CREATE_FAIL,
                                 "failed to create volfiles");
                         goto out;
                 }
@@ -684,7 +882,8 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
                 ret = glusterd_store_volinfo (volinfo,
                                              GLUSTERD_VOLINFO_VER_AC_INCREMENT);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                GD_MSG_VOLINFO_SET_FAIL,
                                 "failed to store volinfo");
                         goto out;
                 }
@@ -692,7 +891,9 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
                 ret = 0;
                 break;
 
+        case GF_DEFRAG_CMD_START_DETACH_TIER:
         case GF_DEFRAG_CMD_STATUS:
+        case GF_DEFRAG_CMD_STATUS_TIER:
                 break;
         default:
                 break;
@@ -708,19 +909,45 @@ out:
 int32_t
 glusterd_defrag_event_notify_handle (dict_t *dict)
 {
-        glusterd_volinfo_t      *volinfo = NULL;
-        char                    *volname = NULL;
-        int32_t                  ret     = -1;
+        glusterd_volinfo_t      *volinfo     = NULL;
+        char                    *volname     = NULL;
+        char                    *volname_ptr = NULL;
+        int32_t                  ret         = -1;
+        xlator_t                *this        = NULL;
+
+        this = THIS;
+        GF_ASSERT (this);
+        GF_ASSERT (dict);
 
         ret = dict_get_str (dict, "volname", &volname);
         if (ret) {
-                gf_log ("", GF_LOG_ERROR, "Failed to get volname");
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_DICT_GET_FAILED, "Failed to get volname");
                 return ret;
         }
 
+        volname_ptr = strstr (volname, "rebalance/");
+        if (volname_ptr) {
+                volname_ptr = strchr (volname_ptr, '/');
+                if (!volname_ptr) {
+                        ret = -1;
+                        goto out;
+                }
+                volname = volname_ptr + 1;
+        } else {
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_NO_REBALANCE_PFX_IN_VOLNAME,
+                        "volname recieved (%s) is not prefixed with rebalance.",
+                        volname);
+                ret = -1;
+                goto out;
+        }
+
         ret = glusterd_volinfo_find (volname, &volinfo);
         if (ret) {
-                gf_log ("", GF_LOG_ERROR, "Failed to get volinfo for %s"
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_VOLINFO_GET_FAIL,
+                        "Failed to get volinfo for %s"
                         , volname);
                 return ret;
         }
@@ -728,6 +955,10 @@ glusterd_defrag_event_notify_handle (dict_t *dict)
         ret = glusterd_defrag_volume_status_update (volinfo, dict);
 
         if (ret)
-                gf_log ("", GF_LOG_ERROR, "Failed to update status");
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_DEFRAG_STATUS_UPDATE_FAIL,
+                        "Failed to update status");
+
+out:
         return ret;
 }
