@@ -8,7 +8,7 @@
    cases as published by the Free Software Foundation.
 */
 #ifndef _GLUSTERD_UTILS_H
-#define _GLUSTERD_UTILS_H_
+#define _GLUSTERD_UTILS_H
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
@@ -16,7 +16,7 @@
 #endif
 
 #include <pthread.h>
-#include "uuid.h"
+#include "compat-uuid.h"
 
 #include "glusterfs.h"
 #include "xlator.h"
@@ -28,7 +28,14 @@
 #include "rpc-clnt.h"
 #include "protocol-common.h"
 
-#define GLUSTERD_SOCK_DIR "/var/run"
+#include "glusterfs3-xdr.h"
+#include "glusterd-peer-utils.h"
+
+#define GLUSTERD_SOCK_DIR "/var/run/gluster"
+#define GLUSTERD_ASSIGN_BRICKID_TO_BRICKINFO(brickinfo, volinfo, brickid) do {\
+        sprintf (brickinfo->brick_id, "%s-client-%d",\
+                 volinfo->volname, brickid);\
+} while (0)
 
 struct glusterd_lock_ {
         uuid_t  owner;
@@ -81,6 +88,11 @@ glusterd_submit_request (struct rpc_clnt *rpc, void *req,
 int32_t
 glusterd_volinfo_new (glusterd_volinfo_t **volinfo);
 
+int32_t
+glusterd_volinfo_dup (glusterd_volinfo_t *volinfo,
+                      glusterd_volinfo_t **dup_volinfo,
+                      gf_boolean_t set_userauth);
+
 char *
 glusterd_auth_get_username (glusterd_volinfo_t *volinfo);
 
@@ -105,15 +117,6 @@ glusterd_brickinfo_new (glusterd_brickinfo_t **brickinfo);
 int32_t
 glusterd_brickinfo_new_from_brick (char *brick, glusterd_brickinfo_t **brickinfo);
 
-int32_t
-glusterd_friend_cleanup (glusterd_peerinfo_t *peerinfo);
-
-int32_t
-glusterd_peer_destroy (glusterd_peerinfo_t *peerinfo);
-
-int32_t
-glusterd_peer_hostname_new (char *hostname, glusterd_peer_hostname_t **name);
-
 int32_t
 glusterd_volinfo_find (char *volname, glusterd_volinfo_t **volinfo);
 
@@ -124,6 +127,9 @@ int32_t
 glusterd_service_stop(const char *service, char *pidfile, int sig,
                       gf_boolean_t force_kill);
 
+int
+glusterd_get_next_available_brickid (glusterd_volinfo_t *volinfo);
+
 int32_t
 glusterd_resolve_brick (glusterd_brickinfo_t *brickinfo);
 
@@ -158,105 +164,40 @@ glusterd_volume_brickinfo_get_by_brick (char *brick,
                                         glusterd_brickinfo_t **brickinfo);
 
 int32_t
-glusterd_build_volume_dict (dict_t **vols);
+glusterd_add_volumes_to_export_dict (dict_t **peer_data);
 
 int32_t
-glusterd_compare_friend_data (dict_t  *vols, int32_t *status, char *hostname);
+glusterd_compare_friend_data (dict_t *peer_data, int32_t *status,
+                              char *hostname);
 
 int
 glusterd_compute_cksum (glusterd_volinfo_t  *volinfo,
                         gf_boolean_t is_quota_conf);
 
-void
-glusterd_get_nodesvc_volfile (char *server, char *workdir,
-                                    char *volfile, size_t len);
-
-gf_boolean_t
-glusterd_is_nodesvc_running ();
-
-gf_boolean_t
-glusterd_is_nodesvc_running ();
-
-void
-glusterd_get_nodesvc_dir (char *server, char *workdir,
-                                char *path, size_t len);
-int32_t
-glusterd_nfs_server_start ();
-
-int32_t
-glusterd_nfs_server_stop ();
-
-int32_t
-glusterd_shd_start ();
-
-int32_t
-glusterd_shd_stop ();
-
-int32_t
-glusterd_quotad_start ();
-
-int32_t
-glusterd_quotad_stop ();
-
 void
 glusterd_set_socket_filepath (char *sock_filepath, char *sockpath, size_t len);
 
-int32_t
-glusterd_nodesvc_set_socket_filepath (char *rundir, uuid_t uuid,
-                                      char *socketpath, int len);
-
 struct rpc_clnt*
 glusterd_pending_node_get_rpc (glusterd_pending_node_t *pending_node);
 
-struct rpc_clnt*
-glusterd_nodesvc_get_rpc (char *server);
-
-int32_t
-glusterd_nodesvc_set_rpc (char *server, struct rpc_clnt *rpc);
-
-int32_t
-glusterd_nodesvc_connect (char *server, char *socketpath);
-
 void
-glusterd_nodesvc_set_online_status (char *server, gf_boolean_t status);
-
-gf_boolean_t
-glusterd_is_nodesvc_online (char *server);
+glusterd_pending_node_put_rpc (glusterd_pending_node_t *pending_node);
 
 int
 glusterd_remote_hostname_get (rpcsvc_request_t *req,
                               char *remote_host, int len);
 int32_t
-glusterd_import_friend_volumes (dict_t  *vols);
+glusterd_import_friend_volumes (dict_t *peer_data);
 void
 glusterd_set_volume_status (glusterd_volinfo_t  *volinfo,
                             glusterd_volume_status status);
-int
-glusterd_check_generate_start_nfs (void);
-
-int
-glusterd_check_generate_start_shd (void);
-
-int
-glusterd_check_generate_start_quotad (void);
-
-int
-glusterd_nodesvcs_handle_graph_change (glusterd_volinfo_t *volinfo);
-
-int
-glusterd_nodesvcs_handle_reconfigure (glusterd_volinfo_t *volinfo);
-
-int
-glusterd_nodesvcs_start (glusterd_volinfo_t *volinfo);
-
-int
-glusterd_nodesvcs_stop (glusterd_volinfo_t *volinfo);
 
 int32_t
 glusterd_volume_count_get (void);
 int32_t
 glusterd_add_volume_to_dict (glusterd_volinfo_t *volinfo,
-                             dict_t  *dict, int32_t count);
+                             dict_t  *dict, int32_t count,
+                             char *prefix);
 int
 glusterd_get_brickinfo (xlator_t *this, const char *brickname,
                         int port, gf_boolean_t localhost,
@@ -269,12 +210,6 @@ glusterd_set_brick_status (glusterd_brickinfo_t  *brickinfo,
 gf_boolean_t
 glusterd_is_brick_started (glusterd_brickinfo_t  *brickinfo);
 
-int
-glusterd_friend_find_by_hostname (const char *hoststr,
-                                  glusterd_peerinfo_t  **peerinfo);
-int
-glusterd_hostname_to_uuid (char *hostname, uuid_t uuid);
-
 int
 glusterd_friend_brick_belongs (glusterd_volinfo_t *volinfo,
                                glusterd_brickinfo_t *brickinfo, void *uuid);
@@ -295,9 +230,7 @@ glusterd_is_defrag_on (glusterd_volinfo_t *volinfo);
 
 int32_t
 glusterd_volinfo_bricks_delete (glusterd_volinfo_t *volinfo);
-int
-glusterd_friend_find_by_uuid (uuid_t uuid,
-                              glusterd_peerinfo_t  **peerinfo);
+
 int
 glusterd_new_brick_validate (char *brick, glusterd_brickinfo_t *brickinfo,
                              char *op_errstr, size_t len);
@@ -312,17 +245,7 @@ glusterd_volume_brickinfo_get (uuid_t uuid, char *hostname, char *path,
 int
 glusterd_brickinfo_get (uuid_t uuid, char *hostname, char *path,
                        glusterd_brickinfo_t **brickinfo);
-int
-glusterd_is_rb_started (glusterd_volinfo_t *volinfo);
 
-int
-glusterd_is_rb_paused (glusterd_volinfo_t *volinfo);
-
-int
-glusterd_set_rb_status (glusterd_volinfo_t *volinfo, gf_rb_status_t status);
-
-gf_boolean_t
-glusterd_is_rb_ongoing (glusterd_volinfo_t *volinfo);
 
 int
 glusterd_rb_check_bricks (glusterd_volinfo_t *volinfo,
@@ -342,10 +265,6 @@ glusterd_sm_tr_log_transition_add (glusterd_sm_tr_log_t *log,
                                            int old_state, int new_state,
                                            int event);
 int
-glusterd_peerinfo_new (glusterd_peerinfo_t **peerinfo,
-                       glusterd_friend_sm_state_t state, uuid_t *uuid,
-                       const char *hostname, int port);
-int
 glusterd_sm_tr_log_init (glusterd_sm_tr_log_t *log,
                          char * (*state_name_get) (int),
                          char * (*event_name_get) (int),
@@ -357,14 +276,12 @@ int
 glusterd_sm_tr_log_add_to_dict (dict_t *dict,
                                 glusterd_sm_tr_log_t *circular_log);
 int
-glusterd_remove_pending_entry (struct list_head *list, void *elem);
+glusterd_remove_pending_entry (struct cds_list_head *list, void *elem);
 int
-glusterd_clear_pending_nodes (struct list_head *list);
-gf_boolean_t
-glusterd_peerinfo_is_uuid_unknown (glusterd_peerinfo_t *peerinfo);
+glusterd_clear_pending_nodes (struct cds_list_head *list);
 int32_t
 glusterd_brick_connect (glusterd_volinfo_t  *volinfo,
-                        glusterd_brickinfo_t  *brickinfo);
+                        glusterd_brickinfo_t  *brickinfo, char *socketpath);
 int32_t
 glusterd_brick_disconnect (glusterd_brickinfo_t *brickinfo);
 int32_t
@@ -372,17 +289,21 @@ glusterd_delete_volume (glusterd_volinfo_t *volinfo);
 int32_t
 glusterd_delete_brick (glusterd_volinfo_t* volinfo,
                        glusterd_brickinfo_t *brickinfo);
+
 int32_t
 glusterd_delete_all_bricks (glusterd_volinfo_t* volinfo);
+
 int
 glusterd_spawn_daemons (void *opaque);
+
 int
 glusterd_restart_gsyncds (glusterd_conf_t *conf);
+
 int
 glusterd_start_gsync (glusterd_volinfo_t *master_vol, char *slave,
                       char *path_list, char *conf_path,
                       char *glusterd_uuid_str,
-                      char **op_errstr);
+                      char **op_errstr, gf_boolean_t is_pause);
 int
 glusterd_get_local_brickpaths (glusterd_volinfo_t *volinfo,
                                char **pathlist);
@@ -424,20 +345,17 @@ glusterd_is_volume_replicate (glusterd_volinfo_t *volinfo);
 gf_boolean_t
 glusterd_is_brick_decommissioned (glusterd_volinfo_t *volinfo, char *hostname,
                                   char *path);
-gf_boolean_t
+int
 glusterd_friend_contains_vol_bricks (glusterd_volinfo_t *volinfo,
                                      uuid_t friend_uuid);
 int
 glusterd_friend_remove_cleanup_vols (uuid_t uuid);
 
-gf_boolean_t
-glusterd_chk_peers_connected_befriended (uuid_t skip_uuid);
-
-void
+int
 glusterd_get_client_filepath (char *filepath,
                               glusterd_volinfo_t *volinfo,
                               gf_transport_type type);
-void
+int
 glusterd_get_trusted_client_filepath (char *filepath,
                                       glusterd_volinfo_t *volinfo,
                                       gf_transport_type type);
@@ -452,8 +370,8 @@ int
 glusterd_add_node_to_dict (char *server, dict_t *dict, int count,
                            dict_t *vol_opts);
 
-char *
-glusterd_uuid_to_hostname (uuid_t uuid);
+int
+glusterd_calc_dist_leaf_count (int rcount, int scount);
 
 int
 glusterd_get_dist_leaf_count (glusterd_volinfo_t *volinfo);
@@ -507,13 +425,10 @@ int32_t
 glusterd_handle_node_rsp (dict_t *req_ctx, void *pending_entry,
                           glusterd_op_t op, dict_t *rsp_dict, dict_t *op_ctx,
                           char **op_errstr, gd_node_type type);
-int
-glusterd_volume_rebalance_use_rsp_dict (dict_t *aggr, dict_t *rsp_dict);
-int
-glusterd_volume_heal_use_rsp_dict (dict_t *aggr, dict_t *rsp_dict);
 
 int32_t
 glusterd_check_if_quota_trans_enabled (glusterd_volinfo_t *volinfo);
+
 int
 glusterd_volume_quota_copy_to_op_ctx_dict (dict_t *aggr, dict_t *rsp);
 int
@@ -524,37 +439,17 @@ glusterd_profile_volume_brick_rsp (void *pending_entry,
                                    dict_t *rsp_dict, dict_t *op_ctx,
                                    char **op_errstr, gd_node_type type);
 
-gf_boolean_t
-glusterd_are_vol_all_peers_up (glusterd_volinfo_t *volinfo,
-                               struct list_head *peers,
-                               char **down_peerstr);
+int32_t
+glusterd_set_originator_uuid (dict_t *dict);
 
 /* Should be used only when an operation is in progress, as that is the only
  * time a lock_owner is set
  */
 gf_boolean_t
-is_origin_glusterd ();
-
-gf_boolean_t
-glusterd_is_quorum_changed (dict_t *options, char *option, char *value);
-
-int
-glusterd_do_quorum_action ();
-
-int
-glusterd_get_quorum_cluster_counts (xlator_t *this, int *active_count,
-                                    int *quorum_count);
+is_origin_glusterd (dict_t *dict);
 
 int
 glusterd_get_next_global_opt_version_str (dict_t *opts, char **version_str);
-gf_boolean_t
-glusterd_is_quorum_option (char *option);
-gf_boolean_t
-glusterd_is_volume_in_server_quorum (glusterd_volinfo_t *volinfo);
-gf_boolean_t
-glusterd_is_any_volume_in_server_quorum (xlator_t *this);
-gf_boolean_t
-does_gd_meet_server_quorum (xlator_t *this);
 
 int
 glusterd_generate_and_set_task_id (dict_t *dict, char *key);
@@ -572,33 +467,35 @@ glusterd_is_same_address (char *name1, char *name2);
 void
 gd_update_volume_op_versions (glusterd_volinfo_t *volinfo);
 
-char*
-gd_peer_uuid_str (glusterd_peerinfo_t *peerinfo);
+int
+op_version_check (xlator_t *this, int min_op_version, char *msg, int msglen);
 
 gf_boolean_t
 gd_is_remove_brick_committed (glusterd_volinfo_t *volinfo);
 
-gf_boolean_t
-glusterd_are_vol_all_peers_up (glusterd_volinfo_t *volinfo,
-                               struct list_head *peers,
-                               char **down_peerstr);
-
 int
-glusterd_get_slave_details_confpath (glusterd_volinfo_t *volinfo, dict_t *dict,
-                                     char **slave_ip, char **slave_vol,
+glusterd_get_slave_details_confpath (glusterd_volinfo_t *volinfo,
+                                     dict_t *dict, char **slave_url,
+                                     char **slave_host, char **slave_vol,
                                      char **conf_path, char **op_errstr);
 
 int
-glusterd_get_slave_info (char *slave, char **slave_ip,
+glusterd_get_slave_info (char *slave,
+                         char **slave_url, char **hostname,
                          char **slave_vol, char **op_errstr);
 
 int
 glusterd_get_statefile_name (glusterd_volinfo_t *volinfo, char *slave,
-                             char *conf_path, char **statefile);
+                             char *conf_path, char **statefile,
+                             gf_boolean_t *is_template_in_use);
 
 int
 glusterd_gsync_read_frm_status (char *path, char *buf, size_t blen);
 
+int
+glusterd_create_status_file (char *master, char *slave, char *slave_url,
+                             char *slave_vol, char *status);
+
 int
 glusterd_check_restart_gsync_session (glusterd_volinfo_t *volinfo, char *slave,
                                       dict_t *resp_dict, char *path_list,
@@ -618,21 +515,18 @@ gd_should_i_start_rebalance  (glusterd_volinfo_t *volinfo);
 int
 glusterd_is_volume_quota_enabled (glusterd_volinfo_t *volinfo);
 
+int
+glusterd_is_volume_inode_quota_enabled (glusterd_volinfo_t *volinfo);
+
+int
+glusterd_is_bitrot_enabled (glusterd_volinfo_t *volinfo);
+
 gf_boolean_t
 glusterd_all_volumes_with_quota_stopped ();
 
-int
-glusterd_reconfigure_quotad ();
-
 void
 glusterd_clean_up_quota_store (glusterd_volinfo_t *volinfo);
 
-int
-glusterd_store_quota_conf_skip_header (xlator_t *this, int fd);
-
-int
-glusterd_store_quota_conf_stamp_header (xlator_t *this, int fd);
-
 int
 glusterd_remove_auxiliary_mount (char *volname);
 
@@ -645,6 +539,127 @@ gd_stop_rebalance_process (glusterd_volinfo_t *volinfo);
 rpc_clnt_t *
 glusterd_rpc_clnt_unref (glusterd_conf_t *conf, rpc_clnt_t *rpc);
 
+int32_t
+glusterd_compare_volume_name(struct cds_list_head *, struct cds_list_head *);
+
+char*
+glusterd_get_brick_mount_device (char *brick_path);
+
+struct mntent *
+glusterd_get_mnt_entry_info (char *mnt_pt, char *buff, int buflen,
+                             struct mntent *entry_ptr);
+
+int
+glusterd_get_brick_root (char *path, char **mount_point);
+
+int32_t
+glusterd_lvm_snapshot_remove (dict_t *rsp_dict, glusterd_volinfo_t *snap_vol);
+
+gf_boolean_t
+gd_vol_is_geo_rep_active (glusterd_volinfo_t *volinfo);
+
+int32_t
+glusterd_get_brick_mount_dir (char *brickpath, char *hostname, char *mount_dir);
+
+int32_t
+glusterd_aggr_brick_mount_dirs (dict_t *aggr, dict_t *rsp_dict);
+
+int32_t
+glusterd_take_lvm_snapshot (glusterd_brickinfo_t *brickinfo,
+                            char *origin_brick_path);
+
 void
 glusterd_launch_synctask (synctask_fn_t fn, void *opaque);
+
+int
+glusterd_enable_default_options (glusterd_volinfo_t *volinfo, char *option);
+
+int
+glusterd_unlink_file (char *sock_file_path);
+
+int32_t
+glusterd_find_brick_mount_path (char *brick_path, char **brick_mount_path);
+
+/*
+ * Function to retrieve list of snap volnames and their uuids
+ */
+int glusterd_snapshot_get_volnames_uuids (dict_t *dict,
+           char *volname, gf_getsnap_name_uuid_rsp *snap_info_rsp);
+
+int
+glusterd_update_mntopts (char *brick_path, glusterd_brickinfo_t *brickinfo);
+
+int
+glusterd_update_fs_label (glusterd_brickinfo_t *brickinfo);
+
+int
+glusterd_get_volopt_content (dict_t *dict, gf_boolean_t xml_out);
+
+int
+glusterd_get_default_val_for_volopt (dict_t *dict, gf_boolean_t all_opts,
+                                     char *key, char *orig_key,
+                                     dict_t *vol_dict, char **err_str);
+
+int
+glusterd_check_client_op_version_support (char *volname, uint32_t op_version,
+                                          char **op_errstr);
+
+gf_boolean_t
+glusterd_have_peers ();
+
+void
+glusterd_get_rebalance_volfile (glusterd_volinfo_t *volinfo,
+                                char *path, int path_len);
+
+int32_t
+glusterd_brickinfo_dup (glusterd_brickinfo_t *brickinfo,
+                        glusterd_brickinfo_t *dup_brickinfo);
+
+int
+glusterd_vol_add_quota_conf_to_dict (glusterd_volinfo_t *volinfo, dict_t *load,
+                                     int vol_idx, char *prefix);
+
+int32_t
+glusterd_import_volinfo (dict_t *peer_data, int count,
+                         glusterd_volinfo_t **volinfo,
+                         char *prefix);
+
+int
+glusterd_import_quota_conf (dict_t *peer_data, int vol_idx,
+                            glusterd_volinfo_t *new_volinfo,
+                            char *prefix);
+
+gf_boolean_t
+glusterd_is_shd_compatible_volume (glusterd_volinfo_t *volinfo);
+
+
+gf_boolean_t
+glusterd_are_all_volumes_stopped ();
+
+gf_boolean_t
+glusterd_all_shd_compatible_volumes_stopped ();
+
+void
+glusterd_nfs_pmap_deregister ();
+
+gf_boolean_t
+glusterd_is_volume_started (glusterd_volinfo_t  *volinfo);
+
+void
+glusterd_list_add_order (struct cds_list_head *new, struct cds_list_head *head,
+                        int (*compare)(struct cds_list_head *,
+                                       struct cds_list_head *));
+
+struct rpc_clnt*
+glusterd_defrag_rpc_get (glusterd_defrag_info_t *defrag);
+
+struct rpc_clnt*
+glusterd_defrag_rpc_put (glusterd_defrag_info_t *defrag);
+
+int
+glusterd_disallow_op_for_tier (glusterd_volinfo_t *volinfo, glusterd_op_t op,
+                               int cmd);
+int32_t
+glusterd_count_connected_peers (int32_t *count);
+
 #endif
