@@ -11,13 +11,17 @@
 #define _KMEMUSER
 #endif
 
-#include "fuse-bridge.h"
 #if defined(GF_SOLARIS_HOST_OS)
 #include <sys/procfs.h>
-#else
+#elif defined(__FreeBSD__)
+#include <sys/types.h>
+#include <libutil.h>
+#include <sys/user.h>
+#elif defined(CTL_KERN)
 #include <sys/sysctl.h>
 #endif
 
+#include "fuse-bridge.h"
 
 static void
 fuse_resolve_wipe (fuse_resolve_t *resolve)
@@ -26,8 +30,6 @@ fuse_resolve_wipe (fuse_resolve_t *resolve)
 
         GF_FREE ((void *)resolve->bname);
 
-        GF_FREE ((void *)resolve->resolved);
-
         if (resolve->fd)
                 fd_unref (resolve->fd);
 
@@ -154,7 +156,8 @@ frame_fill_groups (call_frame_t *frame)
         char        *endptr = NULL;
         int          ret = 0;
 
-        ret = snprintf (filename, sizeof filename, "/proc/%d/status", frame->root->pid);
+        ret = snprintf (filename, sizeof filename, "/proc/%d/status",
+                        frame->root->pid);
         if (ret >= sizeof filename)
                 goto out;
 
@@ -162,8 +165,8 @@ frame_fill_groups (call_frame_t *frame)
         if (!fp)
                 goto out;
 
-	if (call_stack_alloc_groups (frame->root, FUSE_MAX_AUX_GROUPS) != 0)
-		goto out;
+        if (call_stack_alloc_groups (frame->root, FUSE_MAX_AUX_GROUPS) != 0)
+                goto out;
 
         while ((ptr = fgets (line, sizeof line, fp))) {
                 if (strncmp (ptr, "Groups:", 7) != 0)
@@ -197,7 +200,7 @@ out:
         prcred_t    *prcred = (prcred_t *) scratch;
         FILE        *fp = NULL;
         int          ret = 0;
-	int          ngrps;
+        int          ngrps;
 
         ret = snprintf (filename, sizeof filename,
                         "/proc/%d/cred", frame->root->pid);
@@ -207,21 +210,21 @@ out:
                 if (fp != NULL) {
                         if (fgets (scratch, sizeof scratch, fp) != NULL) {
                                 ngrps = MIN(prcred->pr_ngroups,
-					    GF_MAX_AUX_GROUPS);
-				if (call_stack_alloc_groups (frame->root,
-							     ngrps) != 0)
-					return;
+                                            FUSE_MAX_AUX_GROUPS);
+                                if (call_stack_alloc_groups (frame->root,
+                                                             ngrps) != 0)
+                                        return;
                         }
                         fclose (fp);
                  }
          }
 #elif defined(CTL_KERN) /* DARWIN and *BSD */
-        /* 
+        /*
            N.B. CTL_KERN is an enum on Linux. (Meaning, if it's not
-           obvious, that it's not subject to preprocessor directives 
+           obvious, that it's not subject to preprocessor directives
            like '#if defined'.)
            Unlike Linux, on Mac OS and the BSDs it is a #define. We
-           could test to see that KERN_PROC is defined, but, barring any 
+           could test to see that KERN_PROC is defined, but, barring any
            evidence to the contrary, I think that's overkill.
            We might also test that GF_DARWIN_HOST_OS is defined, why
            limit this to just Mac OS. It's equally valid for the BSDs
@@ -235,9 +238,9 @@ out:
 
         if (sysctl(name, namelen, &kp, &kplen, NULL, 0) != 0)
                 return;
-        ngroups = MIN(kp.kp_eproc.e_ucred.cr_ngroups, GF_MAX_AUX_GROUPS);
-	if (call_stack_alloc_groups (frame->root, ngroups) != 0)
-		return;
+        ngroups = MIN(kp.kp_eproc.e_ucred.cr_ngroups, NGROUPS_MAX);
+        if (call_stack_alloc_groups (frame->root, ngroups) != 0)
+                return;
         for (i = 0; i < ngroups; i++)
                 frame->root->groups[i] = kp.kp_eproc.e_ucred.cr_groups[i];
         frame->root->ngrps = ngroups;
@@ -256,16 +259,16 @@ static void get_groups(fuse_private_t *priv, call_frame_t *frame)
 	const gid_list_t *gl;
 	gid_list_t agl;
 
+	if (!priv || !priv->gid_cache_timeout) {
+		frame_fill_groups(frame);
+		return;
+	}
+
         if (-1 == priv->gid_cache_timeout) {
                 frame->root->ngrps = 0;
                 return;
         }
 
-	if (!priv->gid_cache_timeout) {
-		frame_fill_groups(frame);
-		return;
-	}
-
 	gl = gid_cache_lookup(&priv->gid_cache, frame->root->pid,
 			      frame->root->uid, frame->root->gid);
 	if (gl) {
@@ -323,7 +326,7 @@ get_call_frame_for_req (fuse_state_t *state)
                                           state->lk_owner);
         }
 
-	get_groups(priv, frame);
+        get_groups(priv, frame);
 
         if (priv && priv->client_pid_set)
                 frame->root->pid = priv->client_pid;
@@ -383,11 +386,11 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                         parent = fuse_ino_to_inode (par, state->this);
                         loc->parent = parent;
                         if (parent)
-                                uuid_copy (loc->pargfid, parent->gfid);
+                                gf_uuid_copy (loc->pargfid, parent->gfid);
                 }
 
                 inode = loc->inode;
-                if (!inode) {
+                if (!inode && parent) {
                         inode = inode_grep (parent->table, parent, name);
                         loc->inode = inode;
                 }
@@ -406,7 +409,7 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                         inode = fuse_ino_to_inode (ino, state->this);
                         loc->inode = inode;
                         if (inode)
-                                uuid_copy (loc->gfid, inode->gfid);
+                                gf_uuid_copy (loc->gfid, inode->gfid);
                 }
 
                 parent = loc->parent;
@@ -414,7 +417,7 @@ fuse_loc_fill (loc_t *loc, fuse_state_t *state, ino_t ino,
                         parent = inode_parent (inode, null_gfid, NULL);
                         loc->parent = parent;
                         if (parent)
-                                uuid_copy (loc->pargfid, parent->gfid);
+                                gf_uuid_copy (loc->pargfid, parent->gfid);
 
                 }
 
@@ -452,9 +455,6 @@ fail:
         return ret;
 }
 
-/* Use the same logic as the Linux NFS-client */
-#define GF_FUSE_SQUASH_INO(ino) ((uint32_t) ino) ^ (ino >> 32)
-
 /* courtesy of folly */
 void
 gf_fuse_stat2attr (struct iatt *st, struct fuse_attr *fa, gf_boolean_t enable_ino32)
@@ -597,6 +597,8 @@ fuse_ignore_xattr_set (fuse_private_t *priv, char *key)
                            key, FNM_PERIOD) == 0)
               || (fnmatch ("*.glusterfs.volume-mark.*",
                            key, FNM_PERIOD) == 0)
+              || (fnmatch ("system.posix_acl_access",
+                           key, FNM_PERIOD) == 0)
               || (fnmatch ("glusterfs.gfid.newfile",
                            key, FNM_PERIOD) == 0)))
                 ret = -1;
