@@ -10,6 +10,7 @@
 
 #include "quick-read.h"
 #include "statedump.h"
+#include "quick-read-messages.h"
 
 qr_inode_t *qr_inode_ctx_get (xlator_t *this, inode_t *inode);
 void __qr_inode_prune (qr_inode_table_t *table, qr_inode_t *qr_inode);
@@ -101,6 +102,7 @@ qr_inode_ctx_get_or_new (xlator_t *this, inode_t *inode)
 		if (ret) {
 			__qr_inode_prune (&priv->table, qr_inode);
 			GF_FREE (qr_inode);
+                        qr_inode = NULL;
 		}
 	}
 unlock:
@@ -407,6 +409,11 @@ qr_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 goto out;
 	}
 
+        if (dict_get (xdata, GLUSTERFS_BAD_INODE)) {
+                qr_inode_prune (this, inode);
+                goto out;
+        }
+
 	if (dict_get (xdata, "sh-failed")) {
 		qr_inode_prune (this, inode);
 		goto out;
@@ -417,10 +424,11 @@ qr_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	if (content) {
 		/* new content came along, always replace old content */
 		qr_inode = qr_inode_ctx_get_or_new (this, inode);
-		if (!qr_inode)
+		if (!qr_inode) {
 			/* no harm done */
+			GF_FREE (content);
 			goto out;
-
+		}
 		qr_content_update (this, qr_inode, content, buf);
 	} else {
 		/* purge old content if necessary */
@@ -469,8 +477,9 @@ qr_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 		ret = dict_set (xdata, GF_CONTENT_KEY,
 				data_from_uint64 (conf->max_file_size));
 	if (ret)
-		gf_log (this->name, GF_LOG_WARNING,
-			"cannot set key in request dict (%s)",
+		gf_msg (this->name, GF_LOG_WARNING, 0,
+			QUICK_READ_MSG_DICT_SET_FAILED,
+                        "cannot set key in request dict (%s)",
 			loc->path);
 wind:
 	frame->local = inode_ref (loc->inode);
@@ -543,8 +552,6 @@ qr_readv_cached (call_frame_t *frame, qr_inode_t *qr_inode, size_t size,
 
 	LOCK (&table->lock);
 	{
-		op_ret = -1;
-
 		if (!qr_inode->data)
 			goto unlock;
 
@@ -565,7 +572,6 @@ qr_readv_cached (call_frame_t *frame, qr_inode_t *qr_inode, size_t size,
 		iobref = iobref_new ();
 		if (!iobref) {
 			op_ret = -1;
-			iobuf_unref (iobuf);
 			goto unlock;
 		}
 
@@ -581,19 +587,18 @@ qr_readv_cached (call_frame_t *frame, qr_inode_t *qr_inode, size_t size,
 unlock:
 	UNLOCK (&table->lock);
 
-	if (op_ret > 0) {
+	if (op_ret >= 0) {
 		iov.iov_base = iobuf->ptr;
 		iov.iov_len = op_ret;
 
 		STACK_UNWIND_STRICT (readv, frame, op_ret, 0, &iov, 1,
 				     &buf, iobref, xdata);
 	}
+        if (iobuf)
+                iobuf_unref (iobuf);
 
-	if (iobuf)
-		iobuf_unref (iobuf);
-
-	if (iobref)
-		iobref_unref (iobref);
+        if (iobref)
+	        iobref_unref (iobref);
 
 	return op_ret;
 }
@@ -609,7 +614,7 @@ qr_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 	if (!qr_inode)
 		goto wind;
 
-	if (qr_readv_cached (frame, qr_inode, size, offset, flags, xdata) <= 0)
+	if (qr_readv_cached (frame, qr_inode, size, offset, flags, xdata) < 0)
 		goto wind;
 
 	return 0;
@@ -786,8 +791,9 @@ mem_acct_init (xlator_t *this)
         ret = xlator_mem_acct_init (this, gf_qr_mt_end + 1);
 
         if (ret != 0) {
-                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
-                        "failed");
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                        QUICK_READ_MSG_NO_MEMORY,
+                        "Memory accounting init failed");
                 return ret;
         }
 
@@ -807,7 +813,8 @@ check_cache_size_ok (xlator_t *this, int64_t cache_size)
         opt = xlator_volume_option_get (this, "cache-size");
         if (!opt) {
                 ret = _gf_false;
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        QUICK_READ_MSG_INVALID_ARGUMENT,
                         "could not get cache-size option");
                 goto out;
         }
@@ -818,11 +825,12 @@ check_cache_size_ok (xlator_t *this, int64_t cache_size)
         else
                 max_cache_size = total_mem;
 
-        gf_log (this->name, GF_LOG_DEBUG, "Max cache size is %"PRIu64,
-                max_cache_size);
+        gf_msg_debug (this->name, 0, "Max cache size is %"PRIu64,
+                      max_cache_size);
         if (cache_size > max_cache_size) {
                 ret = _gf_false;
-                gf_log (this->name, GF_LOG_ERROR, "Cache size %"PRIu64
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        QUICK_READ_MSG_INVALID_ARGUMENT, "Cache size %"PRIu64
                         " is greater than the max size of %"PRIu64,
                         cache_size, max_cache_size);
                 goto out;
@@ -853,10 +861,11 @@ reconfigure (xlator_t *this, dict_t *options)
         GF_OPTION_RECONF ("cache-timeout", conf->cache_timeout, options, int32,
                           out);
 
-        GF_OPTION_RECONF ("cache-size", cache_size_new, options, size, out);
+        GF_OPTION_RECONF ("cache-size", cache_size_new, options, size_uint64, out);
         if (!check_cache_size_ok (this, cache_size_new)) {
                 ret = -1;
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        QUICK_READ_MSG_INVALID_CONFIG,
                         "Not reconfiguring cache-size");
                 goto out;
         }
@@ -925,10 +934,9 @@ qr_get_priority_list (const char *opt_str, struct list_head *first)
                         goto out;
                 }
 
-                gf_log ("quick-read", GF_LOG_TRACE,
-                        "quick-read priority : pattern %s : priority %s",
-                        pattern,
-                        priority);
+                gf_msg_trace ("quick-read", 0,
+                              "quick-read priority : pattern %s : priority %s",
+                              pattern, priority);
 
                 curr->pattern = gf_strdup (pattern);
                 if (curr->pattern == NULL) {
@@ -974,14 +982,16 @@ init (xlator_t *this)
         qr_conf_t    *conf = NULL;
 
         if (!this->children || this->children->next) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        QUICK_READ_MSG_XLATOR_CHILD_MISCONFIGURED,
                         "FATAL: volume (%s) not configured with exactly one "
                         "child", this->name);
                 return -1;
         }
 
         if (!this->parents) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        QUICK_READ_MSG_VOL_MISCONFIGURED,
                         "dangling volume. check volfile ");
         }
 
@@ -994,11 +1004,11 @@ init (xlator_t *this)
         LOCK_INIT (&priv->table.lock);
         conf = &priv->conf;
 
-        GF_OPTION_INIT ("max-file-size", conf->max_file_size, size, out);
+        GF_OPTION_INIT ("max-file-size", conf->max_file_size, size_uint64, out);
 
         GF_OPTION_INIT ("cache-timeout", conf->cache_timeout, int32, out);
 
-        GF_OPTION_INIT ("cache-size", conf->cache_size, size, out);
+        GF_OPTION_INIT ("cache-size", conf->cache_size, size_uint64, out);
         if (!check_cache_size_ok (this, conf->cache_size)) {
                 ret = -1;
                 goto out;
@@ -1009,8 +1019,8 @@ init (xlator_t *this)
         if (dict_get (this->options, "priority")) {
                 char *option_list = data_to_str (dict_get (this->options,
                                                            "priority"));
-                gf_log (this->name, GF_LOG_TRACE,
-                        "option path %s", option_list);
+                gf_msg_trace (this->name, 0,
+                              "option path %s", option_list);
                 /* parse the list of pattern:priority */
                 conf->max_pri = qr_get_priority_list (option_list,
                                                       &conf->priority_list);
@@ -1053,7 +1063,13 @@ qr_inode_table_destroy (qr_private_t *priv)
         conf = &priv->conf;
 
         for (i = 0; i < conf->max_pri; i++) {
-                GF_ASSERT (list_empty (&priv->table.lru[i]));
+                /* There is a known leak of inodes, hence until
+                 * that is fixed, log the assert as warning.
+                GF_ASSERT (list_empty (&priv->table.lru[i]));*/
+                if (!list_empty (&priv->table.lru[i])) {
+                        gf_log ("quick-read", GF_LOG_INFO,
+                               "quick read inode table lru not empty");
+                }
         }
 
         LOCK_DESTROY (&priv->table.lock);
