@@ -15,6 +15,7 @@
 
 #include "server.h"
 #include "server-helpers.h"
+#include "server-messages.h"
 
 
 int
@@ -42,18 +43,7 @@ resolve_loc_touchup (call_frame_t *frame)
         resolve = state->resolve_now;
         loc     = state->loc_now;
 
-        if (!loc->path) {
-                if (loc->parent && resolve->bname) {
-                        ret = inode_path (loc->parent, resolve->bname, &path);
-                } else if (loc->inode) {
-                        ret = inode_path (loc->inode, NULL, &path);
-                }
-                if (ret)
-                        gf_log (frame->this->name, GF_LOG_TRACE,
-                                "return value inode_path %d", ret);
-                loc->path = path;
-        }
-
+        loc_touchup (loc, resolve->bname);
         return 0;
 }
 
@@ -74,11 +64,18 @@ resolve_gfid_entry_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         resolve_loc = &resolve->resolve_loc;
 
         if (op_ret == -1) {
-                gf_log (this->name, ((op_errno == ENOENT) ? GF_LOG_DEBUG :
-                                     GF_LOG_WARNING),
-                        "%s/%s: failed to resolve (%s)",
-                        uuid_utoa (resolve_loc->pargfid), resolve_loc->name,
-                        strerror (op_errno));
+                if (op_errno == ENOENT) {
+                        gf_msg_debug (this->name, 0, "%s/%s: failed to resolve"
+                                      " (%s)",
+                                      uuid_utoa (resolve_loc->pargfid),
+                                      resolve_loc->name, strerror (op_errno));
+                } else {
+                        gf_msg (this->name, GF_LOG_WARNING, op_errno,
+                                PS_MSG_GFID_RESOLVE_FAILED, "%s/%s: failed to "
+                                "resolve (%s)",
+                                uuid_utoa (resolve_loc->pargfid),
+                                resolve_loc->name, strerror (op_errno));
+                }
                 goto out;
         }
 
@@ -109,47 +106,80 @@ resolve_gfid_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         server_resolve_t     *resolve = NULL;
         inode_t              *link_inode = NULL;
         loc_t                *resolve_loc = NULL;
+        dict_t               *dict = NULL;
 
         state = CALL_STATE (frame);
         resolve = state->resolve_now;
         resolve_loc = &resolve->resolve_loc;
 
         if (op_ret == -1) {
-                gf_log (this->name, ((op_errno == ENOENT) ? GF_LOG_DEBUG :
-                                     GF_LOG_WARNING),
-                        "%s: failed to resolve (%s)",
-                        uuid_utoa (resolve_loc->gfid), strerror (op_errno));
+                if (op_errno == ENOENT) {
+                        gf_msg_debug (this->name, GF_LOG_DEBUG,
+                                      "%s: failed to resolve (%s)",
+                                      uuid_utoa (resolve_loc->gfid),
+                                      strerror (op_errno));
+                } else {
+                        gf_msg (this->name, GF_LOG_WARNING, op_errno,
+                                PS_MSG_GFID_RESOLVE_FAILED,
+                                "%s: failed to resolve (%s)",
+                                uuid_utoa (resolve_loc->gfid),
+                                strerror (op_errno));
+                }
                 loc_wipe (&resolve->resolve_loc);
                 goto out;
         }
 
-        loc_wipe (resolve_loc);
-
         link_inode = inode_link (inode, NULL, NULL, buf);
 
-        if (!link_inode)
+        if (!link_inode) {
+                loc_wipe (resolve_loc);
                 goto out;
+        }
 
         inode_lookup (link_inode);
 
-        if (uuid_is_null (resolve->pargfid)) {
+        /* wipe the loc only after the inode has been linked to the inode
+           table. Otherwise before inode gets linked to the inode table,
+           inode would have been unrefed (this might have been destroyed
+           if refcount becomes 0, and put back to mempool). So once the
+           inode gets destroyed, inode_link is a redundant operation. But
+           without knowing that the destroyed inode's pointer is saved in
+           the resolved_loc as parent (while constructing loc for resolving
+           the entry) and the inode_new call for resolving the entry will
+           return the same pointer to the inode as the parent (because in
+           reality the inode is a free inode present in cold list of the
+           inode mem-pool).
+        */
+        loc_wipe (resolve_loc);
+
+        if (gf_uuid_is_null (resolve->pargfid)) {
                 inode_unref (link_inode);
                 goto out;
         }
 
         resolve_loc->parent = link_inode;
-        uuid_copy (resolve_loc->pargfid, resolve_loc->parent->gfid);
+        gf_uuid_copy (resolve_loc->pargfid, resolve_loc->parent->gfid);
 
         resolve_loc->name = resolve->bname;
 
         resolve_loc->inode = inode_new (state->itable);
+
         inode_path (resolve_loc->parent, resolve_loc->name,
                     (char **) &resolve_loc->path);
 
+        dict = dict_copy_with_ref (state->xdata, NULL);
+        if (!dict && state->xdata)
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM, PS_MSG_NO_MEMORY,
+                        "BUG: dict allocation failed (pargfid: %s, name: %s), "
+                        "still continuing", uuid_utoa (resolve_loc->gfid),
+                        resolve_loc->name);
+
         STACK_WIND (frame, resolve_gfid_entry_cbk,
                     frame->root->client->bound_xl,
                     frame->root->client->bound_xl->fops->lookup,
-                    &resolve->resolve_loc, NULL);
+                    &resolve->resolve_loc, dict);
+        if (dict)
+                dict_unref (dict);
         return 0;
 out:
         resolve_continue (frame);
@@ -165,24 +195,36 @@ resolve_gfid (call_frame_t *frame)
         server_resolve_t     *resolve = NULL;
         loc_t                *resolve_loc = NULL;
         int                   ret = 0;
+        dict_t               *xdata = NULL;
 
         state = CALL_STATE (frame);
         this  = frame->this;
         resolve = state->resolve_now;
         resolve_loc = &resolve->resolve_loc;
 
-        if (!uuid_is_null (resolve->pargfid))
-                uuid_copy (resolve_loc->gfid, resolve->pargfid);
-        else if (!uuid_is_null (resolve->gfid))
-                uuid_copy (resolve_loc->gfid, resolve->gfid);
+        if (!gf_uuid_is_null (resolve->pargfid))
+                gf_uuid_copy (resolve_loc->gfid, resolve->pargfid);
+        else if (!gf_uuid_is_null (resolve->gfid))
+                gf_uuid_copy (resolve_loc->gfid, resolve->gfid);
 
         resolve_loc->inode = inode_new (state->itable);
         ret = loc_path (resolve_loc, NULL);
 
+        xdata = dict_copy_with_ref (state->xdata, NULL);
+        if (!xdata && state->xdata)
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM, PS_MSG_NO_MEMORY,
+                        "BUG: dict allocation failed (gfid: %s), "
+                        "still continuing",
+                        uuid_utoa (resolve_loc->gfid));
+
         STACK_WIND (frame, resolve_gfid_cbk,
                     frame->root->client->bound_xl,
                     frame->root->client->bound_xl->fops->lookup,
-                    &resolve->resolve_loc, NULL);
+                    &resolve->resolve_loc, xdata);
+
+        if (xdata)
+                dict_unref (xdata);
+
         return 0;
 }
 
@@ -204,13 +246,13 @@ resolve_continue (call_frame_t *frame)
         if (resolve->fd_no != -1) {
                 ret = resolve_anonfd_simple (frame);
                 goto out;
-        } else if (!uuid_is_null (resolve->pargfid))
+        } else if (!gf_uuid_is_null (resolve->pargfid))
                 ret = resolve_entry_simple (frame);
-        else if (!uuid_is_null (resolve->gfid))
+        else if (!gf_uuid_is_null (resolve->gfid))
                 ret = resolve_inode_simple (frame);
         if (ret)
-                gf_log (this->name, GF_LOG_DEBUG,
-                        "return value of resolve_*_simple %d", ret);
+                gf_msg_debug (this->name, 0, "return value of resolve_*_"
+                              "simple %d", ret);
 
         resolve_loc_touchup (frame);
 out:
@@ -252,7 +294,7 @@ resolve_entry_simple (call_frame_t *frame)
         }
 
         /* expected @parent was found from the inode cache */
-        uuid_copy (state->loc_now->pargfid, resolve->pargfid);
+        gf_uuid_copy (state->loc_now->pargfid, resolve->pargfid);
         state->loc_now->parent = inode_ref (parent);
         state->loc_now->name = resolve->bname;
 
@@ -277,9 +319,9 @@ resolve_entry_simple (call_frame_t *frame)
         }
 
         if (resolve->type == RESOLVE_NOT) {
-                gf_log (this->name, GF_LOG_DEBUG, "inode (pointer: %p gfid:%s"
-                        " found for path (%s) while type is RESOLVE_NOT",
-                        inode, uuid_utoa (inode->gfid), resolve->path);
+                gf_msg_debug (this->name, 0, "inode (pointer: %p gfid:%s found"
+                              " for path (%s) while type is RESOLVE_NOT",
+                              inode, uuid_utoa (inode->gfid), resolve->path);
                 resolve->op_ret   = -1;
                 resolve->op_errno = EEXIST;
                 ret = -1;
@@ -338,6 +380,7 @@ resolve_inode_simple (call_frame_t *frame)
 
         state = CALL_STATE (frame);
         resolve = state->resolve_now;
+
         inode = inode_find (state->itable, resolve->gfid);
 
         if (!inode) {
@@ -350,7 +393,7 @@ resolve_inode_simple (call_frame_t *frame)
         ret = 0;
 
         state->loc_now->inode = inode_ref (inode);
-        uuid_copy (state->loc_now->gfid, resolve->gfid);
+        gf_uuid_copy (state->loc_now->gfid, resolve->gfid);
 
 out:
         if (inode)
@@ -415,9 +458,9 @@ out:
                 inode_unref (inode);
 
         if (ret != 0)
-                gf_log ("server", GF_LOG_WARNING, "inode for the gfid (%s) is "
-                        "not found. anonymous fd creation failed",
-                        uuid_utoa (resolve->gfid));
+                gf_msg_debug ("server", 0, "inode for the gfid (%s) is "
+                              "not found. anonymous fd creation failed",
+                              uuid_utoa (resolve->gfid));
         return ret;
 }
 
@@ -471,7 +514,8 @@ server_resolve_fd (call_frame_t *frame)
         serv_ctx = server_ctx_get (client, client->this);
 
         if (serv_ctx == NULL) {
-                gf_log ("", GF_LOG_INFO, "server_ctx_get() failed");
+                gf_msg ("", GF_LOG_INFO, ENOMEM, PS_MSG_NO_MEMORY,
+                        "server_ctx_get() failed");
                 resolve->op_ret   = -1;
                 resolve->op_errno = ENOMEM;
                 return 0;
@@ -480,7 +524,8 @@ server_resolve_fd (call_frame_t *frame)
         state->fd = gf_fd_fdptr_get (serv_ctx->fdtable, fd_no);
 
         if (!state->fd) {
-                gf_log ("", GF_LOG_INFO, "fd not found in context");
+                gf_msg ("", GF_LOG_INFO, EBADF, PS_MSG_FD_NOT_FOUND, "fd not "
+                        "found in context");
                 resolve->op_ret   = -1;
                 resolve->op_errno = EBADF;
         }
@@ -504,17 +549,18 @@ server_resolve (call_frame_t *frame)
 
                 server_resolve_fd (frame);
 
-        } else if (!uuid_is_null (resolve->pargfid)) {
+        } else if (!gf_uuid_is_null (resolve->pargfid)) {
 
                 server_resolve_entry (frame);
 
-        } else if (!uuid_is_null (resolve->gfid)) {
+        } else if (!gf_uuid_is_null (resolve->gfid)) {
 
                 server_resolve_inode (frame);
 
         } else {
                 if (resolve == &state->resolve)
-                        gf_log (frame->this->name, GF_LOG_WARNING,
+                        gf_msg (frame->this->name, GF_LOG_WARNING, 0,
+                                PS_MSG_INVALID_ENTRY,
                                 "no resolution type for %s (%s)",
                                 resolve->path, gf_fop_list[frame->root->op]);
 
@@ -575,8 +621,9 @@ server_resolve_all (call_frame_t *frame)
                 server_resolve_done (frame);
 
         } else {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "Invalid pointer for state->resolve_now");
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        PS_MSG_INVALID_ENTRY, "Invalid pointer for "
+                        "state->resolve_now");
         }
 
         return 0;
