@@ -25,6 +25,9 @@
 #include "posix.h"
 #include "xlator.h"
 #include "syscall.h"
+#include "posix-messages.h"
+
+#include "compat-errno.h"
 
 inode_t *
 posix_resolve (xlator_t *this, inode_table_t *itable, inode_t *parent,
@@ -70,12 +73,8 @@ posix_make_ancestral_node (const char *priv_base_path, char *path, int pathsize,
 
         if (type & POSIX_ANCESTRY_DENTRY) {
                 entry = gf_dirent_for_name (dir_name);
-                if (!entry) {
-                        gf_log (THIS->name, GF_LOG_ERROR,
-                                "could not create gf_dirent for entry %s: (%s)",
-                                dir_name, strerror (errno));
+                if (!entry)
                         goto out;
-                }
 
                 entry->d_stat = *iabuf;
                 entry->inode = inode_ref (inode);
@@ -85,10 +84,10 @@ posix_make_ancestral_node (const char *priv_base_path, char *path, int pathsize,
                 strcat (real_path, "/");
                 strcat (real_path, path);
                 loc.inode = inode_ref (inode);
-                uuid_copy (loc.gfid, inode->gfid);
+                gf_uuid_copy (loc.gfid, inode->gfid);
 
-                entry->dict = posix_lookup_xattr_fill (THIS, real_path, &loc,
-                                                       xdata, iabuf);
+                entry->dict = posix_xattr_fill (THIS, real_path, &loc, NULL, -1,
+                                                xdata, iabuf);
                 loc_wipe (&loc);
         }
 
@@ -117,7 +116,7 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
         int          ret        = -1;
         uuid_t       tmp_gfid   = {0, };
 
-        if (!path || !parent || !priv_base_path || uuid_is_null (gfid)) {
+        if (!path || !parent || !priv_base_path || gf_uuid_is_null (gfid)) {
                 goto out;
         }
 
@@ -133,7 +132,7 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
                 inode = itable->root;
 
                 memset (&iabuf, 0, sizeof (iabuf));
-                uuid_copy (iabuf.ia_gfid, inode->gfid);
+                gf_uuid_copy (iabuf.ia_gfid, inode->gfid);
                 iabuf.ia_type = inode->ia_type;
 
                 ret = posix_make_ancestral_node (priv_base_path, path, pathsize,
@@ -145,14 +144,14 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
         dir_handle = alloca (handle_size);
         linkname   = alloca (PATH_MAX);
         snprintf (dir_handle, handle_size, "%s/%s/%02x/%02x/%s",
-                  priv_base_path, HANDLE_PFX, gfid[0], gfid[1],
+                  priv_base_path, GF_HIDDEN_PATH, gfid[0], gfid[1],
                   uuid_utoa (gfid));
 
         len = readlink (dir_handle, linkname, PATH_MAX);
         if (len < 0) {
-                gf_log (this->name, GF_LOG_ERROR, "could not read the link "
-                        "from the gfid handle %s (%s)", dir_handle,
-                        strerror (errno));
+                gf_msg (this->name, GF_LOG_ERROR, errno, P_MSG_READLINK_FAILED,
+                        "could not read the link from the gfid handle %s ",
+                        dir_handle);
                 goto out;
         }
 
@@ -160,9 +159,8 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
 
         pgfidstr = strtok_r (linkname + SLEN("../../00/00/"), "/", &saveptr);
         dir_name = strtok_r (NULL, "/", &saveptr);
-        strcat (dir_name, "/");
 
-        uuid_parse (pgfidstr, tmp_gfid);
+        gf_uuid_parse (pgfidstr, tmp_gfid);
 
         ret = posix_make_ancestryfromgfid (this, path, pathsize, head, type,
                                            tmp_gfid, handle_size,
@@ -175,7 +173,12 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
         memset (&iabuf, 0, sizeof (iabuf));
 
         inode = posix_resolve (this, itable, *parent, dir_name, &iabuf);
+        if (inode == NULL) {
+                ret = -1;
+                goto out;
+        }
 
+        strcat (dir_name, "/");
         ret = posix_make_ancestral_node (priv_base_path, path, pathsize, head,
                                          dir_name, &iabuf, inode, type, xdata);
         if (*parent != NULL) {
@@ -239,9 +242,9 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
         /* is a directory's symlink-handle */
         ret = readlink (base_str, linkname, 512);
         if (ret == -1) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "internal readlink failed on %s (%s)",
-                        base_str, strerror (errno));
+                gf_msg (this->name, GF_LOG_ERROR, errno, P_MSG_READLINK_FAILED,
+                        "internal readlink failed on %s ",
+                        base_str);
                 goto err;
         }
 
@@ -258,14 +261,14 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
         }
 
         if (ret < 50 || ret >= 512) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_LINK_FAILED,
                         "malformed internal link %s for %s",
                         linkname, base_str);
                 goto err;
         }
 
         if (memcmp (linkname, "../../", 6) != 0) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
                         "malformed internal link %s for %s",
                         linkname, base_str);
                 goto err;
@@ -276,7 +279,7 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
             (linkname[8] != '/') ||
             (linkname[11] != '/') ||
             (linkname[48] != '/')) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
                         "malformed internal link %s for %s",
                         linkname, base_str);
                 goto err;
@@ -286,20 +289,27 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
             (linkname[25] != '-') ||
             (linkname[30] != '-') ||
             (linkname[35] != '-')) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
                         "malformed internal link %s for %s",
                         linkname, base_str);
                 goto err;
         }
 
         blen = link_len - 48;
+
+        if (len + blen >= maxlen) {
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
+                        "Unable to form handle path for %s (maxlen = %d)",
+                        buf, maxlen);
+                goto err;
+        }
+
         memmove (buf + base_len + blen, buf + base_len,
                  (strlen (buf) - base_len) + 1);
 
         strncpy (base_str + pfx_len, linkname + 6, 42);
 
-        if (len + blen < maxlen)
-                strncpy (buf + pfx_len, linkname + 6, link_len - 6);
+        strncpy (buf + pfx_len, linkname + 6, link_len - 6);
 out:
         return len + blen;
 err:
@@ -343,13 +353,13 @@ posix_handle_path (xlator_t *this, uuid_t gfid, const char *basename,
                 buf = alloca (maxlen);
         }
 
-        base_len = (priv->base_path_length + SLEN(HANDLE_PFX) + 45);
+        base_len = (priv->base_path_length + SLEN(GF_HIDDEN_PATH) + 45);
         base_str = alloca (base_len + 1);
         base_len = snprintf (base_str, base_len + 1, "%s/%s/%02x/%02x/%s",
-                             priv->base_path, HANDLE_PFX, gfid[0], gfid[1],
+                             priv->base_path, GF_HIDDEN_PATH, gfid[0], gfid[1],
                              uuid_str);
 
-        pfx_len = priv->base_path_length + 1 + SLEN(HANDLE_PFX) + 1;
+        pfx_len = priv->base_path_length + 1 + SLEN(GF_HIDDEN_PATH) + 1;
 
         if (basename) {
                 len = snprintf (buf, maxlen, "%s/%s", base_str, basename);
@@ -366,11 +376,11 @@ posix_handle_path (xlator_t *this, uuid_t gfid, const char *basename,
                 errno = 0;
                 ret = posix_handle_pump (this, buf, len, maxlen,
                                          base_str, base_len, pfx_len);
+                len = ret;
+
                 if (ret == -1)
                         break;
 
-                len = ret;
-
                 ret = lstat (buf, &stat);
         } while ((ret == -1) && errno == ELOOP);
 
@@ -391,7 +401,7 @@ posix_handle_gfid_path (xlator_t *this, uuid_t gfid, const char *basename,
 
         len = priv->base_path_length  /* option directory "/export" */
                 + SLEN("/")
-                + SLEN(HANDLE_PFX)
+                + SLEN(GF_HIDDEN_PATH)
                 + SLEN("/")
                 + SLEN("00/")
                 + SLEN("00/")
@@ -422,10 +432,10 @@ posix_handle_gfid_path (xlator_t *this, uuid_t gfid, const char *basename,
 
         if (basename) {
                 len = snprintf (buf, buflen, "%s/%s/%02x/%02x/%s/%s", priv->base_path,
-                                HANDLE_PFX, gfid[0], gfid[1], uuid_str, basename);
+                                GF_HIDDEN_PATH, gfid[0], gfid[1], uuid_str, basename);
         } else {
                 len = snprintf (buf, buflen, "%s/%s/%02x/%02x/%s", priv->base_path,
-                                HANDLE_PFX, gfid[0], gfid[1], uuid_str);
+                                GF_HIDDEN_PATH, gfid[0], gfid[1], uuid_str);
         }
 out:
         return len;
@@ -448,15 +458,16 @@ posix_handle_init (xlator_t *this)
 
         ret = stat (priv->base_path, &exportbuf);
         if (ret || !S_ISDIR (exportbuf.st_mode)) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        P_MSG_HANDLE_CREATE,
                         "Not a directory: %s", priv->base_path);
                 return -1;
         }
 
-        handle_pfx = alloca (priv->base_path_length + 1 + strlen (HANDLE_PFX)
+        handle_pfx = alloca (priv->base_path_length + 1 + strlen (GF_HIDDEN_PATH)
                              + 1);
 
-        sprintf (handle_pfx, "%s/%s", priv->base_path, HANDLE_PFX);
+        sprintf (handle_pfx, "%s/%s", priv->base_path, GF_HIDDEN_PATH);
 
         ret = stat (handle_pfx, &stbuf);
         switch (ret) {
@@ -464,21 +475,24 @@ posix_handle_init (xlator_t *this)
                 if (errno == ENOENT) {
                         ret = mkdir (handle_pfx, 0600);
                         if (ret != 0) {
-                                gf_log (this->name, GF_LOG_ERROR,
-                                        "Creating directory %s failed: %s",
-                                        handle_pfx, strerror (errno));
+                                gf_msg (this->name, GF_LOG_ERROR, errno,
+                                        P_MSG_HANDLE_CREATE,
+                                        "Creating directory %s failed",
+                                        handle_pfx);
                                 return -1;
                         }
                 } else {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "Checking for %s failed: %s",
-                                handle_pfx, strerror (errno));
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "Checking for %s failed",
+                                handle_pfx);
                         return -1;
                 }
                 break;
         case 0:
                 if (!S_ISDIR (stbuf.st_mode)) {
-                        gf_log (this->name, GF_LOG_ERROR,
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                P_MSG_HANDLE_CREATE,
                                 "Not a directory: %s",
                                 handle_pfx);
                         return -1;
@@ -496,25 +510,26 @@ posix_handle_init (xlator_t *this)
         switch (ret) {
         case -1:
                 if (errno != ENOENT) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "%s: %s", priv->base_path,
-                                strerror (errno));
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "%s", priv->base_path);
                         return -1;
                 }
 
                 ret = posix_handle_mkdir_hashes (this, rootstr);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "mkdir %s failed (%s)",
-                                rootstr, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "mkdir %s failed", rootstr);
                         return -1;
                 }
 
                 ret = symlink ("../../..", rootstr);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "symlink %s creation failed (%s)",
-                                rootstr, strerror (errno));
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "symlink %s creation failed",
+                                rootstr);
                         return -1;
                 }
                 break;
@@ -523,7 +538,8 @@ posix_handle_init (xlator_t *this)
                     (exportbuf.st_dev == rootbuf.st_dev))
                         return 0;
 
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        P_MSG_HANDLE_CREATE,
                         "Different dirs %s (%lld/%lld) != %s (%lld/%lld)",
                         priv->base_path, (long long) exportbuf.st_ino,
                         (long long) exportbuf.st_dev, rootstr,
@@ -547,7 +563,7 @@ posix_does_old_trash_exists (char *old_trash)
         ret = lstat (old_trash, &stbuf);
         if ((ret == 0) && S_ISDIR (stbuf.st_mode)) {
                 ret = sys_lgetxattr (old_trash, "trusted.gfid", gfid, 16);
-                if ((ret < 0) && (errno == ENODATA))
+                if ((ret < 0) && (errno == ENODATA || errno == ENOATTR) )
                         exists = _gf_true;
         }
         return exists;
@@ -565,18 +581,21 @@ posix_handle_new_trash_init (xlator_t *this, char *trash)
                 if (errno == ENOENT) {
                         ret = mkdir (trash, 0755);
                         if (ret != 0) {
-                                gf_log (this->name, GF_LOG_ERROR,
-                                        "Creating directory %s failed: %s",
-                                        trash, strerror (errno));
+                                gf_msg (this->name, GF_LOG_ERROR, errno,
+                                        P_MSG_HANDLE_TRASH_CREATE,
+                                        "Creating directory %s failed",
+                                        trash);
                         }
                 } else {
-                        gf_log (this->name, GF_LOG_ERROR, "Checking for %s "
-                                "failed: %s", trash, strerror (errno));
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_HANDLE_TRASH_CREATE,
+                                "Checking for %s failed", trash);
                 }
                 break;
         case 0:
                 if (!S_ISDIR (stbuf.st_mode)) {
-                        gf_log (this->name, GF_LOG_ERROR,
+                        gf_msg (this->name, GF_LOG_ERROR, errno,
+                                P_MSG_HANDLE_TRASH_CREATE,
                                 "Not a directory: %s", trash);
                         ret = -1;
                 }
@@ -596,13 +615,14 @@ posix_mv_old_trash_into_new_trash (xlator_t *this, char *old, char *new)
 
         if (!posix_does_old_trash_exists (old))
                 goto out;
-        uuid_generate (dest_name);
+        gf_uuid_generate (dest_name);
         snprintf (dest_old, sizeof (dest_old), "%s/%s", new,
                   uuid_utoa (dest_name));
         ret = rename (old, dest_old);
         if (ret < 0) {
-                gf_log (this->name, GF_LOG_ERROR, "Not able to move "
-                        "%s -> %s (%s)", old, dest_old, strerror (errno));
+                gf_msg (this->name, GF_LOG_ERROR, errno,
+                        P_MSG_HANDLE_TRASH_CREATE,
+                        "Not able to move %s -> %s ", old, dest_old);
         }
 out:
         return ret;
@@ -618,7 +638,7 @@ posix_handle_trash_init (xlator_t *this)
         priv = this->private;
 
         priv->trash_path = GF_CALLOC (1, priv->base_path_length + strlen ("/")
-                                      + strlen (HANDLE_PFX) + strlen ("/")
+                                      + strlen (GF_HIDDEN_PATH) + strlen ("/")
                                       + strlen (TRASH_DIR) + 1,
                                       gf_posix_mt_trash_path);
 
@@ -626,7 +646,7 @@ posix_handle_trash_init (xlator_t *this)
                 goto out;
 
         strncpy (priv->trash_path, priv->base_path, priv->base_path_length);
-        strcat (priv->trash_path, "/" HANDLE_PFX "/" TRASH_DIR);
+        strcat (priv->trash_path, "/" GF_HIDDEN_PATH "/" TRASH_DIR);
         ret = posix_handle_new_trash_init (this, priv->trash_path);
         if (ret)
                 goto out;
@@ -651,9 +671,9 @@ posix_handle_mkdir_hashes (xlator_t *this, const char *newpath)
 
         ret = mkdir (parpath, 0700);
         if (ret == -1 && errno != EEXIST) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "error mkdir hash-1 %s (%s)",
-                        parpath, strerror (errno));
+                gf_msg (this->name, GF_LOG_ERROR, errno,
+                        P_MSG_HANDLE_CREATE,
+                        "error mkdir hash-1 %s ", parpath);
                 return -1;
         }
 
@@ -662,9 +682,9 @@ posix_handle_mkdir_hashes (xlator_t *this, const char *newpath)
 
         ret = mkdir (parpath, 0700);
         if (ret == -1 && errno != EEXIST) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "error mkdir hash-2 %s (%s)",
-                        parpath, strerror (errno));
+                gf_msg (this->name, GF_LOG_ERROR, errno,
+                        P_MSG_HANDLE_CREATE,
+                        "error mkdir hash-2 %s ", parpath);
                 return -1;
         }
 
@@ -684,49 +704,43 @@ posix_handle_hard (xlator_t *this, const char *oldpath, uuid_t gfid, struct stat
 
         ret = lstat (newpath, &newbuf);
         if (ret == -1 && errno != ENOENT) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "%s: %s", newpath, strerror (errno));
+                gf_msg (this->name, GF_LOG_WARNING, errno,
+                        P_MSG_HANDLE_CREATE,
+                        "%s", newpath);
                 return -1;
         }
 
         if (ret == -1 && errno == ENOENT) {
                 ret = posix_handle_mkdir_hashes (this, newpath);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "mkdir %s failed (%s)",
-                                newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE, "mkdir %s failed ",
+                                newpath);
                         return -1;
                 }
 
-#ifdef HAVE_LINKAT
-                /*
-                 * Use linkat if the target may be a symlink to a directory
-                 * or without an existing target. See comment about linkat()
-                 * usage in posix_link() in posix.c for details
-                 */
-                ret = linkat (AT_FDCWD, oldpath, AT_FDCWD, newpath, 0);
-#else
-                ret = link (oldpath, newpath);
-#endif
+                ret = sys_link (oldpath, newpath);
+
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "link %s -> %s failed (%s)",
-                                oldpath, newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE, "link %s -> %s"
+                                "failed ", oldpath, newpath);
                         return -1;
                 }
 
                 ret = lstat (newpath, &newbuf);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "lstat on %s failed (%s)",
-                                newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "lstat on %s failed", newpath);
                         return -1;
                 }
         }
 
         if (newbuf.st_ino != oldbuf->st_ino ||
             newbuf.st_dev != oldbuf->st_dev) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        P_MSG_HANDLE_CREATE,
                         "mismatching ino/dev between file %s (%lld/%lld) "
                         "and handle %s (%lld/%lld)",
                         oldpath, (long long) oldbuf->st_ino, (long long) oldbuf->st_dev,
@@ -752,41 +766,43 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
 
         ret = lstat (newpath, &newbuf);
         if (ret == -1 && errno != ENOENT) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "%s: %s", newpath, strerror (errno));
+                gf_msg (this->name, GF_LOG_WARNING, errno,
+                        P_MSG_HANDLE_CREATE, "%s", newpath);
                 return -1;
         }
 
         if (ret == -1 && errno == ENOENT) {
                 ret = posix_handle_mkdir_hashes (this, newpath);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "mkdir %s failed (%s)",
-                                newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "mkdir %s failed ", newpath);
                         return -1;
                 }
 
                 ret = symlink (oldpath, newpath);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "symlink %s -> %s failed (%s)",
-                                oldpath, newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "symlink %s -> %s failed",
+                                oldpath, newpath);
                         return -1;
                 }
 
                 ret = lstat (newpath, &newbuf);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "stat on %s failed (%s)",
-                                newpath, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_CREATE,
+                                "stat on %s failed ", newpath);
                         return -1;
                 }
         }
 
         ret = stat (real_path, &newbuf);
         if (ret) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "stat on %s failed (%s)", newpath, strerror (errno));
+                gf_msg (this->name, GF_LOG_WARNING, errno,
+                        P_MSG_HANDLE_CREATE,
+                        "stat on %s failed ", newpath);
                 return -1;
         }
 
@@ -795,7 +811,8 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
 
         if (newbuf.st_ino != oldbuf->st_ino ||
             newbuf.st_dev != oldbuf->st_dev) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        P_MSG_HANDLE_CREATE,
                         "mismatching ino/dev between file %s (%lld/%lld) "
                         "and handle %s (%lld/%lld)",
                         oldpath, (long long) oldbuf->st_ino, (long long) oldbuf->st_dev,
@@ -807,7 +824,7 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
 }
 
 
-static int
+int
 posix_handle_unset_gfid (xlator_t *this, uuid_t gfid)
 {
         char        *path = NULL;
@@ -820,16 +837,16 @@ posix_handle_unset_gfid (xlator_t *this, uuid_t gfid)
 
         if (ret == -1) {
                 if (errno != ENOENT) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "%s: %s", path, strerror (errno));
+                        gf_msg (this->name, GF_LOG_WARNING, errno,
+                                P_MSG_HANDLE_DELETE, "%s", path);
                 }
                 goto out;
         }
 
         ret = unlink (path);
         if (ret == -1) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "unlink %s failed (%s)", path, strerror (errno));
+                gf_msg (this->name, GF_LOG_WARNING, errno,
+                        P_MSG_HANDLE_DELETE, "unlink %s failed ", path);
         }
 
 out:
@@ -844,19 +861,25 @@ posix_handle_unset (xlator_t *this, uuid_t gfid, const char *basename)
         struct iatt  stat;
         char        *path = NULL;
 
-
         if (!basename) {
                 ret = posix_handle_unset_gfid (this, gfid);
                 return ret;
         }
 
         MAKE_HANDLE_PATH (path, this, gfid, basename);
+        if (!path) {
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        P_MSG_HANDLE_DELETE,
+                        "Failed to create handle path for %s (%s)",
+                        basename, uuid_utoa(gfid));
+                return -1;
+        }
 
         ret = posix_istat (this, gfid, basename, &stat);
 
         if (ret == -1) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "%s: %s", path, strerror (errno));
+                gf_msg (this->name, GF_LOG_WARNING, errno,
+                        P_MSG_HANDLE_DELETE, "%s", path);
                 return -1;
         }
 
@@ -875,18 +898,16 @@ posix_create_link_if_gfid_exists (xlator_t *this, uuid_t gfid,
         char *newpath = NULL;
 
         MAKE_HANDLE_PATH (newpath, this, gfid, NULL);
+        if (!newpath) {
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        P_MSG_HANDLE_CREATE,
+                        "Failed to create handle path (%s)", uuid_utoa(gfid));
+                return ret;
+        }
+
         ret = lstat (newpath, &stbuf);
         if (!ret) {
-#ifdef HAVE_LINKAT
-                /*
-                 * Use linkat if the target may be a symlink to a directory
-                 * or without an existing target. See comment about linkat()
-                 * usage in posix_link() in posix.c for details
-                 */
-                ret = linkat (AT_FDCWD, newpath, AT_FDCWD, real_path, 0);
-#else
-                ret = link (newpath, real_path);
-#endif
+                ret = sys_link (newpath, real_path);
         }
 
         return ret;
