@@ -15,11 +15,17 @@
 #include "config.h"
 #endif
 
+#include <limits.h>
 #include <sys/types.h>
 #include "xlator.h"
 #include "gf-dirent.h"
+#include "posix-messages.h"
+
+/* From Open Group Base Specifications Issue 6 */
+#ifndef _XOPEN_PATH_MAX
+#define _XOPEN_PATH_MAX 1024
+#endif
 
-#define HANDLE_PFX ".glusterfs"
 #define TRASH_DIR "landfill"
 
 #define UUID0_STR "00000000-0000-0000-0000-000000000000"
@@ -30,6 +36,8 @@
                                   UUID0_STR) + 1)
 
 #define LOC_HAS_ABSPATH(loc) (loc && (loc->path) && (loc->path[0] == '/'))
+#define LOC_IS_DIR(loc) (loc && (loc->inode) && \
+                (loc->inode->ia_type == IA_IFDIR))
 
 #define MAKE_PGFID_XATTR_KEY(var, prefix, pgfid) do {                   \
         var = alloca (strlen (prefix) + UUID_CANONICAL_FORM_LEN + 1);   \
@@ -43,21 +51,39 @@
                                 flags);                                 \
         if (op_ret == -1) {                                             \
                 op_errno = errno;                                       \
-                gf_log (this->name, GF_LOG_WARNING,                     \
-                        "setting xattr failed on %s: key = %s (%s)",    \
-                        path, key, strerror (op_errno));                \
+                gf_msg (this->name, GF_LOG_WARNING, errno, P_MSG_PGFID_OP, \
+                        "setting xattr failed on %s: key = %s ",        \
+                        path, key);                                     \
                 goto label;                                             \
         }                                                               \
         } while (0)
 
+#define SET_PGFID_XATTR_IF_ABSENT(path, key, value, flags, op_ret, this, label)\
+        do {                                                                   \
+                op_ret = sys_lgetxattr (path, key, &value, sizeof (value));    \
+                if (op_ret == -1) {                                            \
+                        op_errno = errno;                                      \
+                        if (op_errno == ENOATTR) {                             \
+                                value = 1;                                     \
+                                SET_PGFID_XATTR (path, key, value, flags,      \
+                                                 op_ret, this, label);         \
+                        } else {                                               \
+                                gf_msg (this->name, GF_LOG_WARNING, op_errno,  \
+                                       P_MSG_PGFID_OP, "getting xattr "    \
+                                       "failed on %s: key = %s ",              \
+                                       path, key);                             \
+                        }                                                      \
+                }                                                              \
+        } while (0)
 
 #define REMOVE_PGFID_XATTR(path, key, op_ret, this, label) do {               \
        op_ret = sys_lremovexattr (path, key);                           \
        if (op_ret == -1) {                                              \
                op_errno = errno;                                        \
-               gf_log (this->name, GF_LOG_WARNING, "removing xattr "    \
-                       "failed on %s: key = %s (%s)", path, key,        \
-                       strerror (op_errno));                            \
+               gf_msg (this->name, GF_LOG_WARNING, op_errno,                   \
+                       P_MSG_PGFID_OP,                                     \
+                       "removing xattr failed"                                 \
+                       "on %s: key = %s", path, key);                          \
                goto label;                                              \
        }                                                                \
        } while (0)
@@ -67,12 +93,12 @@
        op_ret = sys_lgetxattr (path, key, &value, sizeof (value));  \
        if (op_ret == -1) {                                              \
                op_errno = errno;                                        \
-               if (op_errno == ENOATTR) {                               \
+               if (op_errno == ENOATTR || op_errno == ENODATA) {        \
                        value = 1;                                       \
                } else {                                                 \
-                       gf_log (this->name, GF_LOG_WARNING,"getting xattr " \
-                               "failed on %s: key = %s (%s)", path, key, \
-                               strerror (op_errno));                    \
+                       gf_msg (this->name, GF_LOG_WARNING, errno,       \
+                               P_MSG_PGFID_OP, "getting xattr "      \
+                               "failed on %s: key = %s ", path, key);      \
                        goto label;                                      \
                }                                                        \
        } else {                                                         \
@@ -87,8 +113,9 @@
        op_ret = sys_lgetxattr (path, key, &value, sizeof (value));  \
        if (op_ret == -1) {                                              \
                op_errno = errno;                                        \
-               gf_log (this->name, GF_LOG_WARNING, "getting xattr failed on " \
-                       "%s: key = %s (%s)", path, key, strerror (op_errno)); \
+               gf_msg (this->name, GF_LOG_WARNING, errno,               \
+                      P_MSG_PGFID_OP, "getting xattr failed on " \
+                       "%s: key = %s ", path, key);                     \
                goto label;                                              \
        } else {                                                         \
                value = ntoh32 (value);                                  \
@@ -103,10 +130,18 @@
     } while (0)
 
 #define MAKE_REAL_PATH(var, this, path) do {                            \
-        var = alloca (strlen (path) + POSIX_BASE_PATH_LEN(this) + 2);   \
-        strcpy (var, POSIX_BASE_PATH(this));                            \
-        strcpy (&var[POSIX_BASE_PATH_LEN(this)], path);                 \
-        } while (0)
+        size_t path_len = strlen(path);                                 \
+        size_t var_len = path_len + POSIX_BASE_PATH_LEN(this) + 1;      \
+        if (POSIX_PATH_MAX(this) != -1 &&                               \
+            var_len >= POSIX_PATH_MAX(this)) {                          \
+                var = alloca (path_len + 1);                            \
+                strcpy (var, (path[0] == '/') ? path + 1 : path);       \
+        } else {                                                        \
+                var = alloca (var_len);                                 \
+                strcpy (var, POSIX_BASE_PATH(this));                    \
+                strcpy (&var[POSIX_BASE_PATH_LEN(this)], path);         \
+        }                                                               \
+    } while (0)
 
 #define MAKE_HANDLE_PATH(var, this, gfid, base) do {                    \
         int __len;                                                      \
@@ -115,6 +150,8 @@
                 break;                                                  \
         var = alloca (__len);                                           \
         __len = posix_handle_path (this, gfid, base, var, __len);       \
+        if (__len <= 0)                                                 \
+                var = NULL;                                             \
         } while (0)
 
 
@@ -148,12 +185,13 @@
 
 
 #define MAKE_INODE_HANDLE(rpath, this, loc, iatt_p) do {                \
-        if (uuid_is_null (loc->gfid)) {                                 \
-                gf_log (this->name, GF_LOG_ERROR,                       \
+        if (gf_uuid_is_null (loc->gfid)) {                              \
+                gf_msg (this->name, GF_LOG_ERROR, 0,                    \
+                        P_MSG_INODE_HANDLE_CREATE,                      \
                         "null gfid for path %s", (loc)->path);          \
                 break;                                                  \
         }                                                               \
-        if (LOC_HAS_ABSPATH (loc)) {                                    \
+        if (LOC_IS_DIR (loc) && LOC_HAS_ABSPATH (loc)) {                \
                 MAKE_REAL_PATH (rpath, this, (loc)->path);              \
                 op_ret = posix_pstat (this, (loc)->gfid, rpath, iatt_p); \
                 break;                                                  \
@@ -161,7 +199,14 @@
         errno = 0;                                                      \
         op_ret = posix_istat (this, loc->gfid, NULL, iatt_p);           \
         if (errno != ELOOP) {                                           \
-                MAKE_HANDLE_PATH (rpath, this, loc->gfid, NULL);        \
+                MAKE_HANDLE_PATH (rpath, this, (loc)->gfid, NULL);      \
+                if (!rpath) {                                           \
+                        op_ret = -1;                                    \
+                        gf_msg (this->name, GF_LOG_ERROR, errno,        \
+                                P_MSG_INODE_HANDLE_CREATE,            \
+                                "Failed to create inode handle "        \
+                                "for path %s", (loc)->path);            \
+                }                                                       \
                 break;                                                  \
         }                                                               \
         /* __ret == -1 && errno == ELOOP */                             \
@@ -171,8 +216,8 @@
 #define MAKE_ENTRY_HANDLE(entp, parp, this, loc, ent_p) do {            \
         char *__parp;                                                   \
                                                                         \
-        if (uuid_is_null (loc->pargfid) || !loc->name) {                \
-                gf_log (this->name, GF_LOG_ERROR,                       \
+        if (gf_uuid_is_null (loc->pargfid) || !loc->name) {             \
+                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_ENTRY_HANDLE_CREATE,\
                         "null pargfid/name for path %s", loc->path);    \
                 break;                                                  \
         }                                                               \
@@ -189,6 +234,12 @@
         if (errno != ELOOP) {                                           \
                 MAKE_HANDLE_PATH (parp, this, loc->pargfid, NULL);      \
                 MAKE_HANDLE_PATH (entp, this, loc->pargfid, loc->name); \
+                if (!parp || !entp) {                                   \
+                        gf_msg (this->name, GF_LOG_ERROR, errno,        \
+                                P_MSG_ENTRY_HANDLE_CREATE,      \
+                                "Failed to create entry handle "        \
+                                "for path %s", loc->path);              \
+                }                                                       \
                 break;                                                  \
         }                                                               \
         /* __ret == -1 && errno == ELOOP */                             \
