@@ -53,8 +53,12 @@
 #define VECTOR_SIZE 64 * 1024 /* vector size 64KB*/
 #define MAX_NO_VECT 1024
 
+#define ACL_BUFFER_MAX 4096 /* size of character buffer */
+
+#define LINKTO "trusted.glusterfs.dht.linkto"
+
 #define POSIX_GFID_HANDLE_SIZE(base_path_len) (base_path_len + SLEN("/") \
-                                               + SLEN(HANDLE_PFX) + SLEN("/") \
+                                               + SLEN(GF_HIDDEN_PATH) + SLEN("/") \
                                                + SLEN("00/")            \
                                                + SLEN("00/") + SLEN(UUID0_STR) + 1) /* '\0' */;
 
@@ -66,6 +70,7 @@ struct posix_fd {
 	int     fd;      /* fd returned by the kernel */
 	int32_t flags;   /* flags for open/creat      */
 	DIR *   dir;     /* handle returned by the kernel */
+	off_t   dir_eof; /* offset at dir EOF */
         int     odirect;
         struct list_head list; /* to add to the janitor list */
 };
@@ -74,6 +79,7 @@ struct posix_fd {
 struct posix_private {
 	char   *base_path;
 	int32_t base_path_length;
+	int32_t path_max;
 
         gf_lock_t lock;
 
@@ -158,6 +164,16 @@ struct posix_private {
         uint32_t        health_check_interval;
         pthread_t       health_check;
         gf_boolean_t    health_check_active;
+
+#ifdef GF_DARWIN_HOST_OS
+        enum {
+                XATTR_NONE = 0,
+                XATTR_STRIP,
+                XATTR_APPEND,
+                XATTR_BOTH,
+        } xattr_user_namespace;
+#endif
+
 };
 
 typedef struct {
@@ -167,7 +183,8 @@ typedef struct {
         struct iatt *stbuf;
         loc_t       *loc;
         inode_t     *inode; /* for all do_xattrop() key handling */
-        int          fd;
+        fd_t        *fd;
+        int          fdnum;
         int          flags;
         int32_t     op_errno;
 } posix_xattr_filler_t;
@@ -177,6 +194,8 @@ typedef struct {
 
 #define POSIX_BASE_PATH_LEN(this) (((struct posix_private *)this->private)->base_path_length)
 
+#define POSIX_PATH_MAX(this) (((struct posix_private *)this->private)->path_max)
+
 /* Helper functions */
 int posix_gfid_set (xlator_t *this, const char *path, loc_t *loc,
                     dict_t *xattr_req);
@@ -185,8 +204,8 @@ int posix_istat (xlator_t *this, uuid_t gfid, const char *basename,
                  struct iatt *iatt);
 int posix_pstat (xlator_t *this, uuid_t gfid, const char *real_path,
                  struct iatt *iatt);
-dict_t *posix_lookup_xattr_fill (xlator_t *this, const char *path,
-                                 loc_t *loc, dict_t *xattr, struct iatt *buf);
+dict_t *posix_xattr_fill (xlator_t *this, const char *path, loc_t *loc,
+                          fd_t *fd, int fdnum, dict_t *xattr, struct iatt *buf);
 int posix_handle_pair (xlator_t *this, const char *real_path, char *key,
                        data_t *value, int flags);
 int posix_fhandle_pair (xlator_t *this, int fd, char *key, data_t *value,
@@ -216,4 +235,20 @@ int
 posix_get_ancestry (xlator_t *this, inode_t *leaf_inode,
                     gf_dirent_t *head, char **path, int type, int32_t *op_errno,
                     dict_t *xdata);
+
+void
+posix_gfid_unset (xlator_t *this, dict_t *xdata);
+
+int
+posix_pacl_set (const char *path, const char *key, const char *acl_s);
+
+int
+posix_pacl_get (const char *path, const char *key, char **acl_s);
+
+int32_t
+posix_get_objectsignature (char *, dict_t *);
+
+int32_t
+posix_fdget_objectsignature (int, dict_t *);
+
 #endif /* _POSIX_H */
